'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var classNames = _interopDefault(require('classnames'));
var React = require('react');
var React__default = _interopDefault(React);
var draftJs = require('draft-js');
var Immutable = require('immutable');
var Immutable__default = _interopDefault(Immutable);
var DraftOffsetKey = _interopDefault(require('draft-js/lib/DraftOffsetKey'));
var generateRandomKey = _interopDefault(require('draft-js/lib/generateRandomKey'));
var Sabar = _interopDefault(require('sabar'));
var tapable = require('tapable');
var ContentBlockNode = _interopDefault(require('draft-js/lib/ContentBlockNode'));
var Prism = _interopDefault(require('prismjs'));
require('draft-js/dist/Draft.css');

function BlockStyleFnPlugin() {
  this.apply = function (getEditor) {
    var _getEditor = getEditor(),
        hooks = _getEditor.hooks;

    hooks.blockStyleFn.tap('BlockStyleFnPlugin', function () {
      var block = arguments.length <= 0 ? undefined : arguments[0];
      var cls = [];
      var blockData = block.getData();
      var depth = blockData.get('depth') || 0;
      var isDataWrapper = blockData.get('data-wrapper');
      var dataDirection = blockData.get('data-direction');

      if (block.getChildKeys().size && isDataWrapper) {
        cls.push("data-wrapper-" + dataDirection);
      }

      cls.push("block-level-" + depth);

      switch (block.getType()) {
        // 控制比如说，最后渲染出来的引用，它的class是
        case 'blockquote':
          cls.push('miuffy-blockquote');
          break;

        case 'unstyled':
          cls.push('miuffy-paragraph');
          break;

        case 'unordered-list-item':
          cls.push('miuffy-unordered-list-item');
          break;

        case 'atomic':
          var _getEditor2 = getEditor(),
              editorState = _getEditor2.editorState;

          var contentState = editorState.getCurrentContent();
          var entity = block.getEntityAt(0);
          if (!entity) return null;
          var entityState = contentState.getEntity(entity);
          var type = entityState.getType();
          var data = entityState.getData();

          if (type === 'IMAGE') {
            var alignment = data.alignment;

            switch (alignment) {
              case 'center':
                cls.push('figure-image-center');
                break;

              case 'right':
                cls.push('figure-image-right');
                break;

              case 'left':
                cls.push('figure-image-left');
                break;

              case 'leftFill':
                cls.push('figure-image-left-fill');
                break;

              case 'rightFill':
                cls.push('figure-image-right-fill');
                break;

              default:
                cls.push('figure-image');
            }
          }

          break;

      }

      return classNames(cls);
    });
  };
}

var resolveChangeType = function resolveChangeType(_ref) {
  var prevIsCollapsed = _ref.prevIsCollapsed,
      prevHasFocus = _ref.prevHasFocus,
      prevSelection = _ref.prevSelection,
      isCollapsed = _ref.isCollapsed,
      hasFocus = _ref.hasFocus,
      selection = _ref.selection;

  if (typeof prevIsCollapsed === 'undefined') {
    return 'init-change';
  }

  if (isCollapsed !== prevIsCollapsed) {
    return 'collapsed-change';
  }

  if (hasFocus !== prevHasFocus) {
    return 'focus-change';
  }

  if (isCollapsed) {
    var prevStartKey = prevSelection.getStartKey();
    var startKey = selection.getStartKey();
    if (prevStartKey === startKey) return 'selection-move-inner-block';
    return 'selection-move-outer-block';
  }

  var prevEndKey = prevSelection.getEndKey();
  var prevStartOffset = prevSelection.getStartOffset();
  var prevEndOffset = prevSelection.getEndOffset();
  var endKey = selection.getEndKey();
  var startOffset = selection.getStartOffset();
  var endOffset = selection.getEndOffset();

  if (prevEndKey === endKey && prevStartOffset === startOffset && prevEndOffset === endOffset) {
    return 'selection-range-content-change';
  }

  return 'selection-range-size-change';
};

function SelectionChangePlugin() {
  var prevIsCollapsed;
  var prevSelection;
  var prevHasFocus;

  this.apply = function (getEditor) {
    var _getEditor = getEditor(),
        hooks = _getEditor.hooks;

    hooks.syncSelectionChange.tap('SelectionChangePlugin', function (editorState) {
      var selection = editorState.getSelection();
      var isCollapsed = selection.isCollapsed();
      var hasFocus = selection.getHasFocus();
      var changeType = resolveChangeType({
        prevIsCollapsed: prevIsCollapsed,
        prevHasFocus: prevHasFocus,
        prevSelection: prevSelection,
        isCollapsed: isCollapsed,
        hasFocus: hasFocus,
        selection: selection
      });
      var payload = {
        changeType: changeType,
        oldValue: {
          prevIsCollapsed: prevIsCollapsed,
          prevHasFocus: prevHasFocus,
          prevSelection: prevSelection
        },
        newValue: {
          isCollapsed: isCollapsed,
          hasFocus: hasFocus,
          selection: selection
        }
      };
      prevIsCollapsed = isCollapsed;
      prevHasFocus = hasFocus;
      prevSelection = selection;

      switch (changeType) {
        case 'init-change':
          hooks.selectionInitChange.call(editorState, payload);
          break;

        case 'collapsed-change':
          hooks.selectionCollapsedChange.call(editorState, payload);
          break;

        case 'focus-change':
          hooks.selectionFocusChange.call(editorState, payload);
          break;

        case 'selection-move-inner-block':
          hooks.selectionMoveInnerBlock.call(editorState, payload);
          break;

        case 'selection-move-outer-block':
          hooks.selectionMoveOuterBlock.call(editorState, payload);
          break;

        case 'selection-range-content-change':
          hooks.selectionRangeContentChange.call(editorState, payload);
          hooks.selectionRangeChange.call(editorState, payload);
          break;

        case 'selection-range-size-change':
          hooks.selectionRangeSizeChange.call(editorState, payload);
          hooks.selectionRangeChange.call(editorState, payload);
          break;

      }
    });
  };
}

function CustomStyleMapPlugin() {
  this.apply = function (getEditor) {
    var _getEditor = getEditor(),
        hooks = _getEditor.hooks;

    hooks.createCustomStyleMap.tap('CustomStyleMapPlugin', function () {
      return {
        CODE: {
          backgroundColor: 'rgba(0, 0, 0, 0.05)',
          fontFamily: '"Inconsolata", "Menlo", "Consolas", monospace',
          fontSize: 16,
          padding: 2
        },
        // https://draftjs.org/docs/advanced-topics-inline-styles/#mapping-a-style-string-to-css
        'STRIKE-THROUGH': {
          textDecoration: 'line-through'
        }
      };
    });
  };
}

var CodeWrapper = function CodeWrapper(props) {
  var children = props.children;
  return React__default.createElement("div", {
    className: "code-mirror"
  }, children);
};

var NextDiv = function NextDiv(props) {
  var children = props.children;
  return React__default.createElement(React__default.Fragment, null, children);
};

var UL_WRAP = /*#__PURE__*/React__default.createElement("ul", {
  className: /*#__PURE__*/classNames('miuffy-ul')
});
var OL_WRAP = /*#__PURE__*/React__default.createElement("ol", {
  className: /*#__PURE__*/classNames('miuffy-ol')
});

function BlockRenderMapPlugin() {
  this.apply = function (getEditor) {
    var _getEditor = getEditor(),
        hooks = _getEditor.hooks;

    hooks.createBlockRenderMap.tap('BlockRenderMapPlugin', function () {
      var newBlockRenderMap = Immutable__default.Map({
        'header-two': {
          element: 'h2'
        },
        'unordered-list-item': {
          element: 'li',
          wrapper: UL_WRAP
        },
        'ordered-list-item': {
          element: 'li',
          wrapper: OL_WRAP
        },
        'code-block': {
          element: 'pre',
          wrapper: React__default.createElement(CodeWrapper, null)
        },
        unstyled: {
          element: 'div',
          wrapper: React__default.createElement(NextDiv, null)
        }
      });
      return draftJs.DefaultDraftBlockRenderMap.merge(newBlockRenderMap);
    });
  };
}

var readFile = function readFile(file) {
  var fileType = file.type,
      lastModified = file.lastModified,
      name = file.name,
      size = file.size,
      type = file.type;
  if (!fileType.startsWith('image/')) return Promise.resolve();
  var reader = new FileReader();
  return new Promise(function (resolve, reject) {
    reader.onload = function () {
      var src = reader.result;
      resolve({
        src: src,
        name: name,
        size: size,
        type: type,
        lastModified: lastModified
      });
    };

    reader.onerror = function () {
      reject();
    };

    reader.readAsDataURL(file);
  });
};

function HandleDroppedFilesPlugin() {
  this.apply = function (getEditor) {
    var _getEditor = getEditor(),
        hooks = _getEditor.hooks;

    hooks.handleDroppedFiles.tap('HandleDroppedFilesPlugin', function (editorState, _, files) {
      var jobs = [];
      files.forEach(function (file) {
        jobs.push(readFile(file));
      });
      Promise.all(jobs).then(function (result) {
        result.forEach(function (file) {
          hooks.addImage.call(editorState, file);
        });
      });
    });
  };
}

var setSelectionToBlock = (function (editorState, contentBlock) {
  var blockKey = contentBlock.getKey(); // TODO verify that always a key-0-0 exists

  var offsetKey = DraftOffsetKey.encode(blockKey, 0, 0);
  var node = document.querySelectorAll("[data-offset-key=\"" + offsetKey + "\"]")[0]; // set the native selection to the node so the caret is not in the text and
  // the selectionState matches the native selection

  var selection = window.getSelection();
  var range = document.createRange();
  range.setStart(node, 0);
  range.setEnd(node, 0);
  selection.removeAllRanges();
  selection.addRange(range);
  return draftJs.EditorState.forceSelection(editorState, new draftJs.SelectionState().merge({
    anchorKey: blockKey,
    anchorOffset: 0,
    focusKey: blockKey,
    focusOffset: 0,
    isBackward: false
  }));
});

var useFocus = function useFocus(_ref) {
  var nodeRef = _ref.nodeRef,
      props = _ref.props;
  var isMounted = React.useRef(false); // TODO: tapable could not be clear on unmount...

  React.useEffect(function () {
    isMounted.current = true;
    return function () {
      isMounted.current = false;
    };
  }, []);
  var blockProps = props.blockProps,
      block = props.block;
  var getEditor = blockProps.getEditor;

  var _getEditor = getEditor(),
      hooks = _getEditor.hooks;

  var _useState = React.useState(false),
      focused = _useState[0],
      setFocus = _useState[1];

  var focusedRef = React.useRef(false);
  var currentBlockKey = block.getKey();
  var timeoutHandler = React.useRef();
  var setState = React.useCallback(function (falsy) {
    setFocus(falsy);
    focusedRef.current = falsy;
  }, []);
  var delaySetState = React.useCallback(function (falsy) {
    if (timeoutHandler.current) clearTimeout(timeoutHandler.current);
    timeoutHandler.current = setTimeout(function () {
      return setState(falsy);
    }, 50);
  }, [setState]); // TODO: pending on fix clear up...

  React.useEffect(function () {
    hooks.selectionCollapsedChange.tap('Focus', function (_, payload) {
      if (!isMounted.current) return;
      var _payload$newValue = payload.newValue,
          isCollapsed = _payload$newValue.isCollapsed,
          selection = _payload$newValue.selection;
      var startKey = selection.getStartKey();

      if (isCollapsed && startKey === currentBlockKey && !focusedRef.current) {
        delaySetState(true);
        return;
      }

      if (!isCollapsed && focusedRef.current) {
        delaySetState(false);
      }
    });
    hooks.selectionMoveOuterBlock.tap('Focus', function (_, payload) {
      if (!isMounted.current) return;
      var selection = payload.newValue.selection;
      var startKey = selection.getStartKey();

      if (startKey === currentBlockKey && !focusedRef.current) {
        delaySetState(true);
        return;
      }

      if (startKey !== currentBlockKey && focusedRef.current) {
        delaySetState(false);
      }
    });
  }, [currentBlockKey, delaySetState, hooks.selectionCollapsedChange, hooks.selectionMoveOuterBlock]); // 对于`Focusable` component, 当被点击的时候，`EditorState`的selection应该指向当前的
  // block

  var handleClick = React.useCallback(function () {
    // 如果已经被选中，再次点击不再触发handler
    // pay attention, editorState should use the latest value
    var _getEditor2 = getEditor(),
        editorState = _getEditor2.editorState;

    var newEditorState = setSelectionToBlock(editorState, block);
    hooks.setState.call(newEditorState);
  }, [block, getEditor, hooks.setState]); // https://github.com/facebook/react/issues/14387
  // if ref.current is used in useEffect return function. it may have potential error..
  // Because ref.current may have change before return function is called.
  // In this condition. you'd better use a copy value...

  var nodeRefCopy = nodeRef.current;
  React.useEffect(function () {
    if (nodeRefCopy) {
      nodeRefCopy.addEventListener('click', handleClick);
      return function () {
        nodeRefCopy.removeEventListener('click', handleClick);
      };
    }

    return function () {};
  }, [handleClick, nodeRefCopy]); // update className after all

  React.useEffect(function () {
    if (nodeRef.current) {
      if (focused) {
        nodeRef.current.classList.remove('focused_atomic');
        nodeRef.current.classList.add('focused_atomic_active');
      } else {
        nodeRef.current.classList.remove('focused_atomic_active');
        nodeRef.current.classList.add('focused_atomic');
      }
    }
  }, [focused, nodeRef]);
};

// https://developer.mozilla.org/en-US/docs/Web/API/GlobalEventHandlers/onmousemove
/**
 *
 * @param {HTMLElement} nodeRef
 * @param {Object} props
 *
 * when calculate nextWidth value, `alignment` param will be a significant factor...
 *   1. alignment value is left.. Moving to left direction is allowed.
 *   2. alignment value is right.. Moving to right direction is allowed.
 *   3. alignment value is center.. Moving to both directions is allowed.
 */

var useResize = function useResize(_ref) {
  var nodeRef = _ref.nodeRef,
      props = _ref.props;
  var block = props.block,
      blockProps = props.blockProps;
  var resizeLayout = blockProps.resizeLayout,
      getEditor = blockProps.getEditor;
  var nextWidth = React.useRef();
  var coordinate = React.useRef();
  var layout = React.useRef();
  var leftBarRef = React.useRef();
  var rightBarRef = React.useRef();
  var mouseMoveTeardown = React.useRef(function () {});
  var resizeModeRef = React.useRef();
  var rightToStretchRef = React.useRef(); // fix: `mouseenter` and `mouseleave` will be triggered on moving under resize mode...

  var statusRef = React.useRef(null);
  React.useEffect(function () {
    // create a bar and append to nodeRef
    var leftNode = document.createElement('div');
    leftNode.classList.add('left-bar-wrapper');
    var leftBar = document.createElement('div');
    leftBar.classList.add('bar');
    leftNode.appendChild(leftBar);
    leftBarRef.current = leftNode;
    var rightNode = document.createElement('div');
    rightNode.classList.add('right-bar-wrapper');
    var rightBar = document.createElement('div');
    rightBar.classList.add('bar');
    rightNode.appendChild(rightBar);
    rightBarRef.current = rightNode;
    nodeRef.current.appendChild(leftBarRef.current);
    nodeRef.current.appendChild(rightBarRef.current);
    nodeRef.current.classList.add('resizable-component');
  }, [nodeRef]); // make resize bar visible when enter into container...

  var onMouseEnterContainer = React.useCallback(function () {
    if (statusRef.current === 'moving') return;
    leftBarRef.current.classList.add('bar-visible');
    rightBarRef.current.classList.add('bar-visible');
  }, []);
  var onMouseLeaveContainer = React.useCallback(function () {
    if (statusRef.current === 'moving') return;
    leftBarRef.current.classList.remove('bar-visible');
    rightBarRef.current.classList.remove('bar-visible');
  }, []);
  var nodeRefCopy = nodeRef.current;
  React.useEffect(function () {
    if (nodeRefCopy) {
      nodeRefCopy.addEventListener('mouseenter', onMouseEnterContainer);
      nodeRefCopy.addEventListener('mouseleave', onMouseLeaveContainer);
      return function () {
        nodeRefCopy.removeEventListener('mouseenter', onMouseEnterContainer);
        nodeRefCopy.removeEventListener('mouseleave', onMouseLeaveContainer);
      };
    }

    return function () {};
  }, [nodeRefCopy, onMouseEnterContainer, onMouseLeaveContainer]);
  var resolveAlignment = React.useCallback(function () {
    return 'center';
  }, []);
  var onMouseMoveHandler = React.useCallback(function (_ref2) {
    var clientX = _ref2.clientX;
    if (!resizeModeRef.current) return;
    statusRef.current = 'moving';

    var _ref3 = coordinate.current || {},
        oldClientX = _ref3.clientX;

    var deltaX;

    if (rightToStretchRef.current) {
      deltaX = clientX - oldClientX;
    } else {
      deltaX = oldClientX - clientX;
    }

    var alignment = resolveAlignment();

    if (layout.current) {
      // 只有当时居中的时候，width的变化需要是滑动距离的两倍
      if (alignment === 'center') {
        nextWidth.current = layout.current.width + deltaX * 2 + "px";
      } else {
        nextWidth.current = layout.current.width + deltaX + "px";
      }
    }

    if (nextWidth.current) nodeRef.current.style.width = nextWidth.current;
  }, [nodeRef, resolveAlignment]);
  var prepareAfterMouseDown = React.useCallback(function (event) {
    resizeModeRef.current = true;
    var clientX = event.clientX,
        clientY = event.clientY;
    var _nodeRef$current = nodeRef.current,
        offsetWidth = _nodeRef$current.offsetWidth,
        offsetHeight = _nodeRef$current.offsetHeight;
    coordinate.current = {
      clientX: clientX,
      clientY: clientY
    };
    layout.current = {
      width: offsetWidth,
      height: offsetHeight
    }; // when narrowing image, mouse may be on the outer of image...
    // So we'd better add event listener on document...

    document.addEventListener('mousemove', onMouseMoveHandler);
    if (mouseMoveTeardown.current) mouseMoveTeardown.current();

    mouseMoveTeardown.current = function () {
      document.removeEventListener('mousemove', onMouseMoveHandler);
      mouseMoveTeardown.current = null;
    };
  }, [nodeRef, onMouseMoveHandler]);
  var onMouseDownLeftHandler = React.useCallback(function (event) {
    prepareAfterMouseDown(event);
    rightToStretchRef.current = false;
  }, [prepareAfterMouseDown]);
  var onMouseDownRightHandler = React.useCallback(function (event) {
    prepareAfterMouseDown(event);
    rightToStretchRef.current = true;
  }, [prepareAfterMouseDown]);
  var onMouseUpHandler = React.useCallback(function () {
    resizeModeRef.current = false;
    statusRef.current = null;
    if (mouseMoveTeardown.current) mouseMoveTeardown.current();

    if (nextWidth.current) {
      var _getEditor = getEditor(),
          editorState = _getEditor.editorState,
          hooks = _getEditor.hooks;

      var contentState = editorState.getCurrentContent();
      var entityKey = block.getEntityAt(0);
      var newContent = contentState.mergeEntityData(entityKey, {
        resizeLayout: {
          width: nextWidth.current
        }
      });
      var nextState = draftJs.EditorState.push(editorState, newContent, 'change-block-data');
      hooks.setState.call(nextState);
    }
  }, [block, getEditor]);
  React.useEffect(function () {
    if (nodeRef.current) nodeRef.current.style.width = resizeLayout.width;
  }, [nodeRef, resizeLayout.width]); // cleanup move event

  React.useEffect(function () {
    return function () {
      if (mouseMoveTeardown.current) {
        mouseMoveTeardown.current();
        mouseMoveTeardown.current = null;
      }
    };
  }, []);
  React.useEffect(function () {
    leftBarRef.current.addEventListener('mousedown', onMouseDownLeftHandler);
    rightBarRef.current.addEventListener('mousedown', onMouseDownRightHandler); // leftBarRef.current.addEventListener("mouseup", onMouseUpHandler);
    // rightBarRef.current.addEventListener("mouseup", onMouseUpHandler);
    // should use document. leftBarRef.current `mouseup` may be not trigger, if
    // you resize the container, but mouse is on the outer.

    document.addEventListener('mouseup', onMouseUpHandler);
    document.addEventListener('mouseup', onMouseUpHandler);
    return function () {
      leftBarRef.current.removeEventListener('mousedown', onMouseDownLeftHandler);
      rightBarRef.current.removeEventListener('mousedown', onMouseDownRightHandler);
      document.removeEventListener('mouseup', onMouseUpHandler);
      document.removeEventListener('mouseup', onMouseUpHandler);
    };
  }, [onMouseDownLeftHandler, onMouseDownRightHandler, onMouseUpHandler]);
};

// https://github.com/facebook/draft-js/blob/master/src/component/base/DraftEditor.react.js#L332
// https://github.com/draft-js-plugins/draft-js-plugins/blob/master/draft-js-inline-toolbar-plugin/src/components/Toolbar/index.js#L67
var getRootNode = function getRootNode(editorRef) {
  if (!editorRef) return;
  var rootNode = editorRef.current.editor;

  while (rootNode && rootNode.className.indexOf('DraftEditor-root') === -1) {
    rootNode = rootNode.parentNode;
  }

  return rootNode;
};

var useAlignment = function useAlignment(_ref) {
  var nodeRef = _ref.nodeRef,
      props = _ref.props;
  var alignmentBarRef = React.useRef(document.querySelector('.image-toolbar'));
  var mouseEnterRemoverRef = React.useRef(function () {});
  var alignmentBarEventRemoverRef = React.useRef(function () {});
  var hideBarTimeoutHandler = React.useRef();
  var isToolbarVisibleRef = React.useRef(false);
  var blockProps = props.blockProps,
      block = props.block;
  var getEditor = blockProps.getEditor;

  var _getEditor = getEditor(),
      editorRef = _getEditor.editorRef,
      hooks = _getEditor.hooks;

  var attemptToClearTimeoutHandler = React.useCallback(function () {
    if (hideBarTimeoutHandler.current) clearTimeout(hideBarTimeoutHandler.current);
    hideBarTimeoutHandler.current = null;
  }, []); // clean up event handler on un-mount component.

  var teardown = React.useCallback(function () {
    attemptToClearTimeoutHandler();
    if (mouseEnterRemoverRef.current) mouseEnterRemoverRef.current();
    if (alignmentBarEventRemoverRef.current) alignmentBarEventRemoverRef.current();
  }, [attemptToClearTimeoutHandler]); // teardown on un-mount

  React.useEffect(function () {
    return function () {
      return teardown();
    };
  }, [teardown]);
  var onMouseEnterHandler = React.useRef(function (e) {
    attemptToClearTimeoutHandler();
    e.stopPropagation();
    var rootNode = getRootNode(editorRef);
    if (!rootNode) return;
    if (isToolbarVisibleRef.current) return;
    if (typeof showToolbar === 'function') showToolbar();
  });
  var onMouseLeaveHandler = React.useRef(function (e) {
    e.stopPropagation();
    if (!isToolbarVisibleRef.current) return;
    if (typeof hideToolbar === 'function') hideToolbar();
  });
  var showToolbar = React.useCallback(function () {
    var rootNode = getRootNode(editorRef);
    if (!nodeRef.current) return;
    var nodeRect = nodeRef.current.getBoundingClientRect();
    var rootOffsetTop = rootNode.offsetTop;
    var rootOffsetLeft = rootNode.offsetLeft;
    var width = nodeRect.width;
    var rootRect = rootNode.getBoundingClientRect();
    var top = rootOffsetTop + nodeRect.top - rootRect.top;
    var left = rootOffsetLeft + nodeRect.left - rootRect.left;
    alignmentBarRef.current.style.display = 'block';
    alignmentBarRef.current.style.visibility = 'visible';
    isToolbarVisibleRef.current = true;
    hooks.toggleImageToolbarVisible.call(true, block);
    var alignmentToolbarHeight = alignmentBarRef.current.offsetHeight;
    var alignmentToolbarWidth = alignmentBarRef.current.offsetWidth;
    var nextTop = top - alignmentToolbarHeight - 15; // 考虑到left的最小和最大值的边界

    var tmpLeft = left - alignmentToolbarWidth / 2 + width / 2;
    var nextLeft = Math.max(0, tmpLeft);
    alignmentBarRef.current.style.top = nextTop + "px";
    alignmentBarRef.current.style.left = nextLeft + "px";
    if (alignmentBarEventRemoverRef.current) alignmentBarEventRemoverRef.current();
    alignmentBarRef.current.addEventListener('mouseenter', onMouseEnterHandler.current);
    alignmentBarRef.current.addEventListener('mouseleave', onMouseLeaveHandler.current);

    alignmentBarEventRemoverRef.current = function () {
      alignmentBarRef.current.removeEventListener('mouseenter', onMouseEnterHandler.current);
      alignmentBarRef.current.removeEventListener('mouseleave', onMouseLeaveHandler.current);
    };
  }, [block, editorRef, hooks.toggleImageToolbarVisible, nodeRef]);
  var hideToolbar = React.useCallback(function () {
    hideBarTimeoutHandler.current = setTimeout(function () {
      alignmentBarRef.current.style.display = 'none';
      alignmentBarRef.current.style.visibility = 'invisible';
      isToolbarVisibleRef.current = false;
      hooks.toggleImageToolbarVisible.call(false, block);
      attemptToClearTimeoutHandler();
    }, 100);
  }, [attemptToClearTimeoutHandler, block, hooks.toggleImageToolbarVisible]); // To make nodeRef react to `mouseenter` and `mouseleave` event.

  React.useEffect(function () {
    if (nodeRef.current) {
      nodeRef.current.addEventListener('mouseenter', onMouseEnterHandler.current); // TODO: should fix...when resize component...mouseleave may not trigger...

      nodeRef.current.addEventListener('mouseleave', onMouseLeaveHandler.current);

      mouseEnterRemoverRef.current = function () {
        if (nodeRef.current) {
          nodeRef.current.removeEventListener('mouseenter', onMouseEnterHandler.current);
          nodeRef.current.removeEventListener('mouseleave', onMouseLeaveHandler.current);
        }

        mouseEnterRemoverRef.current = null;
      };

      return mouseEnterRemoverRef.current;
    }

    return function () {};
  }, [nodeRef, onMouseEnterHandler, onMouseLeaveHandler]);
};

var Image = function Image(props) {
  // createRef does not work. Because the `nodeRef` value is not updated
  // even if after useEffect triggered.
  var ref = React.useRef();

  var _useState = React.useState(false),
      setIsRefReady = _useState[1];

  React.useEffect(function () {
    setIsRefReady(true);
  }, []);
  var block = props.block,
      contentState = props.contentState;
  useFocus({
    nodeRef: ref,
    props: props
  });
  useResize({
    nodeRef: ref,
    props: props
  });
  useAlignment({
    nodeRef: ref,
    props: props
  });
  var meta = contentState.getEntity(block.getEntityAt(0)).getData();
  var src = meta.src;
  return React__default.createElement("div", {
    className: "image-wrapper",
    ref: ref
  }, React__default.createElement("img", {
    src: src,
    className: "image",
    alt: "wrapper"
  }));
};

var DecoratedImage = Image;

function AddImagePlugin() {
  this.apply = function (getEditor) {
    var _getEditor = getEditor(),
        hooks = _getEditor.hooks;

    hooks.addImage.tap('AddImagePlugin', function (editorState, file) {
      var _ref = file || {},
          src = _ref.src;

      if (!src) return;
      var entityType = 'IMAGE';
      var contentState = editorState.getCurrentContent();
      var contentStateWithEntity = contentState.createEntity(entityType, 'IMMUTABLE', {
        src: src,
        alignment: 'center',
        resizeLayout: {
          width: '900px'
        }
      });
      var entityKey = contentStateWithEntity.getLastCreatedEntityKey();
      var newEditorState = draftJs.AtomicBlockUtils.insertAtomicBlock(editorState, entityKey, ' ');
      hooks.setState.call(newEditorState);
    }); // 函数触发的时机，是否可以将alignment属性设置到props

    hooks.blockRendererFn.tap('AddImagePlugin', function (contentBlock, editorState) {
      if (contentBlock.getType() === 'atomic') {
        var contentState = editorState.getCurrentContent();
        var entity = contentBlock.getEntityAt(0);
        if (!entity) return null;
        var entityState = contentState.getEntity(entity);
        var type = entityState.getType();
        var data = entityState.getData();

        if (type === 'IMAGE') {
          var alignment = data.alignment,
              resizeLayout = data.resizeLayout;
          return {
            component: DecoratedImage,
            editable: false,
            props: {
              getEditor: getEditor,
              alignment: alignment,
              resizeLayout: resizeLayout
            }
          };
        }
      }

      return null;
    });
  };
}

// 解决的问题

function DefaultHandleKeyCommandPlugin() {
  this.apply = function (getEditor) {
    var _getEditor = getEditor(),
        hooks = _getEditor.hooks;

    hooks.handleKeyCommand.tap('HandleBackspaceOnStartOfBlockPlugin', function (command, editorState) {
      var newState = draftJs.RichUtils.handleKeyCommand(editorState, command);

      if (newState) {
        hooks.setState.call(newState);
        return 'handled';
      }

      return;
    });
  };
}

var getSelectionRectRelativeToOffsetParent = function getSelectionRectRelativeToOffsetParent(editorRef) {
  var rootNode = getRootNode(editorRef);
  var visibleSelectionRect = draftJs.getVisibleSelectionRect(window);
  if (!rootNode || !visibleSelectionRect) return;
  var rootRect = rootNode.getBoundingClientRect();
  var rootOffsetTop = rootNode.offsetTop;
  var rootOffsetLeft = rootNode.offsetLeft;
  var width = visibleSelectionRect.width,
      height = visibleSelectionRect.height;
  var top = rootOffsetTop + visibleSelectionRect.top - rootRect.top;
  var left = rootOffsetLeft + visibleSelectionRect.left - rootRect.left;
  return {
    top: top,
    left: left,
    right: left + width,
    bottom: top + height,
    width: width,
    height: height
  };
};

function InlineToolbar() {
  var isToolbarVisible = false;
  var lastMouseTargetAtInlineBar = false;
  var inlineToolbarNode;
  var editorStateAfterClickLinkButton; // 比如用户选择一个区域，弹出了inline-bar这个时候点击其它非editor中的其它区域；
  // 选中区域会变成灰色；这个时候再点击其中的一个文字，使editor focus，你会发现
  // `inline-bar`会闪现一下；这个是因为，当用户再次点击editor时，它首先触发的刚刚
  // 的`selection`然后再移动到指定的位置

  var timeoutHandler;

  this.apply = function (getEditor) {
    var _getEditor = getEditor(),
        hooks = _getEditor.hooks,
        editorRef = _getEditor.editorRef,
        inlineToolbarRef = _getEditor.inlineToolbarRef;

    var setupEditorState = function setupEditorState(editorState) {
      editorStateAfterClickLinkButton = editorState;
    };

    var clearUpEditorState = function clearUpEditorState() {
      editorStateAfterClickLinkButton = undefined;
    };

    var nodeHiddenHandler = function nodeHiddenHandler(node) {
      // 如果没有这个逻辑处理，当用户点击了`link` button以后，直接点击`Esc`的话，`inlineToolbar`
      // 消失，但是此时刚刚的selection是灰色的；
      if (editorStateAfterClickLinkButton) {
        var selection = editorStateAfterClickLinkButton.getSelection();
        var nextState = draftJs.EditorState.set(editorStateAfterClickLinkButton, {
          selection: selection.merge({
            hasFocus: false
          })
        });
        hooks.setState.call(nextState);
      }

      var n = node || inlineToolbarNode || document.querySelector('.inline-toolbar');
      n.style.display = 'none';
      n.style.visibility = 'invisible';
      isToolbarVisible = false;
      hooks.inlineBarChange.call(null, 'hidden');
    };

    var hiddenHandler = function hiddenHandler(editorState, inlineToolbarRef) {
      clearTimeout(timeoutHandler);
      if (!isToolbarVisible) return;
      inlineToolbarRef.current.style.display = 'none';
      inlineToolbarRef.current.style.visibility = 'invisible';
      isToolbarVisible = false;
      hooks.inlineBarChange.call(editorState, 'hidden');
    };

    var visibleHandler = function visibleHandler(editorState, // editorRef: RefObject<HTMLDivElement>,
    editorRef, inlineToolbarRef) {
      clearTimeout(timeoutHandler);
      timeoutHandler = setTimeout(function () {
        // if (isToolbarVisible) return 如果加了这个判断就不能够及时的进行位置更新了
        if (!inlineToolbarRef.current) return;
        var rect = getSelectionRectRelativeToOffsetParent(editorRef);
        if (!rect) return;
        inlineToolbarRef.current.style.display = 'block';
        inlineToolbarRef.current.style.visibility = 'visible';
        var top = rect.top,
            left = rect.left,
            _rect$width = rect.width,
            width = _rect$width === void 0 ? 0 : _rect$width;
        var inlineToolbarHeight = inlineToolbarRef.current.offsetHeight;
        var inlineToolbarWidth = inlineToolbarRef.current.offsetWidth;
        var nextTop = top - inlineToolbarHeight - 15;
        var tmpLeft = left - inlineToolbarWidth / 2 + width / 2;
        var nextLeft = Math.max(tmpLeft, 0);
        inlineToolbarRef.current.style.top = nextTop + "px";
        inlineToolbarRef.current.style.left = nextLeft + "px";

        if (tmpLeft !== nextLeft) {
          // 已经移动到了边界，这个时候不应该再显示三角符号
          inlineToolbarRef.current.style.overflow = 'hidden';
        } else {
          inlineToolbarRef.current.style.overflow = 'visible';
        }

        isToolbarVisible = true;
        hooks.inlineBarChange.call(editorState, 'visible');
      }, 100);
    }; // TODO clean up `removeEventListener`


    document.addEventListener('mousedown', function (e) {
      inlineToolbarNode = inlineToolbarNode || document.querySelector('.inline-toolbar');
      var appRoot = document.querySelector('#app');
      var node = e.target;
      if (!inlineToolbarNode) return;
      lastMouseTargetAtInlineBar = false;

      while (node !== document.body) {
        if (node === inlineToolbarNode) {
          lastMouseTargetAtInlineBar = true;
          break;
        }

        if (node === appRoot) {
          if (isToolbarVisible) {
            nodeHiddenHandler(inlineToolbarNode);
          }

          break;
        } // ts-hint: https://stackoverflow.com/questions/45454573/angular-property-parentnode-does-not-exist-on-type-eventtarget
        // ts-hint: https://stackoverflow.com/questions/28900077/why-is-event-target-not-element-in-typescript


        node = node.parentNode;
      }

      if (node === document.body && isToolbarVisible) {
        nodeHiddenHandler(inlineToolbarNode);
      }
    });
    hooks.hideInlineToolbar.tap('inlineToolbarPlugin', function () {
      if (isToolbarVisible) nodeHiddenHandler(inlineToolbarNode);
    });
    hooks.selectionCollapsedChange.tap('InlineToolbar', function (editorState, selectionChanged) {
      var isCollapsed = selectionChanged.newValue.isCollapsed;
      clearTimeout(timeoutHandler);

      if (isCollapsed && isToolbarVisible) {
        hiddenHandler(editorState, inlineToolbarRef);
      }

      if (!isCollapsed) {
        visibleHandler(editorState, editorRef, inlineToolbarRef);
      }
    });
    hooks.selectionRangeChange.tap('InlineToolbar', function (editorState) {
      visibleHandler(editorState, editorRef, inlineToolbarRef);
    });
    hooks.selectionFocusChange.tap('InlineToolbar', function (editorState, selectionChanged) {
      var _selectionChanged$new = selectionChanged.newValue,
          hasFocus = _selectionChanged$new.hasFocus,
          isCollapsed = _selectionChanged$new.isCollapsed; // !hasFocus && isCollapsed当用户选中一个区域以后，这个时候有inlineToolbar, 当点击页面中其它
      // 区域（只是focus变化，selectionRange没变），这个时候应该要消掉toolbar；但是如果说，用户点击
      // 的是`inline-toolbar`区域中的一部分的话，它不应该消失掉；这里面通过`lastMouseTargetAtInlineBar`
      // 进行实现

      if (!hasFocus && !lastMouseTargetAtInlineBar) {
        hiddenHandler(editorState, inlineToolbarRef);
      }

      if (hasFocus && !isCollapsed) {
        visibleHandler(editorState, editorRef, inlineToolbarRef);
      }
    });
    hooks.afterClickLinkButton.tap('InlineToolbarPlugin', function (editorState) {
      setupEditorState(editorState);
    });
    hooks.cleanUpLinkClickState.tap('InlineToolbarPlugin', function () {
      clearUpEditorState();
    });
  };
}

var LinkSpan = (function (props) {
  return React__default.createElement("span", {
    className: "link_span"
  }, props.children);
});

function LinkSpanDecoratorPlugin() {
  this.apply = function (getEditor) {
    var _getEditor = getEditor(),
        hooks = _getEditor.hooks;

    hooks.updateDecorator.tap('LinkSpanDecoratorPlugin', function (pairs) {
      if (pairs === void 0) {
        pairs = [];
      }

      var strategy = function strategy(contentBlock, cb, contentState) {
        if (!contentState) return;
        contentBlock.findEntityRanges(function (character) {
          var entityKey = character.getEntity();
          if (!entityKey) return false;
          var entityType = contentState.getEntity(entityKey).getType();
          return entityType === 'LINK_SPAN';
        }, cb);
      };

      return pairs.concat({
        strategy: strategy,
        component: LinkSpan
      });
    });
  };
}

var Link = function Link(props) {
  var contentState = props.contentState,
      entityKey = props.entityKey,
      children = props.children;
  var entity = contentState.getEntity(entityKey);
  var data = entity.getData();
  var url = data.url;
  return React__default.createElement("a", {
    className: "decorator_link",
    href: url,
    target: "_blank",
    rel: "noreferrer"
  }, children);
};

function LinkDecorator() {
  this.apply = function (getEditor) {
    var _getEditor = getEditor(),
        hooks = _getEditor.hooks;

    hooks.updateDecorator.tap('LinkDecorator', function (pairs) {
      if (pairs === void 0) {
        pairs = [];
      }

      var strategy = function strategy(contentBlock, cb, contentState) {
        if (!contentState) return;
        contentBlock.findEntityRanges(function (character) {
          var entityKey = character.getEntity();
          if (!entityKey) return false;
          var entityType = contentState.getEntity(entityKey).getType();
          return entityType === 'LINK';
        }, cb);
      };

      return pairs.concat({
        strategy: strategy,
        component: Link
      });
    });
  };
}

// ts-hint: https://github.com/github/mini-throttle/blob/master/index.ts
function throttle(fn, timeout) {
  var isAvailable = true;
  return function () {
    for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    if (!isAvailable) return;
    isAvailable = false;
    setTimeout(function () {
      fn.apply(void 0, args);
      isAvailable = true;
    }, timeout);
  };
}

// @ts-ignore
function generateOffsetKey(blockKey) {
  return DraftOffsetKey.encode(blockKey, 0, 0);
} // offsetKey: erm6t-0-0

function extractBlockKeyFromOffsetKey(offsetKey) {
  var parts = offsetKey.split('-');
  return parts[0];
}

function getNodeByBlockKey(blockKey) {
  var offsetKey = generateOffsetKey(blockKey);
  return document.querySelector("[data-block=\"true\"][data-offset-key=\"" + offsetKey + "\"]");
}
var getNodeByOffsetKey = function getNodeByOffsetKey(offsetKey) {
  return document.querySelector("[data-block=\"true\"][data-offset-key=\"" + offsetKey + "\"]");
}; // https://stackoverflow.com/questions/29937768/document-queryselector-multiple-data-attributes-in-one-element
var getSelectableNodeByOffsetKey = function getSelectableNodeByOffsetKey(offsetKey) {
  return document.querySelector("[data-block=\"true\"] [data-id=\"" + offsetKey + "\"] div.selectable");
};

// https://github.com/alexreardon/css-box-model/blob/master/src/index.js
function getBoundingRectWithSafeArea(editorState, safeArea) {
  if (safeArea === void 0) {
    safeArea = 100;
  }

  if (!editorState) return;
  var currentState = editorState.getCurrentContent();
  var blockMap = currentState.getBlockMap(); // mainly, used to display sidebar

  var shiftLeft = []; // mainly, used to display drop direction bar..

  var shiftRight = [];
  blockMap.forEach(function (block) {
    var blockKey = block.getKey();
    var offsetKey = generateOffsetKey(blockKey);
    var node = getNodeByBlockKey(blockKey);
    var childrenSize = block.getChildKeys().size; // node with children should be omitted.

    if (!node || childrenSize) return;

    var _node$getBoundingClie = node.getBoundingClientRect(),
        top = _node$getBoundingClie.top,
        right = _node$getBoundingClie.right,
        bottom = _node$getBoundingClie.bottom,
        left = _node$getBoundingClie.left; // right and left both should minus `safeArea`


    shiftLeft.push({
      blockKey: blockKey,
      offsetKey: offsetKey,
      rect: {
        top: top,
        right: right - safeArea,
        bottom: bottom,
        left: left - safeArea
      }
    });
    shiftRight.push({
      blockKey: blockKey,
      offsetKey: offsetKey,
      rect: {
        top: top,
        right: right + safeArea,
        bottom: bottom,
        left: left + safeArea
      }
    });
  });
  return {
    shiftLeft: shiftLeft.filter(function (v) {
      return v;
    }),
    shiftRight: shiftRight.filter(function (v) {
      return v;
    })
  };
}

// https://stackoverflow.com/questions/18295825/determine-if-point-is-within-bounding-box
function findBlockContainsPoint(coordinateMap, point) {
  var len = coordinateMap.length;

  for (var i = 0; i < len; i++) {
    var data = coordinateMap[i];
    if (!data) continue;
    var rect = data.rect;
    var top = rect.top,
        right = rect.right,
        bottom = rect.bottom,
        left = rect.left;
    var x = point.x,
        y = point.y;
    var falsy = left < x && x < right && y > top && y < bottom;
    if (falsy) return data;
  }

  return null;
}

var svgNS = 'http://www.w3.org/2000/svg';

var createAddOn = function createAddOn(nodeKey) {
  var wrapper = document.createElement('div');
  wrapper.classList.add('sidebar-addon');
  wrapper.setAttribute('data-id', nodeKey); // add plus icon

  var plusWrapper = document.createElement('div');
  plusWrapper.classList.add('plus');
  var plus = document.createElementNS(svgNS, 'svg');
  plus.setAttributeNS(null, 'viewBox', '0 0 18 18');
  plus.setAttributeNS(null, 'width', '14');
  plus.setAttributeNS(null, 'height', '14');
  plus.setAttributeNS(null, 'fill', 'rgba(55, 53, 47, 0.3)');
  var polygon = document.createElementNS(svgNS, 'polygon');
  polygon.setAttributeNS(null, 'points', '17,8 10,8 10,1 8,1 8,8 1,8 1,10 8,10 8,17 10,17 10,10 17,10 ');
  plus.appendChild(polygon);
  plusWrapper.appendChild(plus);
  var selectableWrapper = document.createElement('div');
  selectableWrapper.classList.add('selectable');
  var selectable = document.createElementNS(svgNS, 'svg');
  selectable.setAttributeNS(null, 'viewBox', '0 0 10 10');
  selectable.setAttributeNS(null, 'width', '14');
  selectable.setAttributeNS(null, 'height', '14');
  selectable.setAttributeNS(null, 'fill', 'rgba(55, 53, 47, 0.3)');
  var path = document.createElementNS(svgNS, 'path');
  path.setAttributeNS(null, 'd', 'M3,2 C2.44771525,2 2,1.55228475 2,1 C2,0.44771525 2.44771525,0 3,0 C3.55228475,0 4,0.44771525 4,1 C4,1.55228475 3.55228475,2 3,2 Z M3,6 C2.44771525,6 2,5.55228475 2,5 C2,4.44771525 2.44771525,4 3,4 C3.55228475,4 4,4.44771525 4,5 C4,5.55228475 3.55228475,6 3,6 Z M3,10 C2.44771525,10 2,9.55228475 2,9 C2,8.44771525 2.44771525,8 3,8 C3.55228475,8 4,8.44771525 4,9 C4,9.55228475 3.55228475,10 3,10 Z M7,2 C6.44771525,2 6,1.55228475 6,1 C6,0.44771525 6.44771525,0 7,0 C7.55228475,0 8,0.44771525 8,1 C8,1.55228475 7.55228475,2 7,2 Z M7,6 C6.44771525,6 6,5.55228475 6,5 C6,4.44771525 6.44771525,4 7,4 C7.55228475,4 8,4.44771525 8,5 C8,5.55228475 7.55228475,6 7,6 Z M7,10 C6.44771525,10 6,9.55228475 6,9 C6,8.44771525 6.44771525,8 7,8 C7.55228475,8 8,8.44771525 8,9 C8,9.55228475 7.55228475,10 7,10 Z');
  selectable.appendChild(path);
  selectableWrapper.appendChild(selectable);
  wrapper.appendChild(plusWrapper);
  wrapper.appendChild(selectableWrapper);
  return wrapper;
};

function _extends() {
  _extends = Object.assign || function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];

      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }

    return target;
  };

  return _extends.apply(this, arguments);
}

function _inheritsLoose(subClass, superClass) {
  subClass.prototype = Object.create(superClass.prototype);
  subClass.prototype.constructor = subClass;
  subClass.__proto__ = superClass;
}

function _objectWithoutPropertiesLoose(source, excluded) {
  if (source == null) return {};
  var target = {};
  var sourceKeys = Object.keys(source);
  var key, i;

  for (i = 0; i < sourceKeys.length; i++) {
    key = sourceKeys[i];
    if (excluded.indexOf(key) >= 0) continue;
    target[key] = source[key];
  }

  return target;
}

function _unsupportedIterableToArray(o, minLen) {
  if (!o) return;
  if (typeof o === "string") return _arrayLikeToArray(o, minLen);
  var n = Object.prototype.toString.call(o).slice(8, -1);
  if (n === "Object" && o.constructor) n = o.constructor.name;
  if (n === "Map" || n === "Set") return Array.from(o);
  if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen);
}

function _arrayLikeToArray(arr, len) {
  if (len == null || len > arr.length) len = arr.length;

  for (var i = 0, arr2 = new Array(len); i < len; i++) arr2[i] = arr[i];

  return arr2;
}

function _createForOfIteratorHelperLoose(o, allowArrayLike) {
  var it;

  if (typeof Symbol === "undefined" || o[Symbol.iterator] == null) {
    if (Array.isArray(o) || (it = _unsupportedIterableToArray(o)) || allowArrayLike && o && typeof o.length === "number") {
      if (it) o = it;
      var i = 0;
      return function () {
        if (i >= o.length) return {
          done: true
        };
        return {
          done: false,
          value: o[i++]
        };
      };
    }

    throw new TypeError("Invalid attempt to iterate non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.");
  }

  it = o[Symbol.iterator]();
  return it.next.bind(it);
}

// https://github.com/atlassian/react-beautiful-dnd/blob/master/src/view/event-bindings/bind-events.js
function getOptions(shared, fromBinding) {
  return _extends({}, shared, fromBinding);
}

function bindEvents(el, bindings, sharedOptions) {
  var empty = [];
  var nextBindings = empty.concat(bindings);
  var unBindings = nextBindings.map(function (binding) {
    var options = getOptions(sharedOptions, binding.options); // ts-hint: https://github.com/microsoft/TypeScript/issues/28357#issuecomment-436484705

    el.addEventListener(binding.eventName, binding.fn, options);
    return function unbind() {
      el.removeEventListener(binding.eventName, binding.fn, options);
    };
  }); // Return a function to unbind events

  return function unbindAll() {
    unBindings.forEach(function (unbind) {
      return unbind();
    });
  };
} // once event triggered. it will be teardown first...

function bindEventsOnce(el, bindings, sharedOptions) {
  var empty = [];
  var nextBindings = empty.concat(bindings);
  var unBindings = nextBindings.map(function (binding) {
    var options = getOptions(sharedOptions, binding.options);

    var unbind = function unbind() {};

    var wrappedFn = function wrappedFn(e) {
      binding.fn.call(null, e);
      unbind();
    };

    el.addEventListener(binding.eventName, wrappedFn, options);

    unbind = function unbind() {
      return el.removeEventListener(binding.eventName, wrappedFn, options);
    };

    return unbind;
  }); // Return a function to unbind events

  return function unbindAll() {
    unBindings.forEach(function (unbind) {
      return unbind();
    });
  };
}

/**
 * 1. mouse enter into block (including safe area and block itself), side bar should display on
 *    current block. when mouse position do not belong to any block, side bar will be put by the
 *    side of block where selection exist.
 * 2. If side bar display, only if you begin to typing characters it will disappear.
 *    1. select text in same block will not trigger its disappear
 *    2. select text range including other block, it will make sidebar appear on other block's heading.
 *    3. click a position will not trigger its disappear
 * 3. when typing or enter into new line, sidebar should not display...
 * 4. when hover on a image, how to handle imageBar and sidebar
 *
 * The reason why sidebar should be created every times...
 * 1. An attempt to create a top level Sidebar component, but when fulfill drag functionality,
 *    It has big trouble...How could i set `draggable` attribute and drag the related block, it means
 *    sidebar trigger setting block as `draggable` and trigger related block to move. So it'd better make
 *    sidebar belong to block.
 */

function SidebarPlugin() {
  var current = null;

  this.apply = function (getEditor) {
    var removeNode = function removeNode() {
      try {
        if (!current) return;
        var _current = current,
            node = _current.node,
            child = _current.child,
            teardown = _current.teardown;
        if (typeof teardown === 'function') teardown();
        if (node.contains(child)) node.removeChild(child);
        current = null;
      } catch (err) {
        console.log('[SideBarPlugin]: removeNode error ', err);
      }
    };

    var isDragging = false;

    var mouseMoveHandler = function mouseMoveHandler(e) {
      try {
        var _getEditor = getEditor(),
            editorState = _getEditor.editorState;

        var coordinateMap = getBoundingRectWithSafeArea(editorState);

        var _ref = coordinateMap || {},
            shiftLeft = _ref.shiftLeft;

        if (!shiftLeft) return; // get mouse event point related to viewport
        // https://stackoverflow.com/questions/14717617/how-to-get-the-mouse-position-relative-to-the-window-viewport-in-javascript

        var x = e.clientX;
        var y = e.clientY;
        var nodeInfo = findBlockContainsPoint(shiftLeft, {
          x: x,
          y: y
        });
        if (!nodeInfo) return;
        var offsetKey = nodeInfo.offsetKey;
        var node = getNodeByOffsetKey(offsetKey);
        if (current && current.node === node) return; // Compare the current node with existing node. if it is not the same, it means
        // cursor moves to other block. So we should removeNode...

        if (current && current.node !== node) removeNode();
        var child = createAddOn(offsetKey);
        if (node) node.appendChild(child);
        var selectableNode = getSelectableNodeByOffsetKey(offsetKey);

        var enterHandler = function enterHandler() {
          var node = getNodeByOffsetKey(offsetKey);

          var _getEditor2 = getEditor(),
              hooks = _getEditor2.hooks;

          var blockKey = extractBlockKeyFromOffsetKey(offsetKey);
          hooks.prepareDragStart.call(blockKey);
          bindEventsOnce(node, {
            eventName: 'mousedown',
            fn: function fn() {
              // To make teardown or not...
              if (!isDragging) isDragging = true;
              bindEventsOnce(node, {
                eventName: 'mouseup',
                fn: function fn() {
                  hooks.teardownDragDrop.call();
                }
              });
            }
          });
        };

        var leaveHandler = function leaveHandler(e) {
          e.preventDefault();

          var _getEditor3 = getEditor(),
              hooks = _getEditor3.hooks; // which means `mousedown` is triggered...


          if (isDragging) return;
          hooks.teardownDragDrop.call();
        };

        var teardown = bindEventsOnce(selectableNode, {
          eventName: 'mouseenter',
          fn: function fn() {
            enterHandler();
            bindEventsOnce(selectableNode, {
              eventName: 'mouseleave',
              fn: leaveHandler
            });
          }
        }); // https://stackoverflow.com/questions/24148403/trigger-css-transition-on-appended-element

        requestAnimationFrame(function () {
          return child.classList.add('sidebar-addon-visible');
        });
        current = {
          node: node,
          child: child,
          offsetKey: offsetKey,
          teardown: teardown
        };
      } catch (err) {
        console.log('err in SideBar plugin ', err);
      }
    }; // when user start typing, sidebar should disappear...
    // Note: `removeChild` will cause `block` delete issue in a situation described below.
    //   1. hover cursor on a block
    //   2. type in chinese character on other block
    //   3. the block content under cursor will be cleared...
    // TODO: In composition mode. blocks will be re-rendered, which may cause removed DOM's
    //       event handler is not cleared...


    var keydownHandler = function keydownHandler() {
      var _getEditor4 = getEditor(),
          editorState = _getEditor4.editorState;

      if (editorState.isInCompositionMode()) return;
      removeNode();
    };

    var throttledMoveHandler = throttle(mouseMoveHandler, 50);
    var throttledKeydownHandler = throttle(keydownHandler, 50); // global document mousemove to determine which dom cursor is in...
    // It has a drawback, rect should be calculated on every move...

    bindEvents(document, [{
      eventName: 'mousemove',
      fn: throttledMoveHandler
    }, {
      eventName: 'keydown',
      fn: throttledKeydownHandler
    }]);
  };
}

function findLastBlockWithNullParent(contentState) {
  var blockMap = contentState.getBlockMap();
  return blockMap.reverse().skipUntil(function (block) {
    return !block.getParentKey();
  }).take(1);
}

var List = Immutable__default.List,
    Map = Immutable__default.Map;
var BlockUtil = {
  /**
   * Normally, it will insert a new line block with the same level...
   */
  insertNewLine: function insertNewLine() {},

  /**
   * https://github.com/facebook/draft-js/blob/master/src/model/transaction/splitBlockInContentState.js#L90
   * Append a new empty line after all the blocks.
   * 1. Remember current selection
   * 2. Add a new empty block
   * 3. reset selection
   */
  insertNewLineAfterAll: function insertNewLineAfterAll(currentState) {
    var lastBlock = currentState.getLastBlock();
    var lastBlockText = lastBlock.getText(); // insert an empty line only if the last block is not a empty line.

    if (!lastBlockText) return currentState;
    var keyBelow = generateRandomKey();
    var blockMap = currentState.getBlockMap();
    var blockBelow = lastBlock.merge({
      parent: null,
      children: List([]),
      key: keyBelow,
      type: 'unstyled',
      text: '',
      characterList: List(),
      depth: 0,
      data: Map()
    });
    var newBlockMap = blockMap.toSeq().concat([[keyBelow, blockBelow]]).toOrderedMap();
    var lastBlockWithNullParent = findLastBlockWithNullParent(currentState);

    if (lastBlockWithNullParent) {
      var blockToUpdate = lastBlockWithNullParent.last();
      var blockToUpdateKey = blockToUpdate.getKey();
      blockBelow = blockBelow.merge({
        prevSibling: blockToUpdateKey
      });
      blockToUpdate = blockToUpdate.merge({
        nextSibling: keyBelow
      });
      newBlockMap = newBlockMap.set(keyBelow, blockBelow);
      newBlockMap = newBlockMap.set(blockToUpdateKey, blockToUpdate);
    }

    return currentState.merge({
      blockMap: newBlockMap
    });
  },
  removeEmptyLineAfterAll: function removeEmptyLineAfterAll() {},
  insertDelimiter: function insertDelimiter() {},
  blocksBefore: function blocksBefore(blockMap, block) {
    return blockMap.toSeq().takeUntil(function (v) {
      return v === block;
    });
  },
  blocksAfter: function blocksAfter(blockMap, block) {
    return blockMap.toSeq().skipUntil(function (v) {
      return v === block;
    }).skip(1);
  },
  // moveBlockInContentState
  // removeRangeFromContentState.js
  transformBlock: function transformBlock(key, blockMap, func) {
    if (!key) {
      return;
    }

    var block = blockMap.get(key);

    if (!block) {
      return;
    }

    return func(block);
  },
  getChildrenAfterRemoveBlock: function getChildrenAfterRemoveBlock(parentBlock, block) {
    var parentChildrenList = parentBlock.getChildKeys();
    var blockKey = block.getKey();
    return parentChildrenList["delete"](parentChildrenList.indexOf(blockKey));
  },
  getChildrenSize: function getChildrenSize(parentBlock) {
    if (!parentBlock) return;
    var parentChildrenList = parentBlock.getChildKeys();
    return parentChildrenList.size;
  },
  getChildrenBlocks: function getChildrenBlocks(blockMap, parentBlock) {
    var parentKey = parentBlock.getKey();
    return blockMap.toSeq().skipUntil(function (v) {
      return v === parentBlock;
    }).skip(1).takeUntil(function (v) {
      var parent = v.getParentKey();

      while (parent) {
        if (parent === parentKey) return false;
        var next = blockMap.get(parent);
        if (next) parent = next.getParentKey();
      }

      return true;
    }).toOrderedMap();
  }
};

var BlockMutationUtil = {
  // moveBlockInContentState
  // removeRangeFromContentState.js
  transformBlock: function transformBlock(key, blockMap, func) {
    if (!key) {
      return;
    }

    var block = blockMap.get(key);

    if (!block) {
      return;
    }

    blockMap.set(key, func(block));
  },
  deleteFromChildrenList: function deleteFromChildrenList(block, originalBlockKey) {
    var parentChildrenList = block.getChildKeys();
    return block.merge({
      children: parentChildrenList["delete"](parentChildrenList.indexOf(originalBlockKey))
    });
  }
};

/**
 *
 * @param {*} editorState
 * @param {String} sourceBlockKey
 * @param {String} targetBlockKey
 * @param {String} position : one of values ['top', 'right', 'left']
 */

function removeBlock(blockMap, block, removeParentIfHasNoChild) {
  var blockToRemove;
  var blockKey;

  if (!block) {
    return blockMap;
  }

  if (typeof block === 'string') {
    blockToRemove = blockMap.get(block);
    blockKey = block;
  } else {
    blockToRemove = block;
    blockKey = block.getKey();
  }

  if (!blockToRemove) return blockMap;
  var blocksBefore = BlockUtil.blocksBefore(blockMap, blockToRemove);
  var blocksAfter = BlockUtil.blocksAfter(blockMap, blockToRemove);
  var newBlockMap = blocksBefore.concat(blocksAfter).toOrderedMap();
  var parentKey = blockToRemove.getParentKey();
  newBlockMap = newBlockMap.withMutations(function (blocks) {
    BlockMutationUtil.transformBlock(parentKey, blocks, function (block) {
      return BlockMutationUtil.deleteFromChildrenList(block, blockKey);
    });
  });
  var newParentBlock = newBlockMap.get(parentKey);
  var childrenSize = BlockUtil.getChildrenSize(newParentBlock);

  if (removeParentIfHasNoChild && !childrenSize) {
    return removeBlock(newBlockMap, newParentBlock, removeParentIfHasNoChild);
  }

  return newBlockMap.withMutations(function (blocks) {
    BlockMutationUtil.transformBlock(blockToRemove.getPrevSiblingKey(), blocks, function (block) {
      return block.merge({
        nextSibling: blockToRemove.getNextSiblingKey()
      });
    });
    BlockMutationUtil.transformBlock(blockToRemove.getNextSiblingKey(), blocks, function (block) {
      return block.merge({
        prevSibling: blockToRemove.getPrevSiblingKey()
      });
    });
  });
}

var findRootNode = function findRootNode(blockMap, blockKey) {
  var block = blockMap.get(blockKey);
  if (!block) return null;
  if (!block.getParentKey()) return block;
  return findRootNode(blockMap, block.getParentKey());
};

var flattenBlocks = function flattenBlocks(blockMap, startKey, endKey) {
  var newBlockMap = blockMap;
  var prefixBlocks = Immutable.OrderedMap();
  var suffixBlocks = Immutable.OrderedMap();
  var blocksBefore = newBlockMap.toSeq().takeUntil(function (_, key) {
    return key === startKey;
  }).toOrderedMap();
  var blocksBeforeLast = blocksBefore.toSeq().last();

  if (blocksBeforeLast) {
    blocksBeforeLast = findRootNode(blockMap, blocksBeforeLast.getKey());
    prefixBlocks = prefixBlocks.set(blocksBeforeLast.getKey(), blocksBeforeLast);
  }

  var blocksAfter = newBlockMap.toSeq().skipUntil(function (_, key) {
    return key === endKey;
  }).skip(1).takeWhile(function () {
    return true;
  }).toOrderedMap(); // take all after key

  var blocksAfterFirst = blocksAfter.toSeq().first();

  if (blocksAfterFirst) {
    suffixBlocks = suffixBlocks.set(blocksAfterFirst.getKey(), blocksAfterFirst);
  } // TODO -----------------pending


  var blocks = prefixBlocks.toSeq().concat(newBlockMap.toSeq().skipUntil(function (_, key) {
    return key === startKey;
  }).takeUntil(function (_, key) {
    return key === endKey;
  }).concat([[endKey, newBlockMap.get(endKey)]]).concat(suffixBlocks.toSeq())).toOrderedMap().toSeq().reduce(function (orderMap, currentBlock) {
    var last = orderMap.toSeq().last();
    var currentBlockKey = currentBlock.getKey();

    if (last) {
      var lastBlockKey = last.getKey();
      var lastBlock = orderMap.get(lastBlockKey);
      var newLastBlock = lastBlock.merge({
        nextSibling: currentBlockKey
      });
      orderMap = orderMap.set(lastBlockKey, newLastBlock);
      var newCurrentBlock = currentBlock.merge({
        prevSibling: lastBlockKey
      });
      orderMap = orderMap.set(currentBlockKey, newCurrentBlock);
    } else if (currentBlock) {
      orderMap = orderMap.set(currentBlockKey, currentBlock);
    }

    return orderMap;
  }, Immutable.OrderedMap());

  if (blocksBeforeLast) {
    var prefixBlock = blocks.toSeq().first();
    blocks = blocks.skip(1);
    var key = prefixBlock.getKey();
    blocksBefore = blocksBefore.toOrderedMap().set(key, prefixBlock);
  }

  if (blocksAfterFirst) {
    var suffixBlock = blocks.toSeq().last();
    var suffixBlockKey = suffixBlock.getKey();
    blocks = blocks.toSeq().takeUntil(function (_, key) {
      return key === suffixBlockKey;
    }).toOrderedMap();
    blocksAfter = blocksAfter.toOrderedMap().set(suffixBlockKey, suffixBlock);
  }

  return blocksBefore.concat(blocks, blocksAfter).toOrderedMap();
};

function StateFilterPlugin() {
  this.apply = function (getEditor) {
    var _getEditor = getEditor(),
        hooks = _getEditor.hooks;

    hooks.stateFilter.tap('StateFilterPlugin', function (oldEditorState, editorState, pasteText) {
      var oldBlockMap = oldEditorState.getCurrentContent().getBlockMap();
      var contentState = editorState.getCurrentContent();
      var blockMap = contentState.getBlockMap();
      var changeType = editorState.getLastChangeType(); // 解决的问题
      // 1. 当用户从vscode复制代码到编辑器中时，会保持原有的style
      // 2. TODO 如果说是从github上直接复制的代码的话，目前会存在所有的代码都被放置到一行的问题，目前能够
      //    想到的比较合适的解决方式就是，不再使用editorState中的block，通过`text`直接来创建新的block
      //    然后再插入到contentState中

      if (changeType === 'insert-fragment' && blockMap.size !== oldBlockMap.size) {
        try {
          var oldSelection = oldEditorState.getSelection();
          var oldStartKey = oldSelection.getStartKey();
          var oldStartBlock = oldBlockMap.get(oldStartKey);
          if (!oldStartBlock) return editorState;
          if (oldStartBlock.getType() !== 'code-block') return editorState;
          var parts = pasteText.split('\n').filter(function (part) {
            return part !== '';
          });
          var index = 0;
          var newBlockMap = blockMap;
          var blockKeyHasChildren;
          var lastBlockKey = '';
          blockMap.skipUntil(function (_, key) {
            return key === oldStartKey;
          }).takeUntil(function (_, key) {
            return key !== oldStartKey && !!oldBlockMap.get(key);
          }).toOrderedMap().forEach(function (block, blockKey) {
            lastBlockKey = blockKey; // 通过pasteText中的样式，来补充block中的样式

            var blockText = block.getText();
            var originText = parts[index];
            var childKeys = block.getChildKeys().toArray();

            if (childKeys.length) {
              blockKeyHasChildren = blockKey;
              return;
            }

            if (blockText.trim() === originText.trim()) {
              var paddingLength = originText.length - block.getLength();

              if (paddingLength) {
                var newBlock = block.merge({
                  parent: null,
                  type: 'code-block',
                  text: ' '.repeat(paddingLength) + block.getText(),
                  characterList: Immutable.Repeat(draftJs.CharacterMetadata.create({
                    style: Immutable.OrderedSet(),
                    entity: undefined
                  }), paddingLength).toList().concat(block.getCharacterList())
                });
                newBlockMap = newBlockMap.set(blockKey, newBlock);
              }

              index++;
            } else {
              var _newBlock = block.merge({
                parent: null,
                type: 'code-block'
              });

              newBlockMap = newBlockMap.set(blockKey, _newBlock);
            }
          });

          if (blockKeyHasChildren) {
            newBlockMap = removeBlock(newBlockMap, blockKeyHasChildren);
          }

          newBlockMap = flattenBlocks(newBlockMap, oldStartKey, lastBlockKey);
          var newContent = contentState.merge({
            blockMap: newBlockMap
          });
          var next = draftJs.EditorState.push(editorState, newContent, 'change-block-type');
          return draftJs.EditorState.forceSelection(next, newContent.getSelectionAfter());
        } catch (err) {
          console.error('stateFilterPlugin : ', err);
        }
      }

      return editorState;
    });
  };
}

var containerIndex = 0;
var draggerIndex = 0;
var containerKeyExtractor = function containerKeyExtractor() {
  return "ctx_" + containerIndex++;
};
var draggerKeyExtractor = function draggerKeyExtractor() {
  return "dragger_" + draggerIndex++;
};

var SortedItems = /*#__PURE__*/function () {
  function SortedItems(_ref) {
    var sorter = _ref.sorter;
    this.sorter = sorter;
    this.items = [];
  }

  var _proto = SortedItems.prototype;

  _proto.add = function add(item) {
    this.items.push(item);
    this.items.sort(this.sorter);
  };

  _proto.remove = function remove(item) {
    var index = this.findIndex(item);
    if (index !== -1) this.items.splice(index, 1);
  };

  _proto.findIndex = function findIndex(item) {
    var id = item.id;
    return this.items.findIndex(function (item) {
      return item.id === id;
    });
  };

  _proto.getSize = function getSize() {
    return this.items.length;
  };

  _proto.getItem = function getItem(index) {
    return this.items[index];
  };

  _proto.slice = function slice() {
    for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    return [].slice.apply(this.items, args);
  };

  _proto.splice = function splice() {
    for (var _len2 = arguments.length, args = new Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
      args[_key2] = arguments[_key2];
    }

    return [].splice.apply(this.items, args);
  };

  return SortedItems;
}();

var orientationToAxis = {
  vertical: 'y',
  horizontal: 'x'
};
var axisMeasure = {
  y: ['top', 'bottom'],
  x: ['left', 'right']
};
var orientationToMeasure = function orientationToMeasure(orientation) {
  var axis = orientationToAxis[orientation];
  return axisMeasure[axis];
};
var isFunction = function isFunction(fn) {
  return typeof fn === 'function';
};

var Container = /*#__PURE__*/function () {
  function Container(_ref) {
    var el = _ref.el,
        containers = _ref.containers,
        dndConfig = _ref.dndConfig,
        containerConfig = _ref.containerConfig;
    this.el = el;
    this.id = containerKeyExtractor();
    this.containers = containers;
    this.children = new SortedItems({
      sorter: this.sorter.bind(this)
    });
    this.dndConfig = dndConfig;
    this.containerConfig = containerConfig;
    this.containers[this.id] = this;
    this.dimension = {};
    this.parentContainer = null;
  }

  var _proto = Container.prototype;

  _proto.sorter = function sorter(a, b) {
    var orientation = this.containerConfig.orientation;
    var axis = orientationToAxis[orientation];
    var _axisMeasure$axis = axisMeasure[axis],
        minProperty = _axisMeasure$axis[0];
    var aValue = a.dimension[minProperty] || 0;
    var bValue = b.dimension[minProperty] || 0;
    return aValue - bValue;
  } // used for transition
  ;

  _proto.addDirectChild = function addDirectChild(dragger) {
    var _this = this;

    this.children.add(dragger);
    dragger.container = this;

    dragger._teardown = function () {
      var index = _this.children.findIndex(dragger);

      if (index !== -1) _this.children.splice(index, 1);
    };
  };

  _proto.cleanup = function cleanup() {
    delete this.containers[this.id];
  };

  return Container;
}();

var Dragger = /*#__PURE__*/function () {
  function Dragger(_ref) {
    var el = _ref.el,
        container = _ref.container;
    this.container = container;
    this.el = el;
    this._teardown = null;
    this.id = draggerKeyExtractor();
    this.dimension = {};
  }

  var _proto = Dragger.prototype;

  _proto.teardown = function teardown() {
    if (!this.container) return;
    if (typeof this._teardown === 'function') this._teardown();
  };

  return Dragger;
}();

var Alignment;

(function (Alignment) {
  Alignment["Center"] = "center";
  Alignment["Right"] = "right";
  Alignment["Left"] = "left";
  Alignment["LeftFill"] = "leftFill";
  Alignment["RightFill"] = "rightFill";
})(Alignment || (Alignment = {}));

var Position;

(function (Position) {
  Position["Top"] = "top";
  Position["Right"] = "right";
  Position["Bottom"] = "bottom";
  Position["Left"] = "left";
})(Position || (Position = {}));

var Direction;

(function (Direction) {
  Direction["Row"] = "row";
  Direction["Column"] = "column";
})(Direction || (Direction = {}));

var Label;

(function (Label) {
  Label["H1"] = "fas fa-heading";
  Label["Blockquote"] = "fas fa-quote-left";
  Label["UL"] = "fas fa-list-ul";
  Label["OL"] = "fas fa-list-ol";
  Label["Code Block"] = "fas fa-code";
})(Label || (Label = {}));

var Orientation;

(function (Orientation) {
  Orientation["Vertical"] = "vertical";
  Orientation["Horizontal"] = "horizontal";
})(Orientation || (Orientation = {}));

var Mode;

(function (Mode) {
  Mode["Fluid"] = "fluid";
  Mode["Snap"] = "snap";
  Mode["Nested"] = "nested";
})(Mode || (Mode = {}));

var OnMoveOperation;

(function (OnMoveOperation) {
  OnMoveOperation["OnStart"] = "onStart";
  OnMoveOperation["OnEnter"] = "onEnter";
  OnMoveOperation["OnLeave"] = "onLeave";
  OnMoveOperation["ReOrder"] = "reOrder";
})(OnMoveOperation || (OnMoveOperation = {}));

// https://stackoverflow.com/questions/2752349/fast-rectangle-to-rectangle-intersection

var intersect = function intersect(a, b) {
  return a.left < b.right && a.right > b.left && a.top < b.bottom && a.bottom > b.top;
};
var overlapOnVerticalEdge = function overlapOnVerticalEdge(a, b) {
  return Math.abs(b.right - a.left) === a.right - a.left + b.right - b.left;
};
var overlapOnHorizontalEdge = function overlapOnHorizontalEdge(a, b) {
  return Math.abs(b.bottom - a.top) === a.bottom - a.top + b.bottom - b.top;
};
var overlapOnEdge = function overlapOnEdge(a, b) {
  return overlapOnHorizontalEdge(a, b) || overlapOnVerticalEdge(a, b);
};
var clamped = function clamped(value, min, max) {
  return value >= min && value <= max;
};
var coincide = function coincide(a, b) {
  return a.left === b.left && a.right === b.right && a.top === b.top && a.bottom === b.bottom;
};
var contains = function contains(a, b) {
  return containsRight(a, b) || containsRight(b, a);
};
var containsRight = function containsRight(a, b) {
  return a.top < b.top && a.right > b.right && a.bottom > b.bottom && a.left < b.left;
};
var within = function within(rect, point) {
  var clientX = point[0],
      clientY = point[1];
  var top = rect.top,
      right = rect.right,
      bottom = rect.bottom,
      left = rect.left;
  return clamped(clientX, left, right) && clamped(clientY, top, bottom);
};
var pointInRectWithOrientation = function pointInRectWithOrientation(point, rect, orientation) {
  if (orientation === void 0) {
    orientation = 'vertical';
  }

  var top = rect.top,
      right = rect.right,
      bottom = rect.bottom,
      left = rect.left;

  if (orientation === 'vertical') {
    var topPart = {
      top: top,
      right: right,
      bottom: top + (bottom - top) / 2,
      left: left
    };
    var bottomPart = {
      top: top + (bottom - top) / 2,
      right: right,
      bottom: bottom,
      left: left
    };
    if (within(topPart, point)) return Position.Top;
    if (within(bottomPart, point)) return Position.Bottom;
  } else {
    var leftPart = {
      top: top,
      right: left + (right - left) / 2,
      bottom: bottom,
      left: left
    };
    var rightPart = {
      top: top,
      right: right,
      bottom: bottom,
      left: left + (right - left) / 2
    };
    if (within(leftPart, point)) return Position.Left;
    if (within(rightPart, point)) return Position.Right;
  }

  return null;
};

var isElement = function isElement(el) {
  return el instanceof Element || el instanceof HTMLDocument;
};
var matchesDragger = function matchesDragger(el, configs) {
  if (!isElement(el)) return -1;
  var len = configs.length;

  for (var i = 0; i < len; i++) {
    var config = configs[i];
    var draggerSelector = config.draggerSelector;

    if (el.matches(draggerSelector)) {
      return config;
    }
  }

  return -1;
};
var matchesContainer = function matchesContainer(el, configs) {
  if (!isElement(el)) return -1;
  var len = configs.length;

  for (var i = 0; i < len; i++) {
    var config = configs[i];
    var containerSelector = config.containerSelector;

    if (el.matches(containerSelector)) {
      return config;
    }
  }

  return -1;
};
var getViewport = function getViewport() {
  var _window = window,
      innerHeight = _window.innerHeight,
      innerWidth = _window.innerWidth;
  var _document$documentEle = document.documentElement,
      clientHeight = _document$documentEle.clientHeight,
      clientWidth = _document$documentEle.clientWidth;
  return {
    top: 0,
    right: innerWidth || clientWidth,
    bottom: innerHeight || clientHeight,
    left: 0
  };
}; // https://stackoverflow.com/a/7557433/2006805
var isElementVisibleInViewport = function isElementVisibleInViewport(el) {
  var viewport = getViewport();
  var rect = el.getBoundingClientRect();
  return intersect(viewport, rect);
}; // Has rect cache.. If container's rect is not update.
// It is recommended to use this method

var withinElement = function withinElement(el) {
  var rect = el.getBoundingClientRect();
  return function (point) {
    return within(rect, point);
  };
};

var supportedMatchesName = /*#__PURE__*/function () {
  var base = 'matches'; // Server side rendering

  if (typeof document === 'undefined') {
    return base;
  } // See https://developer.mozilla.org/en-US/docs/Web/API/Page_Visibility_API


  var candidates = [base, 'msMatchesSelector', 'webkitMatchesSelector'];
  var value = /*#__PURE__*/find(candidates, function (name) {
    return name in Element.prototype;
  });
  return value || base;
}();

function closestPonyfill(el, selector) {
  if (el == null) {
    return null;
  } // Element.prototype.matches is supported in ie11 with a different name
  // https://caniuse.com/#feat=matchesselector
  // $FlowFixMe - dynamic property


  if (el[supportedMatchesName](selector)) {
    return el;
  } // recursively look up the tree


  return closestPonyfill(el.parentElement, selector);
} // current element will be excluded from search result
function closest(el, selector) {
  if (!el) return null; // Using native closest for maximum speed where we can

  if (el.closest) {
    return el.closest(selector);
  } // ie11: damn you!


  return closestPonyfill(el, selector);
}

function findIndex(list, predicate) {
  if (list.findIndex) {
    return list.findIndex(predicate);
  } // Using a for loop so that we can exit early


  for (var i = 0; i < list.length; i++) {
    if (predicate(list[i])) {
      return i;
    }
  } // Array.prototype.find returns -1 when nothing is found


  return -1;
}
function find(list, predicate) {
  if (list.find) {
    return list.find(predicate);
  }

  var index = findIndex(list, predicate);

  if (index !== -1) {
    return list[index];
  } // Array.prototype.find returns undefined when nothing is found


  return undefined;
} // `closest` will return itself if self matches selector.
// So, it needs support condition in which self should be excluded.

var findClosestContainer = function findClosestContainer(containers, el, strictMode) {
  if (!el) return -1;
  var node = strictMode ? el.parentNode : el;
  var directParent = closest(node, '[data-is-container="true"]');
  var containerKeys = Object.keys(containers);
  var len = containerKeys.length;

  for (var i = 0; i < len; i++) {
    var key = containerKeys[i];
    var container = containers[key];
    if (container.el === directParent) return container;
  }

  return -1;
};
var findClosestDraggerElementFromEvent = function findClosestDraggerElementFromEvent(event) {
  var target = event.target;
  var dragger = closest(target, '[data-is-dragger="true"]');
  if (dragger) return dragger;
  return -1;
};

var defaultDndConfig = {
  mode: Mode.Fluid,
  collisionPadding: 10,
  withPlaceholder: true,
  isNested: true,
  onDrop: function onDrop() {}
};
var reservedKeys = ['mode', 'collisionPadding', 'withPlaceholder', 'isNested', 'onDrop'];
var resolveDndConfig = (function (props) {
  var next = _extends({}, defaultDndConfig, props);

  var o = {};

  for (var key in next) {
    if (reservedKeys.indexOf(key) !== -1) {
      o[key] = next[key];
    }
  }

  return o;
});

var defaultConfig = {
  orientation: Orientation.Vertical
};
var reservedKeys$1 = ['orientation', 'draggerHandlerSelector', 'containerSelector', 'draggerSelector', 'shouldAcceptDragger', 'containerEffect', 'draggerEffect', 'impactDraggerEffect', 'impactContainerEffect'];
var resolveConfig = (function (configs, props) {
  return configs.map(function (config) {
    var next = _extends({}, props, defaultConfig, config);

    var o = {};

    for (var key in next) {
      if (reservedKeys$1.indexOf(key) !== -1) {
        o[key] = next[key];
      }
    }

    return o;
  });
});

var containerSelector = '[data-is-container="true"]';
var isCloneElement = function isCloneElement(el) {
  var attributeValue = el.getAttribute('data-is-clone');
  return Boolean(attributeValue);
};
var setCloneAttributes = function setCloneAttributes(el) {
  el.setAttribute('data-is-clone', 'true');
  var children = el.children;
  var len = children.length;

  if (len) {
    for (var i = 0; i < len; i++) {
      setCloneAttributes(children[i]);
    }
  }
};
var setContainerAttributes = function setContainerAttributes(container, config) {
  var orientation = config.orientation;
  var id = container.id;
  var el = container.el;
  el.setAttribute('data-is-container', 'true');
  el.setAttribute('data-container-id', id);
  el.setAttribute('data-orientation', orientation);
};
var setDraggerAttributes = function setDraggerAttributes(container, dragger) {
  var containerId = container.id;
  var draggerId = dragger.id;
  var el = dragger.el;
  el.setAttribute('data-is-dragger', 'true');
  el.setAttribute('data-dragger-id', draggerId);
  el.setAttribute('data-container-context', containerId);
};
var getVContainerId = function getVContainerId(containerNode) {
  return containerNode.getAttribute('data-container-id');
};
var getVContainer = function getVContainer(containerNode, vContainers) {
  var vContainerId = getVContainerId(containerNode);
  if (vContainerId) return vContainers[vContainerId];
  return null;
}; // https://developer.mozilla.org/en-US/docs/Web/API/DocumentOrShadowRoot/elementFromPoint

var containerElementFromPoint = function containerElementFromPoint(point) {
  // const root = document.querySelector('.DraftEditor-root')
  var x = point[0],
      y = point[1];
  var elements = document.elementsFromPoint(x, y);
  if (!elements) return null;
  var len = elements.length;
  var candidate = null;

  for (var i = 0; i < len; i++) {
    var node = elements[i];

    if (!isCloneElement(node)) {
      candidate = node;
      break;
    }
  } // Maybe closest is not needed... loop `elements` util find the first
  // element matches dragger selector.


  return closest(candidate, containerSelector);
};
var closestExclusiveContainerNodeFromElement = function closestExclusiveContainerNodeFromElement(el) {
  var parent = el.parentNode;
  if (parent) return closest(parent, containerSelector);
  return null;
};

var capitalize = function capitalize(s) {
  if (typeof s !== 'string') return '';
  return s.charAt(0).toUpperCase() + s.slice(1);
};

function reporter() {
  this.logAddEffect = function (component) {
    var id = component.id;
    console.log("Add effect to component %c" + id, 'background: #222; color: red');
  };

  this.logRemoveEffect = function (component) {
    var id = component.id;
    console.log("Remove effect from component %c" + id, 'background: #222; color: #bada55');
  };

  var manipulateNode = function manipulateNode(action, name) {
    return function (node) {
      console.log(capitalize(action) + " " + name + " node", node);
    };
  };

  this.addContainerNode = manipulateNode('add', 'container');
  this.addDraggerNode = manipulateNode('add', 'dragger');
  this.removeContainerNode = manipulateNode('remove', 'container');

  this.logEnterContainer = function (container) {
    console.log("On enter: %c" + container.id, 'color: #bada55');
  };

  this.logLeaveContainer = function (container) {
    console.log("On leave: %c" + container.id, 'color: #bada55');
  };
} // @ts-ignore


var report = /*#__PURE__*/new reporter();

var mutationHandler = (function (ctx) {
  return function (mutationList) {
    var containers = ctx.containers;
    var configs = ctx.configs;

    for (var _iterator = _createForOfIteratorHelperLoose(mutationList), _step; !(_step = _iterator()).done;) {
      var mutation = _step.value;
      var addedNodes = mutation.addedNodes,
          removedNodes = mutation.removedNodes;

      if (addedNodes.length) {
        (function () {
          var merged = [];
          addedNodes.forEach(function (node) {
            if (node && isElement(node)) {
              try {
                merged.push(node); // If dom is wrapped with a new div container, Only the new parent
                // div will be remarked.
                // https://stackoverflow.com/questions/8321874/how-to-get-all-childnodes-in-js-including-all-the-grandchildren

                var elements = node.getElementsByTagName('div');
                merged = [].concat(merged, elements);
              } catch (err) {
                console.log('err ', err);
              }
            }
          });
          merged.forEach(function (node) {
            var containerConfig = matchesContainer(node, configs);

            if (containerConfig !== -1) {
              ctx.handleContainerElement(node, containerConfig); // In case, dragger is batch updated by container..

              ctx.handleDraggers(node);
            } // A container could be a dragger


            var matchedDragger = matchesDragger(node, configs);

            if (matchedDragger !== -1) {
              ctx.handleDraggerElement(node);
            }
          });
        })();
      }

      if (removedNodes.length) {
        removedNodes.forEach(function (node) {
          var matchedContainer = matchesContainer(node, configs);

          if (matchedContainer !== -1) {
            var containerId = node.getAttribute('data-container-id');
            if (!containerId) return;
            var container = containers[containerId];
            if (container) container.cleanup();
          }
        });
      }
    }
  };
});

// only resolve the style of container or dragger enclosed by container.

var getDimension = function getDimension(v) {
  var el = v.el;
  var rect = el.getBoundingClientRect();
  return rect;
}; // If it is a container, subject property may be required which will indicate
// whether it is visible or not.


var getSubject = function getSubject(el) {
  return {
    isVisible: isElementVisibleInViewport(el)
  };
};

var getDimensions = (function (ctx, actions) {
  var context = ctx;
  var vDraggers = context.vDraggers,
      vContainers = context.vContainers;
  var draggerKeys = Object.keys(vDraggers);
  var containerKeys = Object.keys(vContainers);
  draggerKeys.forEach(function (key) {
    var dragger = vDraggers[key];
    var rect = getDimension(dragger);
    dragger.dimension = {
      rect: rect
    };
  });
  containerKeys.forEach(function (key) {
    var container = vContainers[key];
    var rect = getDimension(container);
    container.dimension = {
      rect: rect,
      subject: getSubject(container.el),
      within: withinElement(container.el)
    };
  });
  actions.next();
});

var getDimensionsNested = (function (ctx, actions) {
  var context = ctx;
  var vDraggers = context.vDraggers,
      dndConfig = context.dndConfig;
  var mode = dndConfig.mode,
      _dndConfig$collisionP = dndConfig.collisionPadding,
      collisionPadding = _dndConfig$collisionP === void 0 ? 0 : _dndConfig$collisionP;

  if (mode !== 'nested') {
    actions.next();
    return;
  } // Only if under `nested` mode, `dimension` should be appended with
  // `collisionRect` property.


  for (var key in vDraggers) {
    var _firstCollisionRect, _secondCollisionRect;

    var dragger = vDraggers[key];
    var container = dragger.container,
        dimension = dragger.dimension;
    var rect = dimension.rect;
    var top = rect.top,
        right = rect.right,
        bottom = rect.bottom,
        left = rect.left;
    var containerConfig = container.containerConfig;
    var orientation = containerConfig.orientation;
    var axis = orientationToAxis[orientation];
    var _axisMeasure$axis = axisMeasure[axis],
        first = _axisMeasure$axis[0],
        second = _axisMeasure$axis[1];
    var firstCollisionRect = (_firstCollisionRect = {
      top: top,
      right: right,
      bottom: bottom,
      left: left
    }, _firstCollisionRect[first] = Math.max(rect[first] - collisionPadding, 0), _firstCollisionRect[second] = rect[first], _firstCollisionRect);
    var secondCollisionRect = (_secondCollisionRect = {
      top: top,
      right: right,
      bottom: bottom,
      left: left
    }, _secondCollisionRect[first] = rect[second], _secondCollisionRect[second] = rect[second] + collisionPadding, _secondCollisionRect);
    dragger.dimension = _extends({}, dimension, {
      firstCollisionRect: firstCollisionRect,
      secondCollisionRect: secondCollisionRect
    });
  }

  actions.next();
});

/**
 * 1. Container should be enclosed by other container entirely.
 * 2. Container should be on the outer of other container.
 * 3. The intersection of containers is not allowed.
 */

var validateContainers = (function (ctx, actions) {
  var context = ctx;
  var vContainers = context.vContainers;
  var keys = Object.keys(vContainers);
  var len = keys.length;

  for (var i = 0; i < len; i++) {
    var containerA = vContainers[keys[i]];

    for (var j = i + 1; j < len; j++) {
      var containerB = vContainers[keys[j]];
      var a = containerA.dimension.rect;
      var b = containerB.dimension.rect; // To ensure there are spaces between containers.

      if (intersect(a, b) && !coincide(a, b) && !contains(a, b) && !overlapOnEdge(a, b)) {
        console.warn('=======================================\n' + 'The interaction of containers is forbidden\n' + ("  containerA's id: " + containerA.id + "\n") + ("  containerB's id: " + containerB.id + "\n"));
      }
    }
  }

  actions.next();
});

/**
 * Clone node should be created if there is no copy when moving
 */
// cloneNode will not preserve node style. It requires to set clone element with fixed style

var attemptToCreateClone = (function (args, ctx, actions) {
  var dragger = args.dragger;
  var context = ctx;
  var el = dragger.el;
  context.extra.clone = el.cloneNode(true);
  var rect = el.getBoundingClientRect();
  var width = rect.width,
      height = rect.height;
  context.extra.clone.style.width = width + "px";
  context.extra.clone.style.height = height + "px";
  context.extra.clone.style.zIndex = '1';
  context.extra.clone.style.backgroundColor = 'transparent';
  setCloneAttributes(context.extra.clone);
  document.body.appendChild(context.extra.clone);
  actions.next();
});

var syncCopyPosition = (function (args, _, actions) {
  var impactPoint = args.impactPoint,
      clone = args.clone;
  var clientX = impactPoint[0],
      clientY = impactPoint[1];
  clone.style.position = 'fixed';
  clone.style.top = clientY + "px";
  clone.style.left = clientX + "px";
  actions.next();
});

var addIntermediateCtxValue = function addIntermediateCtxValue(ctx, actions) {
  var context = ctx;
  context.action = {
    operation: OnMoveOperation.OnStart,
    isHomeContainerFocused: false,
    effectsManager: null
  };
  actions.next();
};

var removeIntermediateCtxValue = function removeIntermediateCtxValue(ctx, actions) {
  var context = ctx;
  delete context.action;
  actions.next();
};

var handleLeaveContainer = function handleLeaveContainer(args, ctx, actions) {
  var isHomeContainer = args.isHomeContainer,
      prevImpact = args.prevImpact;
  var context = ctx;
  var impactRawInfo = context.impactRawInfo,
      dndEffects = context.dndEffects;
  var prevImpactVContainer = prevImpact.impactVContainer;
  var currentImpactVContainer = impactRawInfo.impactVContainer;

  if (!prevImpactVContainer && currentImpactVContainer) {
    actions.next();
    return;
  }

  if (prevImpactVContainer && !currentImpactVContainer || prevImpactVContainer && currentImpactVContainer && prevImpactVContainer.id !== currentImpactVContainer.id) {
    var effectsManager = dndEffects.find(prevImpactVContainer.id);
    report.logLeaveContainer(prevImpactVContainer);
    context.action = {
      operation: OnMoveOperation.OnLeave,
      isHomeContainerFocused: isHomeContainer(prevImpactVContainer),
      effectsManager: effectsManager
    };
  }

  actions.next();
};

var handleLeaveHomeContainer = function handleLeaveHomeContainer(args, ctx, actions) {
  var prevImpact = args.prevImpact,
      liftUpVDraggerIndex = args.liftUpVDraggerIndex;
  var context = ctx;
  var withPlaceholder = context.dndConfig.withPlaceholder,
      _context$action = context.action,
      operation = _context$action.operation,
      isHomeContainerFocused = _context$action.isHomeContainerFocused,
      effectsManager = _context$action.effectsManager;

  if (operation !== 'onLeave' || !isHomeContainerFocused) {
    actions.next();
    return;
  }

  if (!withPlaceholder) {
    effectsManager.teardown();
    actions.next();
    return;
  }

  var index = prevImpact.index,
      impactVContainer = prevImpact.impactVContainer;

  if (!index || !impactVContainer) {
    actions.next();
  }

  var children = impactVContainer.children,
      _impactVContainer$con = impactVContainer.containerConfig,
      orientation = _impactVContainer$con.orientation,
      draggerEffect = _impactVContainer$con.draggerEffect; // Don't care prev index, reset all the effects on the items before
  // `liftUpVDraggerIndex`

  effectsManager.clearDownstreamEffects();

  if (typeof draggerEffect === 'function') {
    var upstreamStartIndex = Math.max(liftUpVDraggerIndex + 1, index);
    var len = children.getSize();

    for (var i = upstreamStartIndex; i < len; i++) {
      var vDragger = children.getItem(i);
      var measure = orientationToMeasure(orientation);
      var teardown = draggerEffect({
        el: vDragger.el,
        shouldMove: true,
        placedPosition: measure[0],
        downstream: false,
        dimension: vDragger.dimension.rect,
        isHighlight: false
      });
      effectsManager.upstreamDraggersEffects.push({
        teardown: teardown,
        vDragger: vDragger
      });
    }
  }

  actions.next();
};

var handleLeaveOtherContainer = function handleLeaveOtherContainer(ctx, actions) {
  var context = ctx;
  var _context$action = context.action,
      operation = _context$action.operation,
      isHomeContainerFocused = _context$action.isHomeContainerFocused,
      effectsManager = _context$action.effectsManager;

  if (operation !== 'onLeave' || isHomeContainerFocused) {
    actions.next();
    return;
  }

  effectsManager.teardown();
  actions.next();
};

var EffectsManager = /*#__PURE__*/function () {
  function EffectsManager(_ref) {
    var dragger = _ref.dragger,
        impactContainer = _ref.impactContainer;
    this.dragger = dragger;
    this.impactContainer = impactContainer;
    this.id = impactContainer.id;
    this.impactDraggerEffects = [];
    this.impactContainerEffects = [];
    this.upstreamDraggersEffects = [];
    this.downstreamDraggersEffects = [];
  }

  var _proto = EffectsManager.prototype;

  _proto.isHomeContainerEffects = function isHomeContainerEffects() {
    var container = this.dragger.container;
    return container.id === this.impactContainer.id;
  };

  _proto.assertRun = function assertRun(fn) {
    if (isFunction(fn)) fn();
  };

  _proto.clearImpactContainerEffects = function clearImpactContainerEffects() {
    var _this = this;

    this.impactContainerEffects.forEach(function (_ref2) {
      var teardown = _ref2.teardown;
      return _this.assertRun(teardown);
    });
    this.impactContainerEffects = [];
  };

  _proto.clearImpactDraggerEffects = function clearImpactDraggerEffects() {
    var _this2 = this;

    this.impactDraggerEffects.forEach(function (_ref3) {
      var teardown = _ref3.teardown;
      return _this2.assertRun(teardown);
    });
    this.impactDraggerEffects = [];
  };

  _proto.clearDownstreamEffects = function clearDownstreamEffects() {
    var _this3 = this;

    this.downstreamDraggersEffects.forEach(function (_ref4) {
      var teardown = _ref4.teardown;
      return _this3.assertRun(teardown);
    });
    this.downstreamDraggersEffects = [];
  };

  _proto.clearUpstreamEffects = function clearUpstreamEffects() {
    var _this4 = this;

    this.upstreamDraggersEffects.forEach(function (_ref5) {
      var teardown = _ref5.teardown;
      return _this4.assertRun(teardown);
    });
    this.upstreamDraggersEffects = [];
  };

  _proto.teardown = function teardown() {
    this.clearImpactContainerEffects();
    this.clearImpactDraggerEffects();
    this.clearUpstreamEffects();
    this.clearDownstreamEffects();
  };

  return EffectsManager;
}();

var handleEnterContainer = function handleEnterContainer(args, ctx, actions) {
  var lifeUpDragger = args.lifeUpDragger,
      isHomeContainer = args.isHomeContainer,
      prevImpact = args.prevImpact;
  var context = ctx;
  var impactRawInfo = context.impactRawInfo,
      dndEffects = context.dndEffects;
  var prevImpactVContainer = prevImpact.impactVContainer;
  var currentImpactVContainer = impactRawInfo.impactVContainer;

  if (!prevImpactVContainer && currentImpactVContainer || prevImpactVContainer && currentImpactVContainer && prevImpactVContainer.id !== currentImpactVContainer.id) {
    var effectsManager = dndEffects.find(currentImpactVContainer.id);
    var impactVContainer = impactRawInfo.impactVContainer;

    if (!effectsManager) {
      effectsManager = new EffectsManager({
        dragger: lifeUpDragger,
        impactContainer: impactVContainer
      });
      dndEffects.add(effectsManager);
    }

    report.logEnterContainer(currentImpactVContainer);
    context.action = {
      operation: OnMoveOperation.OnEnter,
      isHomeContainerFocused: isHomeContainer(currentImpactVContainer),
      effectsManager: effectsManager
    };
    context.impact = {
      impactVContainer: currentImpactVContainer,
      index: null
    };
  }

  actions.next();
};

var handleEnterHomeContainer = function handleEnterHomeContainer(args, ctx, actions) {
  var liftUpVDraggerIndex = args.liftUpVDraggerIndex;
  var context = ctx;
  var impactRawInfo = context.impactRawInfo,
      _context$action = context.action,
      operation = _context$action.operation,
      isHomeContainerFocused = _context$action.isHomeContainerFocused,
      effectsManager = _context$action.effectsManager;

  if (operation !== 'onEnter' || !isHomeContainerFocused) {
    actions.next();
    return;
  }

  var candidateVDragger = impactRawInfo.candidateVDragger,
      impactVContainer = impactRawInfo.impactVContainer,
      impactPosition = impactRawInfo.impactPosition,
      candidateVDraggerIndex = impactRawInfo.candidateVDraggerIndex;
  var _impactVContainer$con = impactVContainer.containerConfig,
      draggerEffect = _impactVContainer$con.draggerEffect,
      orientation = _impactVContainer$con.orientation,
      children = impactVContainer.children;
  var measure = orientationToMeasure(orientation);
  var positionIndex = measure.indexOf(impactPosition);

  if (typeof draggerEffect !== 'function') {
    actions.next();
    return;
  }

  var impact = {
    impactVContainer: impactVContainer,
    index: positionIndex ? candidateVDraggerIndex + 1 : candidateVDraggerIndex
  };

  if (candidateVDraggerIndex === liftUpVDraggerIndex) {
    if (!positionIndex) effectsManager.clearUpstreamEffects();else {
      var remainingEffects = [];
      effectsManager.upstreamDraggersEffects.forEach(function (_ref) {
        var vDragger = _ref.vDragger,
            teardown = _ref.teardown;
        if (vDragger.id !== candidateVDragger.id) teardown();else remainingEffects.push({
          vDragger: vDragger,
          teardown: teardown
        });
      });
      effectsManager.upstreamDraggersEffects = remainingEffects;
    }
  }

  if (candidateVDraggerIndex < liftUpVDraggerIndex) {
    effectsManager.clearUpstreamEffects();
    var initialValue = candidateVDraggerIndex;

    for (var i = initialValue; i < liftUpVDraggerIndex; i++) {
      var vDragger = children.getItem(i);
      var isHighlight = initialValue === i;
      var teardown = draggerEffect({
        el: vDragger.el,
        shouldMove: !isHighlight || !positionIndex,
        downstream: !isHighlight || !positionIndex,
        placedPosition: isHighlight ? impactPosition : measure[0],
        dimension: vDragger.dimension.rect,
        isHighlight: isHighlight
      });
      effectsManager.downstreamDraggersEffects.push({
        teardown: teardown,
        vDragger: vDragger
      });
    }
  }

  if (candidateVDraggerIndex && candidateVDraggerIndex > liftUpVDraggerIndex) {
    var _initialValue = liftUpVDraggerIndex + 1;

    var endValue = candidateVDraggerIndex;
    if (positionIndex) endValue += 1;
    var reserved = [];
    var reservedEffects = [];

    for (var _i = _initialValue; _i < endValue; _i++) {
      var _vDragger = children.getItem(_i);

      reserved.push(_vDragger);
    }

    effectsManager.upstreamDraggersEffects.forEach(function (_ref2) {
      var teardown = _ref2.teardown,
          vDragger = _ref2.vDragger;
      var id = vDragger.id;
      var index = reserved.findIndex(function (vDragger) {
        return vDragger.id === id;
      });

      if (index !== -1) {
        reservedEffects.push({
          teardown: teardown,
          vDragger: vDragger
        });
      } else if (teardown) teardown();
    });
    effectsManager.upstreamDraggersEffects = reservedEffects;
  }

  context.impact = impact;
  actions.next();
};

var handleEnterOtherContainer = function handleEnterOtherContainer(ctx, actions) {
  var context = ctx;
  var impactRawInfo = context.impactRawInfo,
      _context$action = context.action,
      operation = _context$action.operation,
      isHomeContainerFocused = _context$action.isHomeContainerFocused,
      effectsManager = _context$action.effectsManager;

  if (operation !== 'onEnter' || isHomeContainerFocused) {
    actions.next();
    return;
  }

  var impactVContainer = impactRawInfo.impactVContainer,
      impactPosition = impactRawInfo.impactPosition,
      candidateVDraggerIndex = impactRawInfo.candidateVDraggerIndex;
  var _impactVContainer$con = impactVContainer.containerConfig,
      draggerEffect = _impactVContainer$con.draggerEffect,
      orientation = _impactVContainer$con.orientation,
      children = impactVContainer.children;
  var measure = orientationToMeasure(orientation);
  var positionIndex = measure.indexOf(impactPosition);

  if (typeof draggerEffect !== 'function') {
    actions.next();
    return;
  }

  var initialValue = candidateVDraggerIndex || 0;

  if (positionIndex === 1) {
    initialValue += 1;
  }

  var impact = {
    index: initialValue,
    impactVContainer: impactVContainer
  };
  var len = children.getSize();

  for (var i = initialValue; i < len; i++) {
    var vDragger = children.getItem(i);
    var isHighlight = i === initialValue;
    var teardown = draggerEffect({
      placedPosition: isHighlight ? impactPosition : measure[0],
      shouldMove: !isHighlight || !positionIndex,
      downstream: !isHighlight || !positionIndex,
      el: vDragger.el,
      dimension: vDragger.dimension.rect,
      isHighlight: isHighlight
    });
    effectsManager.downstreamDraggersEffects.push({
      teardown: teardown,
      vDragger: vDragger
    });
  }

  context.impact = impact;
  actions.next();
};

var handleReorder = function handleReorder(args, ctx, actions) {
  var prevImpact = args.prevImpact,
      isHomeContainer = args.isHomeContainer;
  var context = ctx;
  var currentImpactVContainer = context.impactRawInfo.impactVContainer,
      dndEffects = context.dndEffects;
  var prevImpactVContainer = prevImpact.impactVContainer;

  if (prevImpactVContainer && currentImpactVContainer && prevImpactVContainer.id === currentImpactVContainer.id) {
    var effectsManager = dndEffects.find(currentImpactVContainer.id);
    context.action = {
      operation: OnMoveOperation.ReOrder,
      isHomeContainerFocused: isHomeContainer(currentImpactVContainer),
      effectsManager: effectsManager
    };
    actions.next();
    return;
  }

  actions.next();
};

var handleReorderOnHomeContainer = function handleReorderOnHomeContainer(args, ctx, actions) {
  var liftUpVDraggerIndex = args.liftUpVDraggerIndex;
  var context = ctx;
  var _context$action = context.action,
      operation = _context$action.operation,
      isHomeContainerFocused = _context$action.isHomeContainerFocused,
      effectsManager = _context$action.effectsManager;

  if (operation !== OnMoveOperation.ReOrder || !isHomeContainerFocused) {
    actions.next();
    return;
  }

  var _context$impactRawInf = context.impactRawInfo,
      candidateVDragger = _context$impactRawInf.candidateVDragger,
      impactVContainer = _context$impactRawInf.impactVContainer,
      impactPosition = _context$impactRawInf.impactPosition,
      candidateVDraggerIndex = _context$impactRawInf.candidateVDraggerIndex;
  var _impactVContainer$con = impactVContainer.containerConfig,
      orientation = _impactVContainer$con.orientation,
      draggerEffect = _impactVContainer$con.draggerEffect;
  var currentIndex = context.impact.index || 0;
  var measure = orientationToMeasure(orientation);

  if (typeof draggerEffect !== 'function') {
    actions.next();
    return;
  }

  var impact = {
    impactVContainer: impactVContainer,
    index: candidateVDraggerIndex
  }; // move down

  if (currentIndex < candidateVDraggerIndex) {
    if (impactPosition === measure[0]) {
      actions.next();
      return;
    }

    if (candidateVDraggerIndex <= liftUpVDraggerIndex) {
      var index = effectsManager.downstreamDraggersEffects.findIndex(function (_ref) {
        var vDragger = _ref.vDragger;
        return vDragger.id === candidateVDragger.id;
      });

      if (index !== -1) {
        var teardown = effectsManager.downstreamDraggersEffects[index].teardown;
        effectsManager.downstreamDraggersEffects.splice(index, 1);
        if (typeof teardown === 'function') teardown();
      }
    }

    if (candidateVDraggerIndex > liftUpVDraggerIndex) {
      var _teardown = draggerEffect({
        el: candidateVDragger.el,
        shouldMove: true,
        placedPosition: measure[1],
        downstream: false,
        dimension: candidateVDragger.dimension.rect,
        isHighlight: true
      });

      effectsManager.upstreamDraggersEffects.push({
        vDragger: candidateVDragger,
        teardown: _teardown
      });
    }
  } // move up


  if (currentIndex > candidateVDraggerIndex) {
    if (impactPosition === measure[1]) {
      actions.next();
      return;
    }

    if (candidateVDraggerIndex < liftUpVDraggerIndex) {
      var _teardown2 = draggerEffect({
        el: candidateVDragger.el,
        shouldMove: true,
        placedPosition: measure[0],
        downstream: true,
        dimension: candidateVDragger.dimension.rect,
        isHighlight: true
      });

      effectsManager.downstreamDraggersEffects.push({
        vDragger: candidateVDragger,
        teardown: _teardown2
      });
    }

    if (candidateVDraggerIndex >= liftUpVDraggerIndex) {
      var _index = effectsManager.upstreamDraggersEffects.findIndex(function (_ref2) {
        var vDragger = _ref2.vDragger;
        return vDragger.id === candidateVDragger.id;
      });

      if (_index !== -1) {
        var _teardown3 = effectsManager.upstreamDraggersEffects[_index].teardown;
        effectsManager.upstreamDraggersEffects.splice(_index, 1);
        if (typeof _teardown3 === 'function') _teardown3();
      }
    }
  }

  context.impact = impact;
  actions.next();
};

var handleReorderOnHomeContainer$1 = function handleReorderOnHomeContainer(ctx, actions) {
  var context = ctx;
  var _context$action = context.action,
      operation = _context$action.operation,
      isHomeContainerFocused = _context$action.isHomeContainerFocused,
      effectsManager = _context$action.effectsManager;

  if (operation !== OnMoveOperation.ReOrder || isHomeContainerFocused) {
    actions.next();
    return;
  }

  var _context$impactRawInf = context.impactRawInfo,
      candidateVDragger = _context$impactRawInf.candidateVDragger,
      impactVContainer = _context$impactRawInf.impactVContainer,
      impactPosition = _context$impactRawInf.impactPosition,
      candidateVDraggerIndex = _context$impactRawInf.candidateVDraggerIndex;
  var _impactVContainer$con = impactVContainer.containerConfig,
      orientation = _impactVContainer$con.orientation,
      draggerEffect = _impactVContainer$con.draggerEffect;
  var currentIndex = context.impact.index || 0;
  var measure = orientationToMeasure(orientation);

  if (typeof draggerEffect !== 'function') {
    actions.next();
    return;
  }

  var impact = {
    impactVContainer: impactVContainer,
    index: candidateVDraggerIndex
  }; // move down

  if (currentIndex < candidateVDraggerIndex) {
    if (impactPosition === measure[0]) {
      actions.next();
      return;
    }

    var index = effectsManager.downstreamDraggersEffects.findIndex(function (_ref) {
      var vDragger = _ref.vDragger;
      return vDragger.id === candidateVDragger.id;
    });

    if (index !== -1) {
      var teardown = effectsManager.downstreamDraggersEffects[index].teardown;
      effectsManager.downstreamDraggersEffects.splice(index, 1);
      if (typeof teardown === 'function') teardown();
    }
  } // move up


  if (currentIndex > candidateVDraggerIndex) {
    if (impactPosition === measure[1]) {
      actions.next();
      return;
    }

    var _teardown = draggerEffect({
      el: candidateVDragger.el,
      shouldMove: true,
      placedPosition: measure[0],
      downstream: true,
      dimension: candidateVDragger.dimension.rect,
      isHighlight: true
    });

    effectsManager.downstreamDraggersEffects.push({
      vDragger: candidateVDragger,
      teardown: _teardown
    });
  }

  context.impact = impact;
  actions.next();
};

var generateDraggerEffectKey = function generateDraggerEffectKey(vContainer, impactVDragger, placedPosition) {
  return vContainer.id + "_" + impactVDragger.id + "_" + placedPosition;
};
var generateContainerEffectKey = function generateContainerEffectKey(vContainer, status) {
  return vContainer.id + "_" + status;
};

var handleImpactDraggerEffect = function handleImpactDraggerEffect(args, ctx, actions) {
  var liftUpVDragger = args.liftUpVDragger;
  var context = ctx;
  var impactRawInfo = context.impactRawInfo,
      dndEffects = context.dndEffects,
      withPlaceholder = context.dndConfig.withPlaceholder;
  var impactVContainer = impactRawInfo.impactVContainer,
      impactPosition = impactRawInfo.impactPosition,
      candidateVDragger = impactRawInfo.candidateVDragger;

  if (withPlaceholder || !impactVContainer) {
    actions.next();
    return;
  }

  var _impactVContainer$con = impactVContainer.containerConfig,
      orientation = _impactVContainer$con.orientation,
      impactDraggerEffect = _impactVContainer$con.impactDraggerEffect;
  var effectsManager = dndEffects.find(impactVContainer.id);
  var measure = orientationToMeasure(orientation);
  var positionIndex = measure.indexOf(impactPosition);

  if (typeof impactDraggerEffect === 'function') {
    var effectKey = generateDraggerEffectKey(impactVContainer, candidateVDragger, impactPosition);
    var index = effectsManager.impactDraggerEffects.findIndex(function (_ref) {
      var key = _ref.key;
      return key === effectKey;
    });

    if (index === -1) {
      effectsManager.clearImpactDraggerEffects();
      var teardown = impactDraggerEffect({
        dragger: liftUpVDragger.el,
        container: impactVContainer.el,
        candidateDragger: candidateVDragger.el,
        shouldMove: !positionIndex,
        downstream: !positionIndex,
        placedPosition: impactPosition,
        dimension: candidateVDragger.dimension.rect,
        isHighlight: true
      });
      effectsManager.impactDraggerEffects.push({
        teardown: teardown,
        vDragger: candidateVDragger,
        key: effectKey
      });
    }

    context.output = {
      dragger: liftUpVDragger.el,
      candidateDragger: candidateVDragger.el,
      container: impactVContainer.el,
      placedPosition: impactPosition
    };
  }

  actions.next();
};

var handleImpactContainerEffect = function handleImpactContainerEffect(args, ctx, actions) {
  var prevImpact = args.prevImpact;
  var context = ctx;
  var dndEffects = context.dndEffects,
      impactRawInfo = context.impactRawInfo,
      _context$action = context.action,
      operation = _context$action.operation,
      isHomeContainerFocused = _context$action.isHomeContainerFocused,
      effectsManager = _context$action.effectsManager;

  if (operation === 'onLeave') {
    var _impactVContainer = prevImpact.impactVContainer;

    var _effectsManager = dndEffects.find(_impactVContainer.id);

    _effectsManager.clearImpactContainerEffects();
  }

  var impactVContainer = impactRawInfo.impactVContainer;

  if (operation === 'onEnter' && !isHomeContainerFocused && effectsManager && impactVContainer) {
    var impactContainerEffect = impactVContainer.containerConfig.impactContainerEffect;

    if (typeof impactContainerEffect === 'function') {
      var effectKey = generateContainerEffectKey(impactVContainer, 'active');
      var index = effectsManager.impactContainerEffects.findIndex(function (_ref) {
        var key = _ref.key;
        return key === effectKey;
      });

      if (index === -1) {
        var teardown = impactContainerEffect({
          container: impactVContainer.el,
          isHighlight: true
        });
        effectsManager.impactContainerEffects.push({
          teardown: teardown,
          vContainer: impactVContainer,
          key: effectKey
        });
      }
    }
  }

  actions.next();
};

var shouldAccept = function shouldAccept(vContainer, vDragger) {
  var containerConfig = vContainer.containerConfig;
  var el = vDragger.el;
  var draggerSelector = containerConfig.draggerSelector,
      shouldAcceptDragger = containerConfig.shouldAcceptDragger;

  if (typeof shouldAcceptDragger === 'function') {
    return shouldAcceptDragger(el);
  }

  return el.matches(draggerSelector);
};

var getRawInfo = function getRawInfo(_ref) {
  var impactPoint = _ref.impactPoint,
      candidateContainerElement = _ref.candidateContainerElement,
      vDraggers = _ref.vDraggers,
      vContainers = _ref.vContainers,
      liftUpVDragger = _ref.liftUpVDragger,
      isNested = _ref.isNested;
  // console.log('candidateContainerElement ', candidateContainerElement)
  var vContainer = getVContainer(candidateContainerElement, vContainers);
  if (!vContainer) return null; // If dragger move on to itself or its children's node container.
  // then return...

  if (liftUpVDragger.el.contains(vContainer.el)) return null;
  var orientation = vContainer.containerConfig.orientation,
      children = vContainer.children;

  if (shouldAccept(vContainer, liftUpVDragger)) {
    for (var i = 0; i < children.getSize(); i++) {
      var vDragger = children.getItem(i); // `inNested` mode, horizontal container's sensitive areas is on two sides.

      if (isNested && orientation === 'horizontal') {
        // console.log('vDragger.dimension ', vDragger.dimension, impactPoint)
        var _vDragger$dimension = vDragger.dimension,
            firstCollisionRect = _vDragger$dimension.firstCollisionRect,
            secondCollisionRect = _vDragger$dimension.secondCollisionRect;

        if (within(firstCollisionRect, impactPoint)) {
          return {
            candidateVDragger: vDragger,
            candidateVDraggerIndex: i,
            impactPosition: Position.Left,
            impactVContainer: vContainer
          };
        }

        if (within(secondCollisionRect, impactPoint)) {
          return {
            candidateVDragger: vDragger,
            candidateVDraggerIndex: i,
            impactPosition: Position.Right,
            impactVContainer: vContainer
          };
        }
      } else {
        var rect = vDragger.dimension.rect;

        if (within(rect, impactPoint)) {
          var position = pointInRectWithOrientation(impactPoint, rect, orientation);
          return {
            candidateVDragger: vDragger,
            candidateVDraggerIndex: i,
            impactPosition: position,
            impactVContainer: vContainer
          };
        }
      }
    }
  }

  var containerElement = vContainer.el;
  var nextCandidateContainerElement = closestExclusiveContainerNodeFromElement(containerElement);
  if (!nextCandidateContainerElement) return null;
  return getRawInfo({
    impactPoint: impactPoint,
    candidateContainerElement: nextCandidateContainerElement,
    vDraggers: vDraggers,
    vContainers: vContainers,
    liftUpVDragger: liftUpVDragger,
    isNested: isNested
  });
};
/**
 *
 * @param {*} param0
 * @param {*} ctx
 * @param {*} actions
 *
 * Difference between `targetContainer` and `impactContainer`...
 * `impactContainer` is bound with `impactDragger`. There is a situation
 * point is in the gap between dragger and container...
 *
 * Why it's called `getImpactRawInfo` ? Because `impactContainer` may be
 * update when checking the side padding on `nested` mode.
 */


var getImpactRawInfo = function getImpactRawInfo(args, ctx, actions) {
  var impactPoint = args.impactPoint,
      liftUpVDragger = args.liftUpVDragger;
  var context = ctx;
  var vContainers = context.vContainers,
      vDraggers = context.vDraggers,
      dndConfig = context.dndConfig;
  var isNested = dndConfig.isNested; // Find the most inner container include point
  // const candidateDraggerElement = draggerElementFromPoint(impactPoint);
  // The reason why use container ? because point maybe placed on the gap between
  // `container` and `dragger`.

  var candidateContainerElement = containerElementFromPoint(impactPoint);
  var impactRawInfo = {
    candidateVDragger: null,
    impactVContainer: null,
    impactPosition: null,
    candidateVDraggerIndex: null
  };

  if (candidateContainerElement) {
    var value = getRawInfo({
      impactPoint: impactPoint,
      candidateContainerElement: candidateContainerElement,
      vDraggers: vDraggers,
      vContainers: vContainers,
      liftUpVDragger: liftUpVDragger,
      isNested: isNested
    });
    if (value) impactRawInfo = value;
  }

  context.impactRawInfo = impactRawInfo;
  actions.next();
};

var hasDraggerHandlerMatched = function hasDraggerHandlerMatched(el, configs) {
  var len = configs.length;

  for (var i = 0; i < len; i++) {
    var draggerHandlerSelector = configs[i].draggerHandlerSelector;
    if (closest(el, draggerHandlerSelector)) return true;
  }

  return false;
};

var Mouse = /*#__PURE__*/function () {
  function Mouse(_ref) {
    var moveAPI = _ref.moveAPI,
        getClone = _ref.getClone,
        onStartHandler = _ref.onStartHandler,
        onMoveHandler = _ref.onMoveHandler,
        getDragger = _ref.getDragger,
        configs = _ref.configs,
        dndEffects = _ref.dndEffects,
        updateImpact = _ref.updateImpact,
        dndConfig = _ref.dndConfig;
    this.moveAPI = moveAPI;
    this.getClone = getClone;
    this.getDragger = getDragger;
    this.onStartHandler = onStartHandler;
    this.onMoveHandler = onMoveHandler;
    this.configs = configs;
    this.dndEffects = dndEffects;
    this.updateImpact = updateImpact;
    this.dndConfig = dndConfig;
  }

  var _proto = Mouse.prototype;

  _proto.start = function start() {
    var _this = this;

    bindEvents(window, {
      eventName: 'mousedown',
      fn: function fn(event) {
        var el = findClosestDraggerElementFromEvent(event);
        if (el === -1) return;
        var target = event.target;
        if (!hasDraggerHandlerMatched(target, _this.configs)) return; // https://stackoverflow.com/a/19164149/2006805 In order to prevent text
        // selection when moving cursor

        event.preventDefault();
        var draggerId = el.getAttribute('data-dragger-id');

        var dragger = _this.getDragger(draggerId);

        if (!dragger) return;
        var vContainer = dragger.container;
        var liftUpVDraggerIndex = vContainer.children.findIndex(dragger);

        _this.onStartHandler.start({
          dragger: dragger,
          event: event
        });

        var clone = _this.getClone();

        var output; // If dragger exists, then start to bind relative listener

        var unbind = bindEvents(window, [{
          // target should be moved by mousemove event.
          eventName: 'mousemove',
          fn: function fn(event) {
            try {
              // ts-hint: https://stackoverflow.com/questions/41110144/property-clientx-does-not-exist-on-type-event-angular2-directive
              var impactPoint = [event.clientX, event.clientY];
              event.preventDefault();
              event.stopPropagation();

              var isHomeContainer = function isHomeContainer(vContainer) {
                return vContainer ? vContainer.id === dragger.container.id : false;
              };

              var result = _this.onMoveHandler.start(_extends({
                // event,
                impactPoint: impactPoint,
                impactVDragger: dragger,
                liftUpVDragger: dragger,
                liftUpVDraggerIndex: liftUpVDraggerIndex,
                dragger: dragger,
                clone: clone,
                isHomeContainer: isHomeContainer
              }, _this.moveAPI()));

              var impact = result.impact;
              output = result.output;
              if (impact) _this.updateImpact(impact);
            } catch (err) {
              console.log('err', err);
            }
          }
        }, {
          eventName: 'mouseup',
          fn: function fn() {
            unbind();

            var _ref2 = output || {},
                dragger = _ref2.dragger;

            if (_this.dndConfig.onDrop && dragger) {
              _this.dndConfig.onDrop(output);
            }

            _this.dndEffects.teardown();

            if (clone) document.body.removeChild(clone);
          }
        }]);
      }
    });
  };

  return Mouse;
}();

var DndEffects = /*#__PURE__*/function () {
  function DndEffects() {
    this.children = {};
  }

  var _proto = DndEffects.prototype;

  _proto.find = function find(id) {
    return this.children[id];
  };

  _proto.add = function add(effectsManager) {
    var _this = this;

    var id = effectsManager.id;
    this.children[id] = effectsManager;
    return function () {
      delete _this.children[id];
    };
  };

  _proto.remove = function remove(effectsManager) {
    var id = effectsManager.id;

    if (this.children[id]) {
      this.children[id].teardown();
      delete this.children[id];
    }
  };

  _proto.teardown = function teardown() {
    for (var id in this.children) {
      var child = this.children[id];
      child.teardown();
    }
  };

  _proto.partialTeardown = function partialTeardown() {
    for (var id in this.children) {
      var child = this.children[id];
      if (!child.isHomeContainerEffects()) child.teardown();
    }
  };

  return DndEffects;
}();

var DND = /*#__PURE__*/function () {
  function DND(_ref) {
    var _this = this;

    var _ref$configs = _ref.configs,
        configs = _ref$configs === void 0 ? [] : _ref$configs,
        rootElement = _ref.rootElement,
        rest = _objectWithoutPropertiesLoose(_ref, ["configs", "rootElement"]);

    this.moveAPI = function () {
      return {
        hooks: _this.hooks,
        prevImpact: _this.impact
      };
    };

    this.updateImpact = function (impact) {
      _this.impact = impact;
    };

    this.getClone = function () {
      return _this.extra.clone;
    };

    this.getDragger = function (draggerId) {
      return _this.draggers[draggerId];
    };

    this.getContainer = function (containerId) {
      return _this.containers[containerId];
    };

    this.containers = {};
    this.draggers = {};
    this.extra = {};
    this.dndEffects = new DndEffects();
    this.configs = resolveConfig(configs, rest); // global config to control all of the containers...

    this.dndConfig = resolveDndConfig(rest);
    this.hooks = {
      syncEffects: new tapable.SyncHook(['values']),
      cleanupEffects: new tapable.SyncHook()
    };
    this.rootElement = rootElement;
    this.impact = {};
    this.sensor = null;
    this.startObserve();
    this.setUp();
    this.onStartHandler = new Sabar({
      ctx: {
        containers: this.containers,
        draggers: this.draggers,
        vDraggers: this.draggers,
        vContainers: this.containers,
        extra: this.extra,
        hooks: this.hooks,
        dndConfig: this.dndConfig,
        // containersEffects: this.containersEffects,
        prevImpact: {},
        dndEffects: new DndEffects()
      }
    });
    this.onMoveHandler = new Sabar({
      ctx: {
        containers: this.containers,
        draggers: this.draggers,
        vDraggers: this.draggers,
        vContainers: this.containers,
        // effects: this.effects,
        hooks: this.hooks,
        dndConfig: this.dndConfig,
        // containersEffects: this.containersEffects,
        dndEffects: this.dndEffects,
        prevImpact: {}
      }
    });
    this.onStartHandler.use(getDimensions, getDimensionsNested, validateContainers, // resolveRawPlacedInfo,
    attemptToCreateClone);
    this.onMoveHandler.use(syncCopyPosition, getImpactRawInfo, addIntermediateCtxValue, handleLeaveContainer, handleLeaveHomeContainer, handleLeaveOtherContainer, handleEnterContainer, handleEnterHomeContainer, handleEnterOtherContainer, handleReorder, handleReorderOnHomeContainer, handleReorderOnHomeContainer$1, handleImpactDraggerEffect, handleImpactContainerEffect, removeIntermediateCtxValue, function () {});
    this.initSensor();
  }

  var _proto = DND.prototype;

  _proto.initSensor = function initSensor() {
    this.sensor = new Mouse({
      moveAPI: this.moveAPI,
      getClone: this.getClone,
      onStartHandler: this.onStartHandler,
      onMoveHandler: this.onMoveHandler,
      getDragger: this.getDragger,
      configs: this.configs,
      dndEffects: this.dndEffects,
      updateImpact: this.updateImpact,
      dndConfig: this.dndConfig
    });
    this.sensor.start();
  };

  _proto.startObserve = function startObserve() {
    var rootElement = this.rootElement;

    if (!isElement(rootElement)) {
      var el = document.querySelector(this.rootElement);
      if (el) rootElement = el;
    }

    var observer = new MutationObserver(mutationHandler(this));
    observer.observe(rootElement, {
      childList: true,
      subtree: true
    });
  }
  /**
   * extract `containerSelectors` and onStartHandler `container` and `dragger`
   */
  ;

  _proto.setUp = function setUp() {
    this.handleContainers();
    this.handleDraggers();
  };

  _proto.handleContainers = function handleContainers(node) {
    var _this2 = this;

    if (node === void 0) {
      node = document;
    }

    this.configs.forEach(function (config) {
      var containerSelector = config.containerSelector;
      var elements = node.querySelectorAll(containerSelector);
      if (!elements) return;
      elements.forEach(function (el) {
        return _this2.handleContainerElement(el, config);
      });
    });
  };

  _proto.handleContainerElement = function handleContainerElement(el, config) {
    var keys = Object.keys(this.containers);

    for (var i = 0; i < keys.length; i++) {
      var key = keys[i];
      var containerInstance = this.containers[key];
      if (containerInstance.el === el) return;
    }

    var container = new Container({
      el: el,
      containers: this.containers,
      containerConfig: config,
      dndConfig: this.dndConfig
    });
    var parentContainerNode = closest(el, '[data-is-container="true"]');

    if (parentContainerNode) {
      var containerId = parentContainerNode.getAttribute('data-container-id');

      if (containerId) {
        var parentContainer = this.containers[containerId];
        container.parentContainer = parentContainer;
      }
    }

    setContainerAttributes(container, config);
  };

  _proto.handleDraggers = function handleDraggers(containerNode) {
    var _this3 = this;

    if (containerNode === void 0) {
      containerNode = document;
    }

    this.configs.forEach(function (config) {
      var draggerSelector = config.draggerSelector;

      _this3.handleDraggerInContainer(containerNode, draggerSelector);
    });
  };

  _proto.handleDraggerInContainer = function handleDraggerInContainer(containerNode, draggerSelector) {
    var _this4 = this;

    var elements = containerNode.querySelectorAll(draggerSelector);
    if (!elements) return;
    elements.forEach(function (el) {
      return _this4.handleDraggerElement(el);
    });
  }
  /**
   *
   * @param {HTMLElement} el
   * 1. Find closest container node first.
   * 2. Update container's children
   * 3. set dragger element's attributes.
   *
   */
  ;

  _proto.handleDraggerElement = function handleDraggerElement(el) {
    var container = findClosestContainer(this.containers, el, true);
    if (container === -1) return; // Maybe it's not required on initial...

    var keys = Object.keys(this.draggers);

    for (var i = 0; i < keys.length; i++) {
      var key = keys[i];
      var draggerInstance = this.draggers[key];
      if (draggerInstance.el === el) return;
    }

    var dragger = new Dragger({
      el: el,
      container: container
    });
    container.addDirectChild(dragger);
    var draggerId = dragger.id;
    this.draggers[draggerId] = dragger;
    setDraggerAttributes(container, dragger);
  };

  return DND;
}();

/**
 *
 * @param blockMap
 *
 * 1. If current block is in `column` direction and text is empty string...
 */

var removeBlockWithClear = function removeBlockWithClear(blockMap, block) {
  var blockToRemove;

  if (!block) {
    return blockMap;
  }

  if (typeof block === 'string') {
    blockToRemove = blockMap.get(block);
  } else {
    blockToRemove = block;
  }

  if (!blockToRemove) return blockMap;
  var parentKey = blockToRemove.getParentKey();
  var newBlockMap = removeBlock(blockMap, block);
  if (!parentKey) return newBlockMap;
  var parentBlock = newBlockMap.get(parentKey);
  var parentBlockData = parentBlock === null || parentBlock === void 0 ? void 0 : parentBlock.getData();
  var parentDirection = parentBlockData === null || parentBlockData === void 0 ? void 0 : parentBlockData.get('data-direction');
  var parentText = parentBlock === null || parentBlock === void 0 ? void 0 : parentBlock.getText();
  var parentChildrenLength = parentBlock === null || parentBlock === void 0 ? void 0 : parentBlock.getCharacterList().size;

  if (parentDirection === 'column' && !parentChildrenLength && parentText === '') {
    newBlockMap = removeBlock(newBlockMap, parentBlock);
  }

  return newBlockMap;
};

// @ts-ignore
var createEmptyBlockNode = (function () {
  var blockKey = generateRandomKey();
  return new ContentBlockNode({
    key: blockKey,
    text: '',
    data: Immutable.Map(),
    children: Immutable.List(),
    type: 'unstyled'
  });
});

/**
 *
 * @param {*} editorState
 * @param {*} sourceBlockKey
 * @param {*} targetBlockKey
 *
 * The parent of topMost block is null...
 */

var wrapBlock = function wrapBlock(originalBlockMap, targetBlockKey, direction) {
  if (direction === void 0) {
    direction = Direction.Row;
  }

  var blockMap = originalBlockMap;
  var targetBlock = blockMap.get(targetBlockKey);
  if (!targetBlock) return originalBlockMap;
  var parentBlockKey = targetBlock.getParentKey();
  var parentBlock = blockMap.get(parentBlockKey);
  var parentData = Immutable.Map();

  if (parentBlock) {
    parentData = parentBlock.getData();
  }

  if (!parentData || !parentData.get('flexRow')) {
    var containerBlock = createEmptyBlockNode().merge({
      data: {
        flexRow: true,
        'data-wrapper': true,
        'data-direction': direction
      },
      children: Immutable.List([targetBlockKey]),
      parent: parentBlockKey
    });
    var containerBlockKey = containerBlock.getKey();

    if (parentBlock) {
      var parentChildrenList = parentBlock.getChildKeys(); // TODO: splitBlockInContentState.js

      var newChildrenArray = parentChildrenList.toArray();
      var insertionIndex = newChildrenArray.indexOf(targetBlockKey);
      newChildrenArray.splice(insertionIndex, 0, targetBlockKey);
      var parentKey = parentBlock.getKey();
      blockMap = blockMap.set(parentKey, parentBlock.merge({
        children: Immutable.List(newChildrenArray)
      }));
    }

    var blocksBefore = BlockUtil.blocksBefore(blockMap, targetBlock);
    var blocksAfter = BlockUtil.blocksAfter(blockMap, targetBlock);
    blockMap = blocksBefore.concat([[containerBlockKey, containerBlock], [targetBlockKey, targetBlock]]).concat(blocksAfter).toOrderedMap();
    return blockMap.withMutations(function (blocks) {
      BlockMutationUtil.transformBlock(containerBlockKey, blocks, function (block) {
        return block.merge({
          prevSibling: targetBlock.getPrevSiblingKey(),
          nextSibling: targetBlock.getNextSiblingKey()
        });
      });
      BlockMutationUtil.transformBlock(targetBlock.getPrevSiblingKey(), blocks, function (block) {
        return block.merge({
          nextSibling: containerBlockKey
        });
      });
      BlockMutationUtil.transformBlock(targetBlock.getNextSiblingKey(), blocks, function (block) {
        return block.merge({
          prevSibling: containerBlockKey
        });
      });
      BlockMutationUtil.transformBlock(targetBlockKey, blocks, function (block) {
        return block.merge({
          prevSibling: null,
          nextSibling: null,
          parent: containerBlock.getKey()
        });
      });
    });
  }

  return originalBlockMap;
};

/**
 * If target block has children, childBlock should be placed after all
 * of its children.
 */

function appendChild(blockMap, parentBlock, childBlock) {
  var blocksBefore = BlockUtil.blocksBefore(blockMap, parentBlock);
  var childBlockKey = childBlock.getKey();
  var parentBlockKey = parentBlock.getKey();
  var childrenBlocks = BlockUtil.getChildrenBlocks(blockMap, parentBlock);
  var lastBlock = childrenBlocks.size ? childrenBlocks.last() : parentBlock;
  var blocksAfter = BlockUtil.blocksAfter(blockMap, lastBlock);
  var newBlockMap = blocksBefore.concat([[parentBlock.getKey(), parentBlock]]).concat(childrenBlocks.size ? childrenBlocks.toSeq() : []).concat([[childBlock.getKey(), childBlock]]).concat(blocksAfter).toOrderedMap();
  return newBlockMap.withMutations(function (blocks) {
    BlockMutationUtil.transformBlock(childBlockKey, blocks, function (block) {
      return block.merge({
        parent: parentBlockKey,
        prevSibling: childrenBlocks.size ? lastBlock.getKey() : null
      });
    });

    if (childrenBlocks.size) {
      BlockMutationUtil.transformBlock(lastBlock.getKey(), blocks, function (block) {
        return block.merge({
          nextSibling: childBlockKey
        });
      });
    }

    BlockMutationUtil.transformBlock(parentBlockKey, blocks, function (block) {
      var parentChildrenList = block.getChildKeys();
      var newChildrenArray = parentChildrenList.toArray();
      newChildrenArray.push(childBlockKey);
      return block.merge({
        children: Immutable.List(newChildrenArray)
      });
    });
  });
}

var InsertChildBlockDirection;

(function (InsertChildBlockDirection) {
  InsertChildBlockDirection["Forward"] = "forward";
  InsertChildBlockDirection["Backward"] = "backward";
})(InsertChildBlockDirection || (InsertChildBlockDirection = {}));
/**
 * If target block has children, childBlock should be placed after all
 * of its children.
 */


function insertChildBlock(_ref) {
  var blockMap = _ref.blockMap,
      parentBlock = _ref.parentBlock,
      indexBlock = _ref.indexBlock,
      childBlock = _ref.childBlock,
      direction = _ref.direction;
  var indexParentKey = indexBlock.getParentKey();
  var parentBlockKey = parentBlock.getKey();

  if (indexParentKey !== parentBlockKey) {
    throw new Error('in `insertChild` function, `indexParentKey` and `parentBlockKey` should be the same');
  }

  var blocksBefore = BlockUtil.blocksBefore(blockMap, indexBlock);
  var blocksAfter = BlockUtil.blocksAfter(blockMap, indexBlock);
  var childBlockKey = childBlock.getKey();
  var indexBlockKey = indexBlock.getKey();
  var newBlockMap;

  if (direction === 'forward') {
    newBlockMap = blocksBefore.concat([[childBlock.getKey(), childBlock]]).concat([[indexBlockKey, indexBlock]]).concat(blocksAfter).toOrderedMap();
  } else {
    var indexBlockChildren = BlockUtil.getChildrenBlocks(blockMap, indexBlock);
    var indexBlockChildrenLast = indexBlockChildren.size ? indexBlockChildren.last() : parentBlock;

    var _blocksAfter = BlockUtil.blocksAfter(blockMap, indexBlockChildrenLast);

    newBlockMap = blocksBefore.concat([[indexBlockKey, indexBlock]]).concat(indexBlockChildren.size ? indexBlockChildren.toSeq() : []).concat([[childBlock.getKey(), childBlock]]).concat(_blocksAfter).toOrderedMap();
  }

  return newBlockMap.withMutations(function (blocks) {
    if (direction === 'forward') {
      BlockMutationUtil.transformBlock(childBlockKey, blocks, function (block) {
        return block.merge({
          parent: parentBlockKey,
          prevSibling: indexBlock ? indexBlock.getPrevSiblingKey() : null,
          nextSibling: indexBlock ? indexBlockKey : null
        });
      });
      BlockMutationUtil.transformBlock(indexBlockKey, blocks, function (block) {
        return block.merge({
          prevSibling: childBlockKey
        });
      });
    } else {
      BlockMutationUtil.transformBlock(childBlockKey, blocks, function (block) {
        return block.merge({
          parent: parentBlockKey,
          prevSibling: indexBlock ? indexBlockKey : null,
          nextSibling: indexBlock ? indexBlock.getNextSiblingKey() : null
        });
      });
      BlockMutationUtil.transformBlock(indexBlockKey, blocks, function (block) {
        return block.merge({
          nextSibling: childBlockKey
        });
      });
    }

    BlockMutationUtil.transformBlock(parentBlockKey, blocks, function (block) {
      var parentChildrenList = block.getChildKeys();
      var newChildrenArray = parentChildrenList.toArray();
      var index = newChildrenArray.indexOf(indexBlockKey);

      if (direction === 'forward') {
        newChildrenArray.splice(index, 0, childBlockKey);
      } else {
        newChildrenArray.splice(index + 1, 0, childBlockKey);
      }

      return block.merge({
        children: Immutable.List(newChildrenArray)
      });
    });
  });
}

var horizontalTransfer = function horizontalTransfer(editorState, sourceBlockKey, targetBlockKey, direction) {
  var containerBlock = createEmptyBlockNode().merge({
    data: {
      flexRow: true,
      'data-wrapper': true,
      'data-direction': 'column'
    },
    children: Immutable.List([]),
    parent: null
  });
  var containerBlockKey = containerBlock.getKey(); // remove the moved block

  var currentState = editorState.getCurrentContent();
  var blockMap = currentState.getBlockMap();
  var sourceBlock = blockMap.get(sourceBlockKey);
  blockMap = removeBlockWithClear(blockMap, sourceBlockKey);
  var sourceBlockWithSibling = sourceBlock === null || sourceBlock === void 0 ? void 0 : sourceBlock.merge({
    prevSibling: null,
    nextSibling: null
  });
  var targetBlock = blockMap.get(targetBlockKey);
  var targetBlockParentKey = targetBlock === null || targetBlock === void 0 ? void 0 : targetBlock.getParentKey();
  var parentBlockData = Immutable.Map();
  var targetParentBlock;

  if (targetBlockParentKey) {
    targetParentBlock = blockMap.get(targetBlockParentKey);
    parentBlockData = targetParentBlock.getData();
  }

  if (parentBlockData.get('data-direction') === 'row' && targetParentBlock) {
    blockMap = insertChildBlock({
      blockMap: blockMap,
      parentBlock: targetParentBlock,
      indexBlock: targetBlock,
      childBlock: containerBlock,
      direction: direction === 'left' ? InsertChildBlockDirection.Forward : InsertChildBlockDirection.Backward
    }); // should use blockMap.get(containerBlockKey)! to get latest `containerBlock`

    blockMap = appendChild(blockMap, blockMap.get(containerBlockKey), sourceBlockWithSibling);
  } else {
    var _blockMap$get;

    blockMap = wrapBlock(blockMap, targetBlockKey, Direction.Column);
    var parentKey = blockMap.get(targetBlockKey).getParentKey();
    blockMap = wrapBlock(blockMap, parentKey, Direction.Row);
    var grandParentKey = (_blockMap$get = blockMap.get(parentKey)) === null || _blockMap$get === void 0 ? void 0 : _blockMap$get.getParentKey();
    blockMap = insertChildBlock({
      blockMap: blockMap,
      parentBlock: blockMap.get(grandParentKey),
      indexBlock: blockMap.get(parentKey),
      childBlock: containerBlock,
      direction: direction === 'left' ? InsertChildBlockDirection.Forward : InsertChildBlockDirection.Backward
    });
    var parentBlock = blockMap.get(containerBlock.getKey());

    if (parentBlock && sourceBlock) {
      blockMap = appendChild(blockMap, parentBlock, sourceBlockWithSibling);
    }
  }

  return blockMap;
};

function insertBlockBefore(blockMap, targetBlock, sourceBlock) {
  if (!targetBlock || !sourceBlock) return blockMap;
  var blocksBefore = BlockUtil.blocksBefore(blockMap, targetBlock);
  var blocksAfter = BlockUtil.blocksAfter(blockMap, targetBlock);
  var sourceBlockKey = sourceBlock.getKey();
  var targetBlockKey = targetBlock.getKey();
  var targetParentKey = targetBlock.getParentKey();
  var targetOriginPrevSibling = targetBlock.getPrevSiblingKey();
  var targetOriginNextSibling = targetBlock.getNextSiblingKey();
  var newBlockMap = blocksBefore.concat([[sourceBlockKey, sourceBlock], [targetBlockKey, targetBlock]]).concat(blocksAfter).toOrderedMap();
  return newBlockMap.withMutations(function (blocks) {
    BlockMutationUtil.transformBlock(sourceBlockKey, blocks, function (block) {
      return block.merge({
        prevSibling: targetOriginPrevSibling,
        nextSibling: targetBlockKey
      });
    });
    BlockMutationUtil.transformBlock(targetBlock.getPrevSiblingKey(), blocks, function (block) {
      return block.merge({
        nextSibling: sourceBlockKey
      });
    });
    BlockMutationUtil.transformBlock(targetBlockKey, blocks, function (block) {
      return block.merge({
        prevSibling: sourceBlockKey,
        nextSibling: targetOriginNextSibling
      });
    });
    BlockMutationUtil.transformBlock(targetParentKey, blocks, function (block) {
      var parentChildrenList = block.getChildKeys();
      var newChildrenArray = parentChildrenList.toArray();
      var index = newChildrenArray.indexOf(targetBlockKey);
      newChildrenArray.splice(index, 0, sourceBlockKey);
      return block.merge({
        children: Immutable.List(newChildrenArray)
      });
    });
  });
}

/**
 * If target block has children, sourceBlock should be placed after all
 * of its children.
 */

function insertBlockAfter(blockMap, targetBlock, sourceBlock) {
  if (!sourceBlock || !targetBlock) return blockMap;
  var blocksBefore = BlockUtil.blocksBefore(blockMap, targetBlock);
  var sourceBlockKey = sourceBlock.getKey();
  var targetBlockKey = targetBlock.getKey();
  var childrenBlocks = BlockUtil.getChildrenBlocks(blockMap, targetBlock);
  var lastBlock = childrenBlocks.size ? childrenBlocks.last() : targetBlock;
  var blocksAfter = BlockUtil.blocksAfter(blockMap, lastBlock);
  var newBlockMap = blocksBefore.concat([[targetBlock.getKey(), targetBlock]]).concat(childrenBlocks.size ? childrenBlocks.toSeq() : []).concat([[sourceBlock.getKey(), sourceBlock]]).concat(blocksAfter).toOrderedMap();
  return newBlockMap.withMutations(function (blocks) {
    BlockMutationUtil.transformBlock(sourceBlockKey, blocks, function (block) {
      return block.merge({
        prevSibling: targetBlockKey,
        nextSibling: targetBlock.getNextSiblingKey(),
        parent: targetBlock.getParentKey()
      });
    });
    BlockMutationUtil.transformBlock(targetBlock.getParentKey(), blocks, function (block) {
      var parentChildrenList = block.getChildKeys();
      var newChildrenArray = parentChildrenList.toArray();
      newChildrenArray.push(sourceBlockKey);
      return block.merge({
        children: Immutable.List(newChildrenArray)
      });
    });
    BlockMutationUtil.transformBlock(targetBlockKey, blocks, function (block) {
      return block.merge({
        nextSibling: sourceBlockKey
      });
    });
  });
}

var verticalTransfer = function verticalTransfer(editorState, sourceBlockKey, targetBlockKey, position) {
  var currentState = editorState.getCurrentContent();
  var blockMap = currentState.getBlockMap();
  var sourceBlock = blockMap.get(sourceBlockKey);
  blockMap = removeBlockWithClear(blockMap, sourceBlockKey); // Note: targetBlock should be placed after `removeBlockWithClear`, because
  // its sibling has changed after `removeBlockWithClear`; `hooks.setState` may
  // cause loop due to the same `prevSibling` and `nextSibling`..

  var targetBlock = blockMap.get(targetBlockKey);

  if (position === 'top') {
    return insertBlockBefore(blockMap, targetBlock, sourceBlock);
  }

  if (position === 'bottom') {
    return insertBlockAfter(blockMap, targetBlock, sourceBlock);
  }

  return blockMap;
};

var requiredError = function requiredError(prop) {
  throw new Error(prop + " is required in transfer function");
};

var transfer = function transfer(editorState, sourceBlockKey, targetBlockKey, position) {
  if (!editorState) requiredError('editorState');
  if (!sourceBlockKey) requiredError('sourceBlockKey');
  if (!targetBlockKey) requiredError('targetBlockKey');
  if (!position) requiredError('position');
  var currentState = editorState.getCurrentContent();
  var blockMap;

  switch (position) {
    case 'top':
      blockMap = verticalTransfer(editorState, sourceBlockKey, targetBlockKey, Position.Top);
      break;

    case 'right':
      blockMap = horizontalTransfer(editorState, sourceBlockKey, targetBlockKey, Position.Right);
      break;

    case 'bottom':
      blockMap = verticalTransfer(editorState, sourceBlockKey, targetBlockKey, Position.Bottom);
      break;

    case 'left':
      blockMap = horizontalTransfer(editorState, sourceBlockKey, targetBlockKey, Position.Left);
      break;
  }

  return currentState.merge({
    blockMap: blockMap
  });
};

function DNDPlugin() {
  var verticalIndicator;
  var horizontalIndicator;

  var createIndicatorBar = function createIndicatorBar() {
    verticalIndicator = document.createElement('div');
    document.body.appendChild(verticalIndicator);
    horizontalIndicator = document.createElement('div');
    document.body.appendChild(horizontalIndicator);
  };

  var logger = function logger(_ref) {
    var dragger = _ref.dragger,
        candidateDragger = _ref.candidateDragger,
        container = _ref.container;
    var draggerOffsetKey = dragger.getAttribute('data-offset-key');
    var candidateDraggerOffsetKey = candidateDragger.getAttribute('data-offset-key');
    var containerOffsetKey = container.getAttribute('data-offset-key');
  };

  this.apply = function (getEditor) {
    createIndicatorBar();

    var _getEditor = getEditor(),
        hooks = _getEditor.hooks;

    hooks.afterMounted.tap('initDNDPlugin', function () {
      new DND({
        onDrop: function onDrop(_ref2) {
          var dragger = _ref2.dragger,
              candidateDragger = _ref2.candidateDragger,
              placedPosition = _ref2.placedPosition;

          try {
            var draggerOffsetKey = dragger.getAttribute('data-offset-key') || '';
            var candidateDraggerOffsetKey = candidateDragger.getAttribute('data-offset-key') || '';
            var sourceBlockKey = extractBlockKeyFromOffsetKey(draggerOffsetKey);
            var targetBlockKey = extractBlockKeyFromOffsetKey(candidateDraggerOffsetKey);

            var _getEditor2 = getEditor(),
                editorState = _getEditor2.editorState,
                _hooks = _getEditor2.hooks;

            var newContent = transfer(editorState, sourceBlockKey, targetBlockKey, placedPosition);
            if (!newContent) return;
            var dismissSelection = draftJs.EditorState.push(editorState, newContent, 'insert-characters');

            _hooks.setState.call(dismissSelection);
          } catch (err) {
            console.log('handle error ', err);
          }
        },
        rootElement: '.DraftEditor-root',
        mode: Mode.Nested,
        draggerHandlerSelector: '.sidebar-addon-visible',
        withPlaceholder: false,
        configs: [{
          containerSelector: '[data-contents="true"]',
          draggerSelector: '[data-contents="true"] >.miuffy-paragraph',
          impactDraggerEffect: function impactDraggerEffect(options) {
            var dimension = options.dimension,
                placedPosition = options.placedPosition;
            var top = dimension.top,
                right = dimension.right,
                left = dimension.left,
                _dimension$height = dimension.height,
                height = _dimension$height === void 0 ? 0 : _dimension$height;
            logger(options);
            requestAnimationFrame(function () {
              if (placedPosition === 'top') {
                horizontalIndicator.style.top = top + "px";
              } else {
                horizontalIndicator.style.top = top + height + "px";
              }

              horizontalIndicator.style.position = 'absolute';
              horizontalIndicator.style.width = right - left + "px";
              horizontalIndicator.style.height = "3px";
              horizontalIndicator.style.left = left - 10 + "px";
              horizontalIndicator.style.backgroundColor = '#722ed1';
              horizontalIndicator.style.opacity = '1';
              horizontalIndicator.style.transition = 'opacity 1000ms ease-in';
            });
            return function () {
              verticalIndicator.style.removeProperty('transition');
              horizontalIndicator.style.position = 'absolute';
              horizontalIndicator.style.width = '0px';
              horizontalIndicator.style.height = "0px";
              horizontalIndicator.style.top = "0px";
              horizontalIndicator.style.left = "0px";
              horizontalIndicator.style.opacity = '0';
              horizontalIndicator.style.backgroundColor = 'transparent';
            };
          }
        }, {
          orientation: Orientation.Horizontal,
          containerSelector: '[data-contents="true"] >div.miuffy-paragraph',
          draggerSelector: '.miuffy-paragraph >div:first-child',
          shouldAcceptDragger: function shouldAcceptDragger(el) {
            return el.matches('.miuffy-paragraph') || el.matches('.miuffy-paragraph >div:first-child');
          },
          impactDraggerEffect: function impactDraggerEffect(options) {
            var dimension = options.dimension,
                placedPosition = options.placedPosition;
            var top = dimension.top,
                bottom = dimension.bottom,
                left = dimension.left,
                right = dimension.right;
            logger(options);

            if (placedPosition === 'left') {
              verticalIndicator.style.left = left - 5 + "px";
            } else {
              verticalIndicator.style.left = right + 5 + "px";
            }

            requestAnimationFrame(function () {
              verticalIndicator.style.position = 'absolute';
              verticalIndicator.style.width = '3px';
              verticalIndicator.style.height = bottom - top + "px";
              verticalIndicator.style.top = top + "px";
              verticalIndicator.style.backgroundColor = '#69c0ff';
              verticalIndicator.style.opacity = '1';
              verticalIndicator.style.transition = 'opacity 250ms ease-in';
              verticalIndicator.style.zIndex = '1';
            });
            return function () {
              verticalIndicator.style.removeProperty('transition');
              verticalIndicator.style.position = 'absolute';
              verticalIndicator.style.width = '0px';
              verticalIndicator.style.height = "0px";
              verticalIndicator.style.top = "0px";
              verticalIndicator.style.left = "0px";
              verticalIndicator.style.opacity = '0';
              verticalIndicator.style.backgroundColor = 'transparent';
            };
          },
          impactContainerEffect: function impactContainerEffect(options) {
            var container = options.container;
            container.style.backgroundColor = '#fa8c16';
            return function () {
              container.style.backgroundColor = 'transparent';
            };
          }
        }, {
          orientation: Orientation.Vertical,
          containerSelector: '.data-wrapper-column >div',
          draggerSelector: '.miuffy-paragraph',
          shouldAcceptDragger: function shouldAcceptDragger(el) {
            return el.matches('.miuffy-paragraph') || el.matches('.miuffy-paragraph >div:first-child');
          },
          impactDraggerEffect: function impactDraggerEffect(options) {
            var dimension = options.dimension,
                placedPosition = options.placedPosition;
            var top = dimension.top,
                bottom = dimension.bottom,
                left = dimension.left,
                right = dimension.right;
            logger(options);

            if (placedPosition === 'top') {
              verticalIndicator.style.top = top - 5 + "px";
            } else {
              verticalIndicator.style.top = bottom + 5 + "px";
            }

            requestAnimationFrame(function () {
              verticalIndicator.style.position = 'absolute';
              verticalIndicator.style.width = right - left + "px";
              verticalIndicator.style.height = '3px';
              verticalIndicator.style.left = left + "px";
              verticalIndicator.style.backgroundColor = '#69c0ff';
              verticalIndicator.style.opacity = '1';
              verticalIndicator.style.transition = 'opacity 250ms ease-in';
            });
            return function () {
              verticalIndicator.style.removeProperty('transition');
              verticalIndicator.style.position = 'absolute';
              verticalIndicator.style.width = '0px';
              verticalIndicator.style.height = "0px";
              verticalIndicator.style.top = "0px";
              verticalIndicator.style.left = "0px";
              verticalIndicator.style.opacity = '0';
              verticalIndicator.style.backgroundColor = 'transparent';
            };
          },
          impactContainerEffect: function impactContainerEffect(options) {
            var container = options.container;
            container.style.background = '#ff4d4f';
            return function () {
              container.style.background = '#fff';
            };
          }
        }]
      });
    });
  };
}

function FinalNewLinePlugin() {
  this.apply = function (getEditor) {
    var _getEditor = getEditor(),
        hooks = _getEditor.hooks;

    hooks.finalNewLine.tap('finalNewLine', function (editorState) {
      var currentState = editorState.getCurrentContent();
      var blockMap = currentState.getBlockMap();
      var last = blockMap.last();
      var type = last.getType();
      var parent = last.getParentKey();

      if (parent
      /** the last block is not the top-most component */
      || type !== 'unstyled') {
        var emptyBlock = createEmptyBlockNode();
        var key = emptyBlock.getKey();
        var lastBlockWithNullParent = blockMap.reverse().toSeq().skipUntil(function (block) {
          return !block.getParentKey();
        }).first();

        if (lastBlockWithNullParent) {
          var newBlock = emptyBlock.merge({
            prevSibling: lastBlockWithNullParent.getKey()
          });
          var newBlockMap = blockMap.set(lastBlockWithNullParent.getKey(), lastBlockWithNullParent.merge({
            nextSibling: key
          }));
          newBlockMap = newBlockMap.toSeq().concat([[key, newBlock]]).toOrderedMap();
          var newContent = currentState.set('blockMap', newBlockMap);
          return draftJs.EditorState.push(editorState, newContent, 'insert-characters');
        }
      }

      return editorState;
    });
  };
}

/**
 * depth is used for `blockStyleFnPlugin`
 */

var UpdateBlockDepthData = function UpdateBlockDepthData() {
  this.apply = function (getEditor) {
    var _getEditor = getEditor(),
        hooks = _getEditor.hooks;

    hooks.updateBlockDepthData.tap('updateBlockDepthData', function (editorState) {
      var currentState = editorState.getCurrentContent();
      var blockMap = currentState.getBlockMap();
      var selectionState = editorState.getSelection();
      var depthMap = Object.create(null);
      blockMap.toSeq().forEach(function (block) {
        var key = block.getKey();
        var parentKey = block.getParentKey();
        depthMap[key] = {
          directParent: parentKey,
          parents: [],
          depth: 0
        };
      });

      for (var key in depthMap) {
        var value = depthMap[key];
        var parent = value.directParent;

        while (parent) {
          value.parents.push(parent);
          parent = depthMap[parent].directParent;
        }

        value.depth = value.parents.length;
      }

      var newBlocks = blockMap.toSeq().map(function (block) {
        var data = block.getData();
        var key = block.getKey();
        var newDepth = depthMap[key].depth;
        var oldDepth = data.get('depth');
        if (newDepth === oldDepth) return block;
        return block.merge({
          data: data.merge({
            depth: depthMap[key].depth
          })
        });
      }).toOrderedMap(); // refer to https://github.com/facebook/draft-js/blob/master/src/model/transaction/modifyBlockForContentState.js#L37

      var newContent = currentState.merge({
        blockMap: newBlocks,
        selectionBefore: selectionState,
        selectionAfter: selectionState
      }); // TODO: why selection
      // The following code will cause selection issues...
      // const newBlockMap = blockMap.toSeq().map(block => {
      //   const data = block.getData();
      //   const key = block.getKey();
      //   const newDepth = depthMap[key].depth;
      //   const oldDepth = data.get('depth');
      //   if (newDepth === oldDepth) return block;
      //   return block.merge({
      //     data: data.merge({ depth: depthMap[key].depth }),
      //   });
      // }).toOrderedMap();
      // // Note: set ``
      // const newContent = (currentState as any).set('blockMap', newBlockMap);

      return draftJs.EditorState.push(editorState, newContent, 'change-block-data');
    });
  };
};

var Context = /*#__PURE__*/React.createContext(null);

// https://github.com/SamyPesse/draft-js-prism/blob/master/lib/index.js

function PrismDecorator() {
  this.highlighted = {};
}
/**
 * Return list of decoration IDs per character
 *
 * @param {ContentBlock}
 * @return {List<String>}
 */


PrismDecorator.prototype.getDecorations = function (block) {
  var tokens;
  var token;
  var tokenId;
  var resultId;
  var offset = 0;
  var tokenCount = 0;
  var blockKey = block.getKey();
  var blockType = block.getType();
  var blockData = block.getData();
  var blockText = block.getText();
  var decorations = Array(blockText.length).fill(null);
  var highlighted = this.highlighted;
  highlighted[blockKey] = {};

  if (blockType !== 'code-block') {
    return Immutable__default.List(decorations);
  }

  var syntax = blockData.get('syntax') || 'javascript'; // Allow for no syntax highlighting

  if (syntax == null) {
    return Immutable__default.List(decorations);
  } // Parse text using Prism


  var grammar = Prism.languages[syntax];
  tokens = Prism.tokenize(blockText, grammar);

  function processToken(decorations, token, offset) {
    if (typeof token === 'string') {
      return;
    } // First write this tokens full length


    tokenId = "tok" + tokenCount++;
    resultId = blockKey + "-" + tokenId;
    highlighted[blockKey][tokenId] = token;
    occupySlice(decorations, offset, offset + token.length, resultId); // Then recurse through the child tokens, overwriting the parent

    var childOffset = offset;

    if (Array.isArray(token.content)) {
      var content = token.content;

      for (var i = 0; i < content.length; i++) {
        var childToken = content[i];
        processToken(decorations, childToken, childOffset);
        childOffset += childToken.length;
      }
    }
  }

  for (var i = 0; i < tokens.length; i++) {
    token = tokens[i];
    processToken(decorations, token, offset);
    offset += token.length;
  }

  return Immutable__default.List(decorations);
};
/**
 * Return component to render a decoration
 *
 * @param {String}
 * @return {Function}
 */


PrismDecorator.prototype.getComponentForKey = function () {
  var Component = function Component(props) {
    var type = props.type,
        children = props.children;
    var className = "prism-token token " + type;
    return React__default.createElement("span", {
      className: className
    }, children);
  };

  return Component;
};
/**
 * Return props to render a decoration
 *
 * @param {String}
 * @return {Object}
 */


PrismDecorator.prototype.getPropsForKey = function (key) {
  var parts = key.split('-');
  var blockKey = parts[0];
  var tokId = parts[1];
  var token = this.highlighted[blockKey][tokId];
  return {
    type: token.type
  };
};

function occupySlice(targetArr, start, end, componentKey) {
  for (var ii = start; ii < end; ii++) {
    targetArr[ii] = componentKey;
  }
}

var KEY_SEPARATOR = '-'; // https://stackoverflow.com/questions/52431074/how-to-solve-this-implicitly-has-type-any-when-typescript-checking-classic

function MultiDecorator(decorators) {
  this.decorators = Immutable.List(decorators);
}
/**
    Return list of decoration IDs per character

    @param {ContentBlock}
    @return {List<String>}
*/


MultiDecorator.prototype.getDecorations = function (block, contentState) {
  var decorations = Array(block.getText().length).fill(null);
  this.decorators.forEach(function (decorator, i) {
    var _decorations = decorator.getDecorations(block, contentState);

    _decorations.forEach(function (key, offset) {
      if (!key) {
        return;
      }

      key = i + KEY_SEPARATOR + key;
      decorations[offset] = key;
    });
  });
  return Immutable__default.List(decorations);
};
/**
    Return component to render a decoration

    @param {String}
    @return {Function}
*/


MultiDecorator.prototype.getComponentForKey = function (key) {
  var decorator = this.getDecoratorForKey(key);
  return decorator.getComponentForKey(this.getInnerKey(key));
};
/**
    Return props to render a decoration

    @param {String}
    @return {Object}
*/


MultiDecorator.prototype.getPropsForKey = function (key) {
  var decorator = this.getDecoratorForKey(key);
  return decorator.getPropsForKey(this.getInnerKey(key));
};
/**
    Return a decorator for a specific key

    @param {String}
    @return {Decorator}
*/


MultiDecorator.prototype.getDecoratorForKey = function (key) {
  var parts = key.split(KEY_SEPARATOR);
  var index = Number(parts[0]);
  return this.decorators.get(index);
};
/**
    Return inner key for a decorator

    @param {String}
    @return {String}
*/


MultiDecorator.prototype.getInnerKey = function (key) {
  var parts = key.split(KEY_SEPARATOR);
  return parts.slice(1).join(KEY_SEPARATOR);
};

function getDisplayName(WrappedComponent) {
  return WrappedComponent.displayName || WrappedComponent.name || 'WithEditorComponent';
} // ts-hint: refer to https://medium.com/@jrwebdev/react-higher-order-component-patterns-in-typescript-42278f7590fb
// and take note on `Subtract` method...


function withEditor( // ts-hint: according to WrappedComponent type. it will implicate the generic type
// of `T` which exclude `getEditor` props
WrappedComponent) {
  return function (props) {
    WrappedComponent.displayName = "WrappedComponent(" + getDisplayName(WrappedComponent) + ")";
    var getEditor = React.useContext(Context);
    return React__default.createElement(WrappedComponent, Object.assign({}, props, {
      getEditor: getEditor
    }));
  };
}

var Title = function Title() {
  return React__default.createElement("div", {
    className: "article-title"
  }, React__default.createElement("input", {
    className: "title-input",
    placeholder: "Untitled"
  }));
};

var Title$1 = /*#__PURE__*/withEditor(Title);

var withFillColor = (function (WrappedComponent) {
  return (// memo(
    function (props) {
      var active = props.active,
          rest = _objectWithoutPropertiesLoose(props, ["active"]);

      var fill = active ? '#34e79a' : '#fff';
      return React__default.createElement(WrappedComponent, Object.assign({}, rest, {
        fill: fill
      }));
    }
  );
}); //   (next: Omit<Props, 'fill'>, prev: Omit<Props, 'fill'>) => next.active === prev.active
// );

var withAction = (function (WrappedComponent) {
  return function (props) {
    var onClick = props.onClick,
        rest = _objectWithoutPropertiesLoose(props, ["onClick"]);

    var handleClick = React.useCallback(function () {
      if (typeof onClick === 'function') onClick();
    }, [onClick]); // 之所以要有这个handler，因为在点击`inlineToolbar`上的按钮时，它会清掉selection；
    // 有人发现是`onMouseDown`造成的问题；具体参考
    // https://github.com/facebook/draft-js/issues/696#issuecomment-302903086

    var onMouseDownHandler = React.useCallback(function (e) {
      e.preventDefault();
    }, []);
    return React__default.createElement("button", {
      onClick: handleClick,
      onMouseDown: onMouseDownHandler,
      className: "icon-button icon-wrapper"
    }, React__default.createElement(WrappedComponent, Object.assign({}, rest)));
  };
}); // // <Props extends WithAction, WithActionProps>
// function withAction<T> (
//   WrappedComponent: ComponentType<IWithActionProps<T> & {
//     onClick: () => void
//   }>
// ): FC<IWithActionProps<T>> {
//   return (props: T) => {
//     const { onClick, ...rest } = props;
//     const handleClick = useCallback(() => {
//       if (typeof onClick === 'function') onClick();
//     }, [onClick]);
//     // 之所以要有这个handler，因为在点击`inlineToolbar`上的按钮时，它会清掉selection；
//     // 有人发现是`onMouseDown`造成的问题；具体参考
//     // https://github.com/facebook/draft-js/issues/696#issuecomment-302903086
//     const onMouseDownHandler = useCallback(e => {
//       e.preventDefault();
//     }, []);
//     return (
//       <button
//         onClick={handleClick}
//         onMouseDown={onMouseDownHandler}
//         className="icon-button icon-wrapper"
//       >
//         <WrappedComponent {...rest} />
//       </button>
//     );
// }

var ImageAlignCenter = function ImageAlignCenter(_ref) {
  var fill = _ref.fill;
  return React__default.createElement("svg", {
    width: "25",
    height: "25",
    fill: fill
  }, React__default.createElement("path", {
    d: "M3,21 L21,21 L21,19 L3,19 L3,21 Z M3,3 L3,5 L21,5 L21,3 L3,3 Z M5,7 L5,17 L19,17 L19,7 L5,7 Z"
  }), React__default.createElement("path", {
    d: "M0 0h24v24H0z",
    fill: "none"
  }));
}; // export default withAction(ImageAlignCenter)


var ImageAlignCenter$1 = /*#__PURE__*/withFillColor( /*#__PURE__*/withAction(ImageAlignCenter)); // interface FooProps {
//   foo: string;
// }
// interface BarProps {
//   bar: string;
// }
// export const withFoo = <Props extends FooProps>(
//   Component: React.ComponentType<Props>
// ): React.ComponentType<Omit<Props, keyof FooProps>> => props => (
//   <Component {...props as Props} foo="foo" />
// );
// export const withBar = <Props extends BarProps>(
//   Component: React.ComponentType<Props>
// ): React.ComponentType<Omit<Props, keyof BarProps>> => props => (
//   <Component {...props as Props} bar="bar" />
// );
// const WrappedComponent = ({ bar, foo, more }) => {
//   console.log(more);
//   console.log(bar);
//   console.log(foo);
//   return <div>woohoo</div>;
// };
// const EnhancedComponent = withFoo(withBar(WrappedComponent));

var ImageAlignLeftFillContent = function ImageAlignLeftFillContent(_ref) {
  var fill = _ref.fill;
  return React__default.createElement("svg", {
    width: "25",
    height: "25",
    fill: fill
  }, React__default.createElement("path", {
    d: "M21,15 L15,15 L15,17 L21,17 L21,15 Z M21,7 L15,7 L15,9 L21,9 L21,7 Z M15,13 L21,13 L21,11 L15,11 L15,13 Z M3,21 L21,21 L21,19 L3,19 L3,21 Z M3,3 L3,5 L21,5 L21,3 L3,3 Z M3,7 L3,17 L13,17 L13,7 L3,7 Z"
  }), React__default.createElement("path", {
    d: "M0 0h24v24H0z",
    fill: "none"
  }));
};

var ImageAlignLeftFillContent$1 = /*#__PURE__*/withFillColor( /*#__PURE__*/withAction(ImageAlignLeftFillContent));

var ImageAlignRightFillContent = function ImageAlignRightFillContent(_ref) {
  var fill = _ref.fill;
  return React__default.createElement("svg", {
    width: "25",
    height: "25",
    fill: fill
  }, React__default.createElement("path", {
    d: "M9,15 L3,15 L3,17 L9,17 L9,15 Z M9,7 L3,7 L3,9 L9,9 L9,7 Z M3,13 L9,13 L9,11 L3,11 L3,13 Z M3,21 L21,21 L21,19 L3,19 L3,21 Z M3,3 L3,5 L21,5 L21,3 L3,3 Z M11,7 L11,17 L21,17 L21,7 L11,7 Z"
  }), React__default.createElement("path", {
    d: "M0 0h24v24H0z",
    fill: "none"
  }));
};

var ImageAlignRightFillContent$1 = /*#__PURE__*/withFillColor( /*#__PURE__*/withAction(ImageAlignRightFillContent));

var ImageAlignCenterButton = function ImageAlignCenterButton(_ref) {
  var activeKey = _ref.activeKey,
      clickHandler = _ref.clickHandler,
      active = _ref.active;

  var handleClick = function handleClick() {
    return clickHandler(activeKey);
  };

  return React__default.createElement(ImageAlignCenter$1, {
    active: active,
    onClick: handleClick
  });
};

var ImageAlignLeftFillContentButton = function ImageAlignLeftFillContentButton(_ref2) {
  var activeKey = _ref2.activeKey,
      clickHandler = _ref2.clickHandler,
      active = _ref2.active;

  var handleClick = function handleClick() {
    return clickHandler(activeKey);
  };

  return React__default.createElement(ImageAlignLeftFillContent$1, {
    active: active,
    onClick: handleClick
  });
};

var ImageAlignRightFillContentButton = function ImageAlignRightFillContentButton(_ref3) {
  var activeKey = _ref3.activeKey,
      clickHandler = _ref3.clickHandler,
      active = _ref3.active;

  var handleClick = function handleClick() {
    return clickHandler(activeKey);
  };

  return React__default.createElement(ImageAlignRightFillContent$1, {
    active: active,
    onClick: handleClick
  });
};

var ImageToolbar = function ImageToolbar(props) {
  var forwardRef = props.forwardRef,
      getEditor = props.getEditor;

  var _getEditor = getEditor(),
      hooks = _getEditor.hooks;

  var _useState = React.useState(),
      alignment = _useState[0],
      setAlignment = _useState[1];

  var blockRef = React.useRef();

  var clickHandler = function clickHandler(alignment) {
    var entityKey = blockRef.current.getEntityAt(0);

    if (entityKey) {
      var _getEditor2 = getEditor(),
          editorState = _getEditor2.editorState;

      var contentState = editorState.getCurrentContent();
      var resizeLayout = {};

      switch (alignment) {
        case Alignment.Center:
          resizeLayout.width = '900px';
          break;

        case Alignment.Right:
          resizeLayout.width = '450px';
          break;

        case Alignment.Left:
          resizeLayout.width = '450px';
          break;

        case Alignment.LeftFill:
          resizeLayout.width = '450px';
          break;

        case Alignment.RightFill:
          resizeLayout.width = '450px';
          break;
      } // 设置alignment的同时，要设置resizeLayout;否则比如插入一个图片，然后resize图片，这个
      // 再点击alignment中的值的话，输入问题，它的width会变成刚刚resize时的值；


      var newContent = contentState.mergeEntityData(entityKey, {
        alignment: alignment,
        resizeLayout: resizeLayout
      });
      var nextState = draftJs.EditorState.push(editorState, newContent, 'insert-fragment');
      var newState = draftJs.EditorState.forceSelection(nextState, nextState.getSelection());
      hooks.setState.call(newState, function (editorState) {
        var contentState = editorState.getCurrentContent();
        var entity = blockRef.current.getEntityAt(0);
        if (!entity) return null;
        var entityState = contentState.getEntity(entity);
        var type = entityState.getType();
        var data = entityState.getData();

        if (type === 'IMAGE') {
          var _alignment = data.alignment;
          if (_alignment) setAlignment(_alignment);
        }

        return null;
      });
    }
  };

  React.useEffect(function () {
    hooks.toggleImageToolbarVisible.tap('ImageToolbar', function (visibility, block) {
      if (visibility) {
        blockRef.current = block;

        var _getEditor3 = getEditor(),
            editorState = _getEditor3.editorState;

        var contentState = editorState.getCurrentContent();
        var entity = blockRef.current.getEntityAt(0);
        if (!entity) return null;
        var entityState = contentState.getEntity(entity);
        var type = entityState.getType();
        var data = entityState.getData();

        if (type === 'IMAGE') {
          var _alignment2 = data.alignment;
          if (_alignment2) setAlignment(_alignment2);
        }
      }

      return null;
    });
  }, [getEditor, hooks.toggleImageToolbarVisible]);
  return React__default.createElement("div", {
    className: "image-toolbar",
    ref: forwardRef
  }, React__default.createElement("div", {
    className: "image-toolbar-inner"
  }, React__default.createElement("div", {
    className: "image-toolbar-action-group"
  }, React__default.createElement(ImageAlignLeftFillContentButton, {
    activeKey: "leftFill",
    active: alignment === 'leftFill',
    clickHandler: clickHandler
  }), React__default.createElement(ImageAlignCenterButton, {
    activeKey: "center",
    active: alignment === 'center',
    clickHandler: clickHandler
  }), React__default.createElement(ImageAlignRightFillContentButton, {
    activeKey: "rightFill",
    active: alignment === 'rightFill',
    clickHandler: clickHandler
  }))), React__default.createElement("div", {
    className: "arrow-down"
  }));
};

var ImageToolbar$1 = /*#__PURE__*/withEditor(ImageToolbar);

var H1 = function H1(_ref) {
  var fill = _ref.fill;
  return React__default.createElement("svg", {
    width: "25",
    height: "25",
    fill: fill
  }, React__default.createElement("g", null, React__default.createElement("path", {
    d: "M5.53 11h6.96V5.34c0-.18.16-.34.35-.34h.82c.2 0 .36.16.36.34v13.3c0 .2-.16.35-.36.35h-.82a.35.35 0 0 1-.35-.34v-6.17H5.53v6.17c0 .19-.16.34-.36.34h-.82a.35.35 0 0 1-.35-.34V5.36c0-.2.16-.35.35-.35h.82c.2 0 .36.16.36.34V11z"
  }), React__default.createElement("path", {
    d: "M16.59 13.5c-.1.06-.24 0-.24-.13v-1.2a.3.3 0 0 1 .15-.26l1.6-.87a.3.3 0 0 1 .15-.04h1c.16 0 .3.13.3.3v7.4a.3.3 0 0 1-.3.3h-.93a.3.3 0 0 1-.3-.3v-5.96l-1.43.75z"
  })));
};

var H1$1 = /*#__PURE__*/withFillColor( /*#__PURE__*/withAction(H1));

var H2 = function H2(_ref) {
  var fill = _ref.fill;
  return React__default.createElement("svg", {
    width: "25",
    height: "25",
    fill: fill
  }, React__default.createElement("g", null, React__default.createElement("path", {
    d: "M5.53 10.98h6.96V5.34c0-.19.16-.34.35-.34h.82c.2 0 .36.15.36.34v13.3c0 .2-.16.35-.36.35h-.82a.35.35 0 0 1-.35-.34v-6.17H5.53v6.17c0 .18-.16.34-.36.34h-.82a.35.35 0 0 1-.35-.34V5.35c0-.2.16-.35.35-.35h.82c.2 0 .36.15.36.34v5.64z"
  }), React__default.createElement("path", {
    d: "M21 17.69v.92a.3.3 0 0 1-.3.3h-4.4a.3.3 0 0 1-.3-.3v-.95a.3.3 0 0 1 .07-.19l3.3-4.12c.11-.16.2-.3.23-.42s.04-.24.04-.35c0-.28-.08-.53-.26-.74a.98.98 0 0 0-.76-.32c-.3 0-.54.1-.74.29-.2.19-.32.45-.36.81 0 .08-.06.14-.14.14h-1.22a.16.16 0 0 1-.16-.16c.03-.75.29-1.37.76-1.86a2.44 2.44 0 0 1 1.76-.74c.78 0 1.41.26 1.9.75.49.48.74 1.1.75 1.84 0 .6-.18 1.13-.55 1.6l-2.55 3.2h2.63c.17 0 .3.13.3.3z"
  })));
};

var H2$1 = /*#__PURE__*/withFillColor( /*#__PURE__*/withAction(H2));

var H3 = function H3(_ref) {
  var fill = _ref.fill;
  return React__default.createElement("svg", {
    width: "25",
    height: "25",
    fill: fill
  }, React__default.createElement("g", null, React__default.createElement("path", {
    d: "M5.53 10.98h6.96V5.34c0-.19.16-.34.35-.34h.82c.2 0 .36.15.36.34v13.3c0 .2-.16.35-.36.35h-.82a.35.35 0 0 1-.35-.34v-6.17H5.53v6.17c0 .18-.16.34-.36.34h-.82a.35.35 0 0 1-.35-.34V5.35c0-.2.16-.35.35-.35h.82c.2 0 .36.15.36.34v5.64zM20.76 15.14c.16.35.24.72.24 1.1a2.66 2.66 0 0 1-4.38 2.13 3.94 3.94 0 0 1-.94-1.73c.04-.12.06-.13.2-.13h1.14c.14 0 .17.05.2.12.1.23.25.47.4.6.17.17.4.25.7.25.33 0 .6-.11.81-.33.22-.2.34-.5.34-.88 0-.4-.12-.69-.34-.9-.21-.2-.49-.3-.83-.3h-.22a.3.3 0 0 1-.3-.3v-.79c0-.16.14-.3.3-.3h.12c.39 0 .67-.1.87-.3a1 1 0 0 0 .3-.76c0-.32-.1-.57-.3-.78-.21-.2-.47-.3-.8-.3a.99.99 0 0 0-.62.22 1.4 1.4 0 0 0-.4.63c-.02.06-.05.18-.21.18h-1.11c-.17 0-.18-.11-.17-.17.1-.62.35-1.27.78-1.68a2.38 2.38 0 0 1 1.69-.7c.82 0 1.47.27 1.96.79.46.5.7 1.1.7 1.79 0 .34-.05.68-.18 1-.13.29-.33.54-.61.76.29.22.5.48.66.78z"
  })));
};

var H3$1 = /*#__PURE__*/withFillColor( /*#__PURE__*/withAction(H3));

var H4 = function H4(_ref) {
  var fill = _ref.fill;
  return React__default.createElement("svg", {
    width: "25",
    height: "25",
    fill: fill
  }, React__default.createElement("g", null, React__default.createElement("path", {
    d: "M5.53 10.98h6.96V5.34c0-.19.16-.34.35-.34h.82c.2 0 .36.15.36.34v13.3c0 .2-.16.35-.36.35h-.82a.35.35 0 0 1-.35-.34v-6.17H5.53v6.17c0 .18-.16.34-.36.34h-.82a.35.35 0 0 1-.35-.34V5.35c0-.2.16-.35.35-.35h.82c.2 0 .36.15.36.34v5.64z"
  }), React__default.createElement("path", {
    d: "M20 10.43V16h.7c.17 0 .3.13.3.3v.81a.3.3 0 0 1-.3.3H20v1.29a.3.3 0 0 1-.3.3h-.8a.3.3 0 0 1-.3-.3v-1.29h-3.3a.3.3 0 0 1-.3-.3V15.9a.2.2 0 0 1 .04-.1l3.58-5.53a.3.3 0 0 1 .25-.14h.83c.16 0 .3.14.3.3zm-1.4 2.44l-2 3.13h2v-3.13z"
  })));
};

var H4$1 = /*#__PURE__*/withFillColor( /*#__PURE__*/withAction(H4));

var Blockquote = function Blockquote(_ref) {
  var fill = _ref.fill;
  return React__default.createElement("svg", {
    width: "25",
    height: "25",
    fill: fill
  }, React__default.createElement("path", {
    d: "M15.48 18.024c-2.603 0-4.45-2.172-4.45-4.778 0-3.263 2.498-6.3 6.517-8.803l1.297 1.303c-2.497 1.63-3.91 3.042-3.91 5.214 0 2.824 3.91 3.582 3.91 3.91.11 1.41-1.194 3.15-3.366 3.15h.004v.004z"
  }), React__default.createElement("path", {
    d: "M6.578 18.024c-2.606 0-4.453-2.172-4.453-4.778 0-3.263 2.497-6.3 6.515-8.803l1.303 1.303c-2.606 1.63-3.907 3.042-3.907 5.106 0 2.823 3.91 3.58 3.91 3.91 0 1.518-1.304 3.257-3.368 3.257z"
  }));
};

var Blockquote$1 = /*#__PURE__*/withFillColor( /*#__PURE__*/withAction(Blockquote));

var CodeBlock = function CodeBlock(_ref) {
  var fill = _ref.fill;
  return React__default.createElement("svg", {
    width: "16",
    height: "16",
    fill: fill,
    viewBox: "0 0 1024 1024"
  }, React__default.createElement("path", {
    d: "M341.333333 128a85.333333 85.333333 0 0 0-85.333333 85.333333v170.666667a85.333333 85.333333 0 0 1-85.333333 85.333333H128v85.333334h42.666667a85.333333 85.333333 0 0 1 85.333333 85.333333v170.666667a85.333333 85.333333 0 0 0 85.333333 85.333333h85.333334v-85.333333H341.333333v-213.333334a85.333333 85.333333 0 0 0-85.333333-85.333333 85.333333 85.333333 0 0 0 85.333333-85.333333V213.333333h85.333334V128m256 0a85.333333 85.333333 0 0 1 85.333333 85.333333v170.666667a85.333333 85.333333 0 0 0 85.333333 85.333333h42.666667v85.333334h-42.666667a85.333333 85.333333 0 0 0-85.333333 85.333333v170.666667a85.333333 85.333333 0 0 1-85.333333 85.333333h-85.333334v-85.333333h85.333334v-213.333334a85.333333 85.333333 0 0 1 85.333333-85.333333 85.333333 85.333333 0 0 1-85.333333-85.333333V213.333333h-85.333334V128h85.333334z",
    fill: "",
    "p-id": "7920"
  }));
};

var CodeBlock$1 = /*#__PURE__*/withFillColor( /*#__PURE__*/withAction(CodeBlock));

var Bold = function Bold(_ref) {
  var fill = _ref.fill;
  return React__default.createElement("svg", {
    width: "25",
    height: "25",
    fill: fill
  }, React__default.createElement("path", {
    d: "M10.308 17.993h-5.92l.11-.894.783-.12c.56-.11.79-.224.79-.448V5.37c0-.225-.113-.336-.902-.448H4.5l-.114-.894h6.255c4.02 0 5.58 1.23 5.58 3.13 0 1.896-1.78 3.125-3.79 3.463v.11c2.69.34 4.25 1.56 4.25 3.57 0 2.35-2.01 3.69-6.37 3.69l.02.01h-.02zm-.335-12.96H8.967V10.5h1.23c1.788 0 2.79-1.23 2.79-2.683 0-1.685-1.004-2.803-3.006-2.803v.02zm-.223 6.36h-.783v5.588l1.225.23h.22c1.67 0 3.01-1.004 3.01-2.792 0-2.122-1.566-3.016-3.69-3.016h.018z"
  }));
};

var Bold$1 = /*#__PURE__*/withFillColor( /*#__PURE__*/withAction(Bold));

var Italic = function Italic(_ref) {
  var fill = _ref.fill;
  return React__default.createElement("svg", {
    width: "25",
    height: "25",
    fill: fill
  }, React__default.createElement("path", {
    d: "M9.847 18.04c-.533 0-2.027-.64-1.92-.853l2.027-7.68-.64-.214-1.387 1.494-.427-.427c.534-1.173 1.707-2.667 2.774-2.667.533 0 2.24.534 2.133.854l-2.133 7.786.533.214 1.6-1.067.427.427c-.64 1.066-1.92 2.133-2.987 2.133zm2.347-11.733c-.96 0-1.387-.64-1.387-1.387 0-1.067.747-1.92 1.493-1.92.854 0 1.387.64 1.387 1.493-.107 1.067-.747 1.814-1.493 1.814z"
  }));
};

var Italic$1 = /*#__PURE__*/withFillColor( /*#__PURE__*/withAction(Italic));

var StrikeThrough = function StrikeThrough(_ref) {
  var fill = _ref.fill;
  return React__default.createElement("svg", {
    width: "25",
    height: "25",
    fill: fill
  }, React__default.createElement("g", null, React__default.createElement("path", {
    d: "M9.47 10H7.41a3.82 3.82 0 0 1 .77-4.13 4.88 4.88 0 0 1 3.68-1.37c1.61 0 2.84.43 3.65 1.27.53.51.83 1.04 1.07 1.95.06.2.03.35-.07.36l-1.4.26c-.1.02-.13-.02-.18-.23-.1-.51-.24-.8-.52-1.12-.55-.7-1.43-1.07-2.54-1.07-1.8 0-3.11 1.05-3.15 2.55-.01.63.26 1.18.75 1.53zm4.33 2.94H5.9a.4.4 0 0 1-.4-.4v-.6c0-.23.18-.4.4-.4h12.32c.23 0 .4.17.4.4v.6a.4.4 0 0 1-.4.4h-1.94c.4.6.72 1.46.7 2.29-.06 2.63-2.08 4.27-5.3 4.27-2.16 0-3.77-.72-4.6-2.06a4.61 4.61 0 0 1-.6-1.58c-.07-.37-.05-.4.25-.46l1.06-.2c.33-.07.37-.04.43.27.14.73.34 1.15.75 1.57.6.65 1.57.98 2.82.98 2.13 0 3.52-1.04 3.56-2.7.02-.8-.23-1.4-.75-1.78l-.8-.6z"
  })));
};

var StrikeThrough$1 = /*#__PURE__*/withFillColor( /*#__PURE__*/withAction(StrikeThrough));

var Underline = function Underline(_ref) {
  var fill = _ref.fill;
  return React__default.createElement("svg", {
    width: "25",
    height: "25",
    fill: fill
  }, React__default.createElement("g", null, React__default.createElement("g", {
    transform: "translate(5 5)"
  }, React__default.createElement("path", {
    d: "M3.5.4v5.1c0 2.27 1.33 4 3.5 4s3.5-1.73 3.5-4V.4c0-.22.18-.4.4-.4h.7c.22 0 .4.18.4.4v5.1c0 3.17-1.96 5.5-5 5.5S2 8.67 2 5.5V.4c0-.22.18-.4.4-.4h.7c.22 0 .4.18.4.4z"
  }), React__default.createElement("rect", {
    y: "13.5",
    width: "14",
    height: "1.5",
    rx: ".4"
  }))));
};

var Underline$1 = /*#__PURE__*/withFillColor( /*#__PURE__*/withAction(Underline));

var InlineCode = function InlineCode(_ref) {
  var fill = _ref.fill;
  return React__default.createElement("svg", {
    width: "25",
    height: "25",
    fill: fill
  }, React__default.createElement("path", {
    d: "M9.44 8.3l-3.8 4.13 3.59 4.13c.12.15.12.37-.02.51l-.49.5a.38.38 0 0 1-.55 0L3.9 12.67a.38.38 0 0 1 0-.5l4.5-4.9a.37.37 0 0 1 .54 0l.49.5c.14.14.14.37 0 .52zm9.02 4.13L14.84 8.3a.37.37 0 0 1 .01-.51l.5-.5a.37.37 0 0 1 .54 0l4.32 4.89c.13.14.13.36 0 .5l-4.5 4.9a.38.38 0 0 1-.54 0l-.49-.5a.38.38 0 0 1 0-.52l3.78-4.13zm-5.7-8.35l.74.09c.2.02.35.2.33.41l-1.8 15.38a.38.38 0 0 1-.4.33l-.75-.08a.37.37 0 0 1-.33-.42l1.79-15.38c.02-.2.2-.35.41-.33z"
  }));
};

var InlineCode$1 = /*#__PURE__*/withFillColor( /*#__PURE__*/withAction(InlineCode));

var Link$1 = function Link(_ref) {
  var fill = _ref.fill;
  return React__default.createElement("svg", {
    width: "25",
    height: "25",
    fill: fill
  }, React__default.createElement("path", {
    d: "M2.2 13.17c0-.575.125-1.11.375-1.605l.02-.018v-.02c.014 0 .02-.008.02-.02 0-.014 0-.02.02-.02.122-.256.31-.52.576-.805l3.19-3.18c0-.008 0-.015.01-.02.01-.006.01-.013.01-.02.44-.413.91-.7 1.44-.853-.63.71-1.03 1.5-1.19 2.36-.04.24-.06.52-.06.81 0 .14.01.24.02.33L4.67 12.1c-.19.19-.316.407-.376.653a1.33 1.33 0 0 0-.057.415c0 .155.02.314.06.477.075.21.2.403.376.58l1.286 1.31c.27.276.62.416 1.03.416.42 0 .78-.14 1.06-.42l1.23-1.25.79-.78 1.15-1.16c.08-.09.19-.22.28-.4.103-.2.15-.42.15-.67 0-.16-.02-.31-.056-.45l-.02-.02v-.02l-.07-.14c0-.01-.013-.03-.04-.06l-.06-.13-.02-.02c0-.02-.01-.03-.02-.05a.592.592 0 0 0-.143-.16l-.48-.5c0-.042.015-.1.04-.15l.06-.12 1.17-1.14.087-.09.56.57c.023.04.08.1.16.18l.05.04c.006.018.02.036.035.06l.04.054c.01.01.02.025.03.04.03.023.04.046.04.058.04.04.08.09.1.14l.02.02c0 .018.01.03.024.04l.105.197v.02c.098.157.19.384.297.68a1 1 0 0 1 .04.255c.06.21.08.443.08.7 0 .22-.02.43-.06.63-.12.71-.44 1.334-.95 1.865l-.66.67-.97.972-1.554 1.57C8.806 17.654 7.98 18 7.01 18s-1.8-.34-2.487-1.026l-1.296-1.308a3.545 3.545 0 0 1-.913-1.627 4.541 4.541 0 0 1-.102-.88v-.01l-.012.01zm5.385-3.433c0-.183.023-.393.07-.63.13-.737.448-1.362.956-1.87l.66-.662.97-.983 1.56-1.56C12.48 3.34 13.3 3 14.27 3c.97 0 1.8.34 2.483 1.022l1.29 1.314c.44.438.744.976.913 1.618.067.32.102.614.102.87 0 .577-.123 1.11-.375 1.605l-.02.01v.02l-.02.04c-.148.27-.35.54-.6.81l-3.187 3.19c0 .01 0 .01-.01.02-.01 0-.01.01-.01.02-.434.42-.916.7-1.427.83.63-.67 1.03-1.46 1.19-2.36.04-.26.06-.53.06-.81 0-.14-.01-.26-.02-.35l1.99-1.97c.18-.21.3-.42.35-.65.04-.12.05-.26.05-.42 0-.16-.02-.31-.06-.48-.07-.19-.19-.38-.36-.58l-1.3-1.3a1.488 1.488 0 0 0-1.06-.42c-.42 0-.77.14-1.06.41L11.98 6.7l-.79.793-1.157 1.16c-.088.075-.186.21-.294.4-.09.233-.14.46-.14.67 0 .16.02.31.06.452l.02.02v.023l.06.144c0 .006.01.026.05.06l.06.125.02.02c0 .01 0 .013.01.02 0 .005.01.01.01.02.05.08.1.134.14.16l.47.5c0 .04-.02.093-.04.15l-.06.12-1.15 1.15-.1.08-.56-.56a2.31 2.31 0 0 0-.18-.187c-.02-.01-.02-.03-.02-.04l-.02-.02a.375.375 0 0 1-.1-.122c-.03-.024-.05-.043-.05-.06l-.1-.15-.02-.02-.02-.04L8 11.4v-.02a5.095 5.095 0 0 1-.283-.69 1.035 1.035 0 0 1-.04-.257 2.619 2.619 0 0 1-.093-.7v.007z"
  }));
};

var Link$2 = /*#__PURE__*/withFillColor( /*#__PURE__*/withAction(Link$1));

var NumberedList = function NumberedList(_ref) {
  var fill = _ref.fill;
  return React__default.createElement("svg", {
    width: "25",
    height: "25",
    fill: fill
  }, React__default.createElement("path", {
    d: "M5.44 15.78c.46 0 .83.13 1.1.38.27.24.4.53.4.87 0 .17-.03.33-.1.5-.06.1-.14.2-.25.3.11.08.2.19.27.3.1.17.14.35.14.54 0 .4-.16.73-.45.96a1.68 1.68 0 0 1-2.01.07c-.25-.17-.59-.76-.53-.89.03-.08.08-.11.19-.11h.6c.1 0 .15.03.2.1.05.1.12.2.18.25a.5.5 0 0 0 .31.08c.15 0 .27-.04.37-.12.1-.08.14-.18.15-.32 0-.16-.06-.27-.15-.34a.59.59 0 0 0-.38-.12h-.09a.28.28 0 0 1-.28-.27v-.26c0-.15.13-.27.28-.27h.04c.18 0 .3-.04.4-.12a.33.33 0 0 0 .12-.27.37.37 0 0 0-.13-.28.56.56 0 0 0-.63-.03.54.54 0 0 0-.18.24L5 17c-.03.06-.1.1-.18.1h-.59c-.13 0-.2-.09-.17-.18.05-.33.2-.63.44-.82.26-.22.58-.33.95-.33zM19.7 17.5c.17 0 .3.13.3.3v.9a.3.3 0 0 1-.3.3H9.3a.3.3 0 0 1-.3-.3v-.9c0-.17.13-.3.3-.3h10.4zM5.47 10c.44 0 .8.12 1.09.36.29.24.44.54.44.9 0 .3-.1.56-.32.79l-1.25 1.3h1.3c.15 0 .27.11.27.25v.37c0 .14-.12.25-.26.25H4.26a.26.26 0 0 1-.26-.25v-.49l1.85-1.93c.06-.06.1-.12.11-.15.02-.05.02-.1.02-.14 0-.1-.03-.2-.11-.27-.08-.08-.2-.12-.35-.12a.52.52 0 0 0-.34.1.44.44 0 0 0-.17.3c0 .1-.07.16-.15.16h-.7a.17.17 0 0 1-.16-.17c.02-.36.17-.66.44-.9.28-.24.62-.36 1.03-.36zm14.23 1c.17 0 .3.13.3.3v.9a.3.3 0 0 1-.3.3H9.3a.3.3 0 0 1-.3-.3v-.9c0-.17.13-.3.3-.3h10.4zM5.75 4.28c.14 0 .25.12.25.26V8.2c0 .14-.11.25-.25.25h-.5A.25.25 0 0 1 5 8.2V5.5l-.72.36A.18.18 0 0 1 4 5.72v-.68c0-.1.05-.18.14-.22l.95-.5a.25.25 0 0 1 .13-.04h.53zM19.7 5c.17 0 .3.13.3.3v.9a.3.3 0 0 1-.3.3H9.3a.3.3 0 0 1-.3-.3v-.9c0-.17.13-.3.3-.3h10.4z"
  }));
};

var NumberedList$1 = /*#__PURE__*/withFillColor( /*#__PURE__*/withAction(NumberedList));

var BulletedList = function BulletedList(_ref) {
  var fill = _ref.fill;
  return React__default.createElement("svg", {
    width: "25",
    height: "25",
    fill: fill
  }, React__default.createElement("path", {
    d: "M19.7 17.5c.17 0 .3.13.3.3v.9a.3.3 0 0 1-.3.3H9.3a.3.3 0 0 1-.3-.3v-.9c0-.17.13-.3.3-.3h10.4zm-14-.5c.17 0 .3.13.3.3v1.4a.3.3 0 0 1-.3.3H4.3a.3.3 0 0 1-.3-.3v-1.4c0-.17.13-.3.3-.3h1.4zm0-6c.17 0 .3.13.3.3v1.4a.3.3 0 0 1-.3.3H4.3a.3.3 0 0 1-.3-.3v-1.4c0-.17.13-.3.3-.3h1.4zm14 0c.17 0 .3.13.3.3v.9a.3.3 0 0 1-.3.3H9.3a.3.3 0 0 1-.3-.3v-.9c0-.17.13-.3.3-.3h10.4zm-14-6c.17 0 .3.13.3.3v1.4a.3.3 0 0 1-.3.3H4.3a.3.3 0 0 1-.3-.3V5.3c0-.17.13-.3.3-.3h1.4zm14 0c.17 0 .3.13.3.3v.9a.3.3 0 0 1-.3.3H9.3a.3.3 0 0 1-.3-.3v-.9c0-.17.13-.3.3-.3h10.4z"
  }));
};

var BulletedList$1 = /*#__PURE__*/withFillColor( /*#__PURE__*/withAction(BulletedList));

var createLinkAtSelection = function createLinkAtSelection(editorState, url) {
  var contentState = editorState.getCurrentContent().createEntity('LINK', 'MUTABLE', {
    url: url
  });
  var entityKey = contentState.getLastCreatedEntityKey();
  return draftJs.RichUtils.toggleLink(editorState, editorState.getSelection(), entityKey);
};

var createLinkSpanAtSelection = function createLinkSpanAtSelection(editorState) {
  var selection = editorState.getSelection();
  var startKey = selection.getStartKey();
  var startOffset = selection.getStartOffset();
  var endOffset = selection.getEndOffset();
  var currentContent = editorState.getCurrentContent();
  var contentBlock = currentContent.getBlockForKey(startKey);
  var text = contentBlock.getText().slice(startOffset, endOffset);
  var contentState = editorState.getCurrentContent().createEntity('LINK_SPAN', 'MUTABLE', {
    text: text
  });
  var entityKey = contentState.getLastCreatedEntityKey();
  return draftJs.RichUtils.toggleLink(editorState, editorState.getSelection(), entityKey);
};

var Divider = /*#__PURE__*/React__default.memo(function () {
  return React__default.createElement("div", {
    className: "divider"
  });
}, function () {
  return true;
});

var buildBlockTypeHandler = function buildBlockTypeHandler(getEditor, type) {
  return function () {
    var _getEditor = getEditor(),
        hooks = _getEditor.hooks,
        editorState = _getEditor.editorState;

    hooks.toggleBlockType.call(editorState, type);
  };
};

var buildInlineTypeHandler = function buildInlineTypeHandler(getEditor, inlineStyle) {
  return function () {
    var _getEditor2 = getEditor(),
        hooks = _getEditor2.hooks;

    hooks.toggleInlineStyleV2.call(inlineStyle);
  };
};

var H1Button = function H1Button(_ref) {
  var active = _ref.active,
      getEditor = _ref.getEditor;
  var handleClick = React.useRef(buildBlockTypeHandler(getEditor, 'header-one'));
  return React__default.createElement(H1$1, {
    active: active,
    onClick: handleClick.current
  });
};

var H2Button = function H2Button(_ref2) {
  var active = _ref2.active,
      getEditor = _ref2.getEditor;
  var handleClick = React.useRef(buildBlockTypeHandler(getEditor, 'header-two'));
  return React__default.createElement(H2$1, {
    active: active,
    onClick: handleClick.current
  });
};

var H3Button = function H3Button(_ref3) {
  var active = _ref3.active,
      getEditor = _ref3.getEditor;
  var handleClick = React.useRef(buildBlockTypeHandler(getEditor, 'header-three'));
  return React__default.createElement(H3$1, {
    active: active,
    onClick: handleClick.current
  });
};

var H4Button = function H4Button(_ref4) {
  var active = _ref4.active,
      getEditor = _ref4.getEditor;
  var handleClick = React.useRef(buildBlockTypeHandler(getEditor, 'header-four'));
  return React__default.createElement(H4$1, {
    active: active,
    onClick: handleClick.current
  });
};

var BlockquoteButton = function BlockquoteButton(_ref5) {
  var active = _ref5.active,
      getEditor = _ref5.getEditor;
  var handleClick = React.useRef(buildBlockTypeHandler(getEditor, 'blockquote'));
  return React__default.createElement(Blockquote$1, {
    active: active,
    onClick: handleClick.current
  });
};

var CodeBlockButton = function CodeBlockButton(_ref6) {
  var active = _ref6.active,
      getEditor = _ref6.getEditor;
  var handleClick = React.useRef(buildBlockTypeHandler(getEditor, 'code-block'));
  return React__default.createElement(CodeBlock$1, {
    active: active,
    onClick: handleClick.current
  });
};

var BoldButton = function BoldButton(_ref7) {
  var active = _ref7.active,
      getEditor = _ref7.getEditor;
  var handleClick = React.useRef(buildInlineTypeHandler(getEditor, 'BOLD'));
  return React__default.createElement(Bold$1, {
    active: active,
    onClick: handleClick.current
  });
};

var ItalicButton = function ItalicButton(_ref8) {
  var active = _ref8.active,
      getEditor = _ref8.getEditor;
  var handleClick = React.useRef(buildInlineTypeHandler(getEditor, 'ITALIC'));
  return React__default.createElement(Italic$1, {
    active: active,
    onClick: handleClick.current
  });
};

var StrikeThroughButton = function StrikeThroughButton(_ref9) {
  var active = _ref9.active,
      getEditor = _ref9.getEditor;
  var handleClick = React.useRef(buildInlineTypeHandler(getEditor, 'STRIKE-THROUGH'));
  return React__default.createElement(StrikeThrough$1, {
    active: active,
    onClick: handleClick.current
  });
};

var UnderlineButton = function UnderlineButton(_ref10) {
  var active = _ref10.active,
      getEditor = _ref10.getEditor;
  var handleClick = React.useRef(buildInlineTypeHandler(getEditor, 'UNDERLINE'));
  return React__default.createElement(Underline$1, {
    active: active,
    onClick: handleClick.current
  });
};

var InlineCodeButton = function InlineCodeButton(_ref11) {
  var active = _ref11.active,
      getEditor = _ref11.getEditor;
  var handleClick = React.useRef(buildInlineTypeHandler(getEditor, 'CODE'));
  return React__default.createElement(InlineCode$1, {
    active: active,
    onClick: handleClick.current
  });
};

var LinkButton = function LinkButton(_ref12) {
  var handleClick = _ref12.handleClick,
      getEditor = _ref12.getEditor,
      active = _ref12.active;

  var onClickHandler = function onClickHandler() {
    var _getEditor3 = getEditor(),
        editorState = _getEditor3.editorState,
        hooks = _getEditor3.hooks;

    hooks.afterClickLinkButton.call(editorState);
    var nextState = createLinkSpanAtSelection(editorState);
    hooks.setState.call(nextState);
    handleClick();
  };

  return React__default.createElement(Link$2, {
    onClick: onClickHandler,
    active: active
  });
};

var NumberedListButton = function NumberedListButton(_ref13) {
  var active = _ref13.active,
      getEditor = _ref13.getEditor;
  var handleClick = React.useRef(buildBlockTypeHandler(getEditor, 'ordered-list-item'));
  return React__default.createElement(NumberedList$1, {
    active: active,
    onClick: handleClick.current
  });
};

var BulletedListButton = function BulletedListButton(_ref14) {
  var active = _ref14.active,
      getEditor = _ref14.getEditor;
  var handleClick = React.useRef(buildBlockTypeHandler(getEditor, 'unordered-list-item'));
  return React__default.createElement(BulletedList$1, {
    active: active,
    onClick: handleClick.current
  });
};

var onlyContains = function onlyContains(arr, item) {
  if (arr === void 0) {
    arr = [];
  }

  if (arr.length > 1) return false;
  return arr[0] === item;
};

var StyleControls = function StyleControls(_ref15) {
  var blockTypes = _ref15.blockTypes,
      styles = _ref15.styles,
      getEditor = _ref15.getEditor,
      toggleDisplayMode = _ref15.toggleDisplayMode,
      hasLink = _ref15.hasLink;
  return React__default.createElement("div", {
    className: "inline-toolbar-inner"
  }, React__default.createElement("div", {
    className: "inline-toolbar-action-group"
  }, React__default.createElement(H1Button, {
    getEditor: getEditor,
    active: onlyContains(blockTypes, 'header-one')
  }), React__default.createElement(H2Button, {
    getEditor: getEditor,
    active: onlyContains(blockTypes, 'header-two')
  }), React__default.createElement(H3Button, {
    getEditor: getEditor,
    active: onlyContains(blockTypes, 'header-three')
  }), React__default.createElement(H4Button, {
    getEditor: getEditor,
    active: onlyContains(blockTypes, 'header-four')
  }), React__default.createElement(BlockquoteButton, {
    getEditor: getEditor,
    active: onlyContains(blockTypes, 'blockquote')
  }), React__default.createElement(CodeBlockButton, {
    getEditor: getEditor,
    active: onlyContains(blockTypes, 'code-block')
  }), React__default.createElement(Divider, null), React__default.createElement(BoldButton, {
    getEditor: getEditor,
    active: styles.has('BOLD')
  }), React__default.createElement(ItalicButton, {
    getEditor: getEditor,
    active: styles.has('ITALIC')
  }), React__default.createElement(StrikeThroughButton, {
    getEditor: getEditor,
    active: styles.has('STRIKE-THROUGH')
  }), React__default.createElement(UnderlineButton, {
    getEditor: getEditor,
    active: styles.has('UNDERLINE')
  }), React__default.createElement(InlineCodeButton, {
    getEditor: getEditor,
    active: styles.has('CODE')
  }), React__default.createElement(LinkButton, {
    handleClick: toggleDisplayMode,
    getEditor: getEditor,
    active: hasLink
  }), React__default.createElement(Divider, null), React__default.createElement(NumberedListButton, {
    getEditor: getEditor,
    active: onlyContains(blockTypes, 'ordered-list-item')
  }), React__default.createElement(BulletedListButton, {
    getEditor: getEditor,
    active: onlyContains(blockTypes, 'unordered-list-item')
  })));
};

var Unlink = function Unlink(_ref) {
  var fill = _ref.fill;
  return React__default.createElement("svg", {
    width: "14",
    height: "14",
    xmlns: "http://www.w3.org/2000/svg",
    viewBox: "0 0 512 512",
    fill: fill
  }, React__default.createElement("path", {
    d: "M304.083 405.907c4.686 4.686 4.686 12.284 0 16.971l-44.674 44.674c-59.263 59.262-155.693 59.266-214.961 0-59.264-59.265-59.264-155.696 0-214.96l44.675-44.675c4.686-4.686 12.284-4.686 16.971 0l39.598 39.598c4.686 4.686 4.686 12.284 0 16.971l-44.675 44.674c-28.072 28.073-28.072 73.75 0 101.823 28.072 28.072 73.75 28.073 101.824 0l44.674-44.674c4.686-4.686 12.284-4.686 16.971 0l39.597 39.598zm-56.568-260.216c4.686 4.686 12.284 4.686 16.971 0l44.674-44.674c28.072-28.075 73.75-28.073 101.824 0 28.072 28.073 28.072 73.75 0 101.823l-44.675 44.674c-4.686 4.686-4.686 12.284 0 16.971l39.598 39.598c4.686 4.686 12.284 4.686 16.971 0l44.675-44.675c59.265-59.265 59.265-155.695 0-214.96-59.266-59.264-155.695-59.264-214.961 0l-44.674 44.674c-4.686 4.686-4.686 12.284 0 16.971l39.597 39.598zm234.828 359.28l22.627-22.627c9.373-9.373 9.373-24.569 0-33.941L63.598 7.029c-9.373-9.373-24.569-9.373-33.941 0L7.029 29.657c-9.373 9.373-9.373 24.569 0 33.941l441.373 441.373c9.373 9.372 24.569 9.372 33.941 0z"
  }));
};

var Unlink$1 = /*#__PURE__*/withFillColor( /*#__PURE__*/withAction(Unlink));

var InputBar = function InputBar(_ref) {
  var getEditor = _ref.getEditor;
  var inputRef = React.createRef();

  var _getEditor = getEditor(),
      hooks = _getEditor.hooks;

  React.useEffect(function () {
    if (inputRef.current) inputRef.current.focus();
  }, [inputRef]);
  var submit = React.useCallback(function (value) {
    var _getEditor2 = getEditor(),
        editorState = _getEditor2.editorState,
        hooks = _getEditor2.hooks;

    var newState = createLinkAtSelection(editorState, value);
    var currentContent = newState.getCurrentContent();
    var selection = newState.getSelection();
    var focusOffset = selection.getFocusOffset();
    var focusKey = selection.getFocusKey(); // 通过下面的方式，并不能够将cursor放置在刚刚的selection末尾
    // const nextState = EditorState.set(newState, {
    //   currentContent: currentContent.merge({
    //     selectionAfter: currentContent.getSelectionAfter().merge({
    //       hasFocus: true,
    //       anchorOffset: focusOffset,
    //       anchorKey: focusKey,
    //     })
    //   })
    // })
    // hooks.setState.call(nextState)
    // 当用户输入完以后，指针是放置在selection的后面

    var nextState = draftJs.EditorState.forceSelection(newState, currentContent.getSelectionAfter().merge({
      anchorOffset: focusOffset,
      anchorKey: focusKey
    }));
    hooks.setState.call(nextState);
  }, [getEditor]);
  var onKeyDownHandler = React.useCallback(function (e) {
    var key = e.key;

    if (key === 'Enter') {
      hooks.cleanUpLinkClickState.call();
      e.preventDefault();
      var inputValue = e.target.value;
      submit(inputValue);
      hooks.hideInlineToolbar.call();
    } else if (key === 'Escape') {
      e.preventDefault();
      hooks.hideInlineToolbar.call();
    }
  }, [hooks.cleanUpLinkClickState, hooks.hideInlineToolbar, submit]);
  return React__default.createElement("div", {
    className: "inline-toolbar-link-inner"
  }, React__default.createElement("input", {
    ref: inputRef,
    className: "inline-link-input",
    placeholder: "Paste your link, such as http://google.com...",
    onKeyDown: onKeyDownHandler
  }), React__default.createElement(Divider, null), React__default.createElement("div", {
    className: "link-action-group"
  }, React__default.createElement(Link$2, {
    active: false,
    onClick: function onClick() {}
  }), React__default.createElement(Unlink$1, {
    active: false,
    onClick: function onClick() {}
  })));
};

var Map$1 = Immutable__default.Map;

function getSelectionBlockTypes(editorState) {
  var contentState = editorState.getCurrentContent();
  var selectionState = editorState.getSelection();
  var startKey = selectionState.getStartKey();
  var endKey = selectionState.getEndKey();
  var blockMap = contentState.getBlockMap();
  var blockTypes = [];
  blockMap.toSeq().skipUntil(function (_, k) {
    return k === startKey;
  }).takeUntil(function (_, k) {
    return k === endKey;
  }).concat(Map$1([[endKey, blockMap.get(endKey)]])).forEach(function (block) {
    if (!block) return;
    var currentBlockType = block.getType();

    if (blockTypes.indexOf(currentBlockType) === -1) {
      blockTypes.push(currentBlockType);
    }
  });
  return blockTypes;
}

// https://github.com/facebook/draft-js/blob/master/src/model/transaction/ContentStateInlineStyle.js#L38 modifyInlineStyle
var Map$2 = Immutable__default.Map;

function getInlineToolbarInlineInfo(editorState) {
  var contentState = editorState.getCurrentContent();
  var selectionState = editorState.getSelection();
  var blockMap = contentState.getBlockMap();
  var startKey = selectionState.getStartKey();
  var startOffset = selectionState.getStartOffset();
  var endKey = selectionState.getEndKey();
  var endOffset = selectionState.getEndOffset();
  var values;
  var hasChanceToInit = true;
  var intersectionIsEmpty = false;
  var hasLink = false;
  blockMap.skipUntil(function (_, k) {
    return k === startKey;
  }).takeUntil(function (_, k) {
    return k === endKey;
  }).concat(Map$2([[endKey, blockMap.get(endKey)]])).forEach(function (block, blockKey) {
    if (hasLink && intersectionIsEmpty) {
      return;
    }

    var styles = Immutable__default.OrderedSet();
    var sliceStart;
    var sliceEnd;

    if (startKey === endKey) {
      sliceStart = startOffset;
      sliceEnd = endOffset;
    } else {
      sliceStart = blockKey === startKey ? startOffset : 0;
      sliceEnd = blockKey === endKey ? endOffset : block.getLength();
    }

    var chars = block.getCharacterList();
    var current;

    while (sliceStart < sliceEnd) {
      if (hasLink && intersectionIsEmpty) {
        break;
      }

      var _char = chars.get(sliceStart);

      current = _char.getStyle();

      var entityKey = _char.getEntity();

      if (entityKey) {
        var entityType = contentState.getEntity(entityKey).getType();
        hasLink = entityType === 'LINK';
      } // 计算inline `styles`；一旦已经知道`intersect`为空，那么就不再进行处理了


      if (!intersectionIsEmpty) {
        // 只有当前的char包含`inline` style时才进行处理
        if (current.size > 0) {
          if (!styles.size) {
            // 如果已经初始化过了，这个时候`styles`还为空的话，证明`intersect`
            // 以后为空；所以也就没有处理的必要了
            if (!hasChanceToInit) {
              intersectionIsEmpty = true;
            } else {
              current.forEach(function (style) {
                return styles.add(style);
              }); // eslint-disable-line

              hasChanceToInit = false;
            }
          } else {
            styles = styles.intersect(current);
          }
        }
      }

      sliceStart++;
    }

    values = styles;
  });
  return {
    styles: values,
    hasLink: hasLink
  };
}

var InlineToolbar$1 = function InlineToolbar(props) {
  var forwardRef = props.forwardRef,
      getEditor = props.getEditor;

  var _useState = React.useState({
    styles: Immutable__default.OrderedSet(),
    blockTypes: [],
    inDisplayMode: true,
    hasLink: false
  }),
      value = _useState[0],
      setValue = _useState[1];

  var inDisplayModeRef = React.useRef(true);
  React.useEffect(function () {
    var _getEditor = getEditor(),
        hooks = _getEditor.hooks;

    hooks.inlineBarChange.tap('InlineToolbar', function (editorState, visibility) {
      var nextValue = {
        inDisplayMode: visibility === 'hidden' ? true : inDisplayModeRef.current
      };

      if (editorState) {
        var _getInlineToolbarInli = getInlineToolbarInlineInfo(editorState),
            _styles = _getInlineToolbarInli.styles,
            _hasLink = _getInlineToolbarInli.hasLink;

        if (_styles) {
          nextValue.styles = _styles;
        }

        nextValue.hasLink = _hasLink;
        nextValue.blockTypes = getSelectionBlockTypes(editorState);
      } else {
        nextValue.styles = Immutable__default.OrderedSet();
        nextValue.blockTypes = [];
        nextValue.hasLink = false;
      }

      inDisplayModeRef.current = nextValue.inDisplayMode;
      setValue(nextValue);
    });
  }, [getEditor]);
  var styles = value.styles,
      blockTypes = value.blockTypes,
      inDisplayMode = value.inDisplayMode,
      hasLink = value.hasLink;
  var toggleDisplayMode = React.useCallback(function () {
    setValue(_extends({}, value, {
      inDisplayMode: !inDisplayMode
    }));
    inDisplayModeRef.current = !inDisplayMode;
  }, [inDisplayMode, value]);
  return React__default.createElement("div", {
    className: "inline-toolbar",
    ref: forwardRef
  }, inDisplayMode && React__default.createElement(StyleControls, {
    styles: styles,
    hasLink: hasLink,
    blockTypes: blockTypes,
    getEditor: getEditor,
    toggleDisplayMode: toggleDisplayMode
  }), !inDisplayMode && React__default.createElement(InputBar, {
    getEditor: getEditor
  }), React__default.createElement("div", {
    className: "arrow-down"
  }));
};

var MemoToolbar = /*#__PURE__*/React__default.memo(function (props) {
  return React__default.createElement(InlineToolbar$1, Object.assign({}, props));
}, function () {
  return true;
});
var InlineToolbar$2 = /*#__PURE__*/withEditor(MemoToolbar);

var Plus = function Plus() {
  return React__default.createElement("svg", {
    viewBox: "0 0 18 18",
    width: "14",
    height: "14",
    fill: "rgba(55, 53, 47, 0.3)"
  }, React__default.createElement("polygon", {
    points: "17,8 10,8 10,1 8,1 8,8 1,8 1,10 8,10 8,17 10,17 10,10 17,10 "
  }));
};

var Dragger$1 = function Dragger() {
  return React__default.createElement("svg", {
    viewBox: "0 0 10 10",
    width: "14",
    height: "14",
    fill: "rgba(55, 53, 47, 0.3)"
  }, React__default.createElement("path", {
    d: "M3,2 C2.44771525,2 2,1.55228475 2,1 C2,0.44771525 2.44771525,0 3,0 C3.55228475,0 4,0.44771525 4,1 C4,1.55228475 3.55228475,2 3,2 Z M3,6 C2.44771525,6 2,5.55228475 2,5 C2,4.44771525 2.44771525,4 3,4 C3.55228475,4 4,4.44771525 4,5 C4,5.55228475 3.55228475,6 3,6 Z M3,10 C2.44771525,10 2,9.55228475 2,9 C2,8.44771525 2.44771525,8 3,8 C3.55228475,8 4,8.44771525 4,9 C4,9.55228475 3.55228475,10 3,10 Z M7,2 C6.44771525,2 6,1.55228475 6,1 C6,0.44771525 6.44771525,0 7,0 C7.55228475,0 8,0.44771525 8,1 C8,1.55228475 7.55228475,2 7,2 Z M7,6 C6.44771525,6 6,5.55228475 6,5 C6,4.44771525 6.44771525,4 7,4 C7.55228475,4 8,4.44771525 8,5 C8,5.55228475 7.55228475,6 7,6 Z M7,10 C6.44771525,10 6,9.55228475 6,9 C6,8.44771525 6.44771525,8 7,8 C7.55228475,8 8,8.44771525 8,9 C8,9.55228475 7.55228475,10 7,10 Z"
  }));
};

var Sidebar = function Sidebar(props) {
  var forwardRef = props.forwardRef;
  var containerStyleRef = React.useRef(classNames('container'));
  return React__default.createElement("div", {
    className: containerStyleRef.current,
    ref: forwardRef
  }, React__default.createElement("div", {
    className: "plus"
  }, React__default.createElement(Plus, null)), React__default.createElement("div", {
    className: "dragger"
  }, React__default.createElement(Dragger$1, null)));
};

// https://stackoverflow.com/questions/3115982/how-to-check-if-two-arrays-are-equal-with-javascript
var compareArray = (function (a, b) {
  if (a === void 0) {
    a = [];
  }

  if (b === void 0) {
    b = [];
  }

  if (a === b) return [];
  var result = [];
  var aLen = a.length;
  var bLen = b.length;

  for (var i = 0; i < aLen; ++i) {
    var blockKey = a[i];

    if (b.indexOf(blockKey) === -1) {
      result.push({
        op: 'remove',
        blockKey: blockKey
      });
    }
  }

  for (var _i = 0; _i < bLen; ++_i) {
    var _blockKey = b[_i];

    if (a.indexOf(_blockKey) === -1) {
      result.push({
        op: 'add',
        blockKey: _blockKey
      });
    }
  }

  return result;
});

var NewEditor = function NewEditor(props) {
  var getEditor = props.getEditor,
      forwardRef = props.forwardRef,
      placeholder = props.placeholder,
      imageRef = props.imageRef,
      inlineRef = props.inlineRef,
      sidebarRef = props.sidebarRef,
      blockRenderMap = props.blockRenderMap,
      customStyleMap = props.customStyleMap;

  var _getEditor = getEditor(),
      hooks = _getEditor.hooks,
      editorState = _getEditor.editorState;

  var didUpdate = React.useRef(false);
  var pasteText = React.useRef(); // ts-hint: useRef([]) as RefObject<string[]> is not right. please refer to
  // https://github.com/DefinitelyTyped/DefinitelyTyped/issues/31065

  var lastBlockMapKeys = React.useRef([]);
  var isInCompositionModeRef = React.useRef(false);
  React.useEffect(function () {
    var currentContent = editorState.getCurrentContent();
    var currentBlockMap = currentContent.getBlockMap();
    var currentBlockMapKeys = currentBlockMap.keySeq().toArray();
    var diff = compareArray(lastBlockMapKeys.current, currentBlockMapKeys);
    var isInCompositionMode = editorState.isInCompositionMode();
    var force = diff.length || isInCompositionModeRef.current && !isInCompositionMode;
    hooks.syncBlockKeys.call(currentBlockMapKeys, force);
    isInCompositionModeRef.current = editorState.isInCompositionMode();
  });
  React.useEffect(function () {
    if (didUpdate.current) {
      hooks.didUpdate.call(editorState);
      hooks.updatePlaceholder.call(editorState, placeholder);
      hooks.syncSelectionChange.call(editorState);
    }
  });
  React.useEffect(function () {
    hooks.updatePlaceholder.call(editorState, placeholder);
    didUpdate.current = true;
  }, [editorState, hooks.updatePlaceholder, placeholder]);
  React.useEffect(function () {
    var currentContent = editorState.getCurrentContent();
    var currentBlockMap = currentContent.getBlockMap();
    var currentBlockMapKeys = currentBlockMap.keySeq().toArray();
    var diff = compareArray(lastBlockMapKeys.current, currentBlockMapKeys);

    if (diff.length) {
      hooks.updateDragSubscription.call(diff);
    }

    lastBlockMapKeys.current = currentBlockMapKeys;
  });
  var onChange = React.useCallback(function (newEditorState) {
    var _getEditor2 = getEditor(),
        editorState = _getEditor2.editorState;

    var nextState = hooks.stateFilter.call(editorState, newEditorState, pasteText.current);
    hooks.onChange.call(nextState);
  }, [getEditor, hooks.onChange, hooks.stateFilter]);
  var handleKeyCommand = React.useCallback(function (command, es) {
    return hooks.handleKeyCommand.call(command, es);
  }, [hooks.handleKeyCommand]);
  var getBlockStyle = React.useCallback(function (block) {
    return hooks.blockStyleFn.call(block);
  }, [hooks.blockStyleFn]);
  var handleBlockRender = React.useCallback(function (contentBlock) {
    var _getEditor3 = getEditor(),
        editorState = _getEditor3.editorState;

    return hooks.blockRendererFn.call(contentBlock, editorState);
  }, [getEditor, hooks.blockRendererFn]);
  /**
   * To trigger `addImage`
   */

  var handleDroppedFiles = React.useCallback(function (dropSelection, files) {
    hooks.handleDroppedFiles.call(editorState, dropSelection, files);
    return 'handled';
  }, [editorState, hooks.handleDroppedFiles]);
  return React__default.createElement("div", {
    className: "miuffy-editor-root"
  }, React__default.createElement("div", {
    className: "miuffy-editor"
  }, React__default.createElement(Title$1, null), React__default.createElement(draftJs.Editor, {
    editorState: editorState,
    blockStyleFn: getBlockStyle,
    customStyleMap: customStyleMap,
    blockRenderMap: blockRenderMap,
    blockRendererFn: handleBlockRender,
    onChange: onChange,
    handleKeyCommand: handleKeyCommand,
    handleDroppedFiles: handleDroppedFiles,
    ref: forwardRef
  })), React__default.createElement(ImageToolbar$1, {
    forwardRef: imageRef
  }), React__default.createElement(InlineToolbar$2, {
    forwardRef: inlineRef
  }), React__default.createElement(Sidebar, {
    forwardRef: sidebarRef
  }));
};

var WrappedEditor = /*#__PURE__*/withEditor(NewEditor); // ts-hint: https://medium.com/@jrwebdev/react-higher-order-component-patterns-in-typescript-42278f7590fb
// ts-hint: https://medium.com/@martin_hotell/react-refs-with-typescript-a32d56c4d315

var Editor = /*#__PURE__*/React__default.forwardRef(function (props, ref) {
  return React__default.createElement(WrappedEditor, Object.assign({}, props, {
    forwardRef: ref
  }));
});

var Provider = Context.Provider; // https://fettblog.eu/typescript-react/components/: TypeScript and React: Components

var createEditor = function createEditor(defaultPlugins) {
  return /*#__PURE__*/function (_PureComponent) {
    _inheritsLoose(_class, _PureComponent);

    function _class(props) {
      var _this;

      _this = _PureComponent.call(this, props) || this;

      _this.getEditor = function () {
        return {
          hooks: _this.hooks,
          editorState: _this.state.editorState,
          editorRef: _this.editorRef,
          inlineToolbarRef: _this.inlineToolbarRef,
          imageToolbarRef: _this.imageToolbarRef,
          sidebarRef: _this.sidebarRef
        };
      };

      _this.init = function () {
        _this.blockRenderMap = _this.hooks.createBlockRenderMap.call();
        _this.customStyleMap = _this.hooks.createCustomStyleMap.call();
      };

      var _props$plugins = props.plugins,
          plugins = _props$plugins === void 0 ? [] : _props$plugins;
      _this.plugins = plugins.concat(defaultPlugins);
      _this.customStyleMap = null;
      _this.blockRenderMap = null;
      _this.hooks = {
        setState: new tapable.SyncHook(['editorState', 'callback']),
        onChange: new tapable.SyncWaterfallHook(['editorState']),
        stateFilter: new tapable.SyncBailHook(['oldEditorState', 'editorState', 'pasteText']),
        toggleWaterfallBlockType: new tapable.SyncWaterfallHook(['newEditorState', 'editorState', 'blockType']),
        toggleBlockType: new tapable.SyncHook(['editorState', 'blockType']),
        toggleInlineStyleV2: new tapable.SyncHook(['inlineStyle']),
        toggleInlineStyle: new tapable.SyncHook(['inlineStyle']),
        afterInlineStyleApplied: new tapable.SyncWaterfallHook(['newEditorState', 'editorState', 'inlineStyle']),
        createBlockRenderMap: new tapable.SyncBailHook(),
        createCustomStyleMap: new tapable.SyncBailHook(),
        blockStyleFn: new tapable.SyncBailHook(['block']),
        handleKeyCommand: new tapable.SyncBailHook(['command', 'editorState']),
        handleDroppedFiles: new tapable.SyncHook(['editorState', 'dropSelection', 'files']),
        addImage: new tapable.SyncHook(['editorState', 'file']),
        blockRendererFn: new tapable.SyncBailHook(['contentBlock', 'editorState']),
        createPlaceholder: new tapable.SyncHook(['editorState', 'placeholder']),
        didUpdate: new tapable.SyncHook(['editorState']),
        // 用来更新placeholder
        updatePlaceholder: new tapable.SyncHook(['editorState', 'placeholder']),
        // decorators
        compositeDecorator: new tapable.SyncWaterfallHook(['decorators']),
        keyBindingFn: new tapable.SyncBailHook(['e']),
        syncSelectionChange: new tapable.SyncHook(['editorState']),
        // 第一次打开editor时，因为prev值都是undefined这个时候可以认为是初始化时的变化
        selectionInitChange: new tapable.SyncHook(['editorState', 'payload']),
        selectionCollapsedChange: new tapable.SyncHook(['editorState', 'payload']),
        selectionFocusChange: new tapable.SyncHook(['editorState', 'payload']),
        selectionMoveInnerBlock: new tapable.SyncHook(['editorState', 'payload']),
        selectionMoveOuterBlock: new tapable.SyncHook(['editorState', 'payload']),
        selectionRangeChange: new tapable.SyncHook(['editorState', 'payload']),
        selectionRangeSizeChange: new tapable.SyncHook(['editorState', 'payload']),
        selectionRangeContentChange: new tapable.SyncHook(['editorState', 'payload']),
        inlineBarChange: new tapable.SyncHook(['editorState', 'visibility']),
        hideInlineToolbar: new tapable.SyncHook(),
        afterClickLinkButton: new tapable.SyncHook(['editorState']),
        cleanUpLinkClickState: new tapable.SyncHook(),
        toggleImageToolbarVisible: new tapable.SyncHook(['visibility', 'contentBlock']),
        updateDecorator: new tapable.SyncWaterfallHook(['pairs', 'editorState', 'context']),
        updateDragSubscription: new tapable.SyncHook(['diff']),
        syncBlockKeys: new tapable.SyncHook(['blockKeys', 'blockChanges']),
        // for drag
        prepareDragStart: new tapable.SyncHook(['sourceBlockKey']),
        teardownDragDrop: new tapable.SyncHook(),
        afterMounted: new tapable.SyncHook(),
        finalNewLine: new tapable.SyncBailHook(['editorState']),
        updateBlockDepthData: new tapable.SyncBailHook(['editorState'])
      };
      _this.editorRef = React.createRef();
      _this.inlineToolbarRef = React.createRef();
      _this.imageToolbarRef = React.createRef();
      _this.sidebarRef = React.createRef(); // const rawState = dndHelperData;
      // const newContentState = convertFromRaw(rawState);

      _this.state = {
        editorState: draftJs.EditorState.createEmpty()
      };
      /**
       * Call instance's apply method to initial plugins.
       */

      _this.plugins.forEach(function (plugin) {
        plugin.apply(_this.getEditor);
      }); // decorateComposer.getEditor = this.getEditor;


      _this.init();

      return _this;
    }

    var _proto = _class.prototype;

    _proto.setFinalState = function setFinalState(editorState, callback) {
      var _this2 = this;

      var nextState = this.hooks.finalNewLine.call(editorState);
      nextState = this.hooks.updateBlockDepthData.call(nextState);
      this.setState({
        editorState: nextState
      }, function () {
        var newState = _this2.state.editorState;

        if (typeof callback === 'function') {
          callback(newState);
        }
      });
    };

    _proto.componentDidMount = function componentDidMount() {
      var _this3 = this;

      this.hooks.afterMounted.call();
      this.hooks.onChange.tap('onChange', function (editorState) {
        _this3.setFinalState(editorState);
      });
      this.hooks.setState.tap('setState', function (editorState, callback) {
        _this3.setFinalState(editorState, callback);
      });
      this.hooks.updateDecorator.tap('updateDecorate', function (pairs, editorState) {
        if (pairs.length > 0) {
          var newDecorator = new draftJs.CompositeDecorator(pairs);

          _this3.setState({
            editorState: draftJs.EditorState.set(editorState, {
              // @ts-ignore
              decorator: new MultiDecorator([// @ts-ignore
              new PrismDecorator(), newDecorator])
            })
          });
        }
      });
      this.hooks.toggleWaterfallBlockType.tap('toggleWaterfallBlockType', function (newEditorState, editorState, blockType) {
        var nextEditorState = newEditorState || editorState;

        _this3.setState({
          editorState: draftJs.RichUtils.toggleBlockType(nextEditorState, blockType)
        });
      });
      this.hooks.toggleBlockType.tap('toggleBlockType', function (editorState, blockType) {
        _this3.setState({
          editorState: draftJs.RichUtils.toggleBlockType(editorState, blockType)
        });
      });
      this.hooks.toggleInlineStyleV2.tap('toggleInlineStyle', function (inlineStyle) {
        var editorState = _this3.state.editorState;

        _this3.setState({
          editorState: draftJs.RichUtils.toggleInlineStyle(editorState, inlineStyle)
        });
      });
      this.hooks.toggleInlineStyle.tap('toggleInlineStyle', function (inlineStyle) {
        var editorState = _this3.state.editorState;

        _this3.setState({
          editorState: draftJs.RichUtils.toggleInlineStyle(editorState, inlineStyle)
        }, function () {
          var editorState = _this3.state.editorState;

          _this3.hooks.afterInlineStyleApplied.call(editorState, editorState, inlineStyle);
        });
      });
      this.hooks.afterInlineStyleApplied.tap('afterInlineStyleApplied', function (newEditorState, editorState) {
        var nextEditorState = newEditorState || editorState;

        _this3.setState({
          editorState: nextEditorState
        });
      }); // 需要在下一个Event loop中触发setState才能够将`decorator`生效

      setTimeout(function () {
        var editorState = _this3.state.editorState;

        _this3.hooks.updateDecorator.call([], editorState);
      }, 0);
    };

    _proto.render = function render() {
      var _this$props = this.props,
          rest = _objectWithoutPropertiesLoose(_this$props, ["plugins"]);

      return React__default.createElement(Provider, {
        value: this.getEditor
      }, React__default.createElement(Editor, Object.assign({}, rest, {
        ref: this.editorRef,
        imageRef: this.imageToolbarRef,
        inlineRef: this.inlineToolbarRef,
        sidebarRef: this.sidebarRef,
        customStyleMap: this.customStyleMap,
        blockRenderMap: this.blockRenderMap,
        placeholder: ""
      })));
    };

    return _class;
  }(React.PureComponent);
};

var defaultPlugins = [
/*#__PURE__*/
// @ts-ignore
new BlockStyleFnPlugin(),
/*#__PURE__*/
// @ts-ignore
new SelectionChangePlugin(),
/*#__PURE__*/
// @ts-ignore
new CustomStyleMapPlugin(),
/*#__PURE__*/
// @ts-ignore
new BlockRenderMapPlugin(),
/*#__PURE__*/
// @ts-ignore
new HandleDroppedFilesPlugin(),
/*#__PURE__*/
// @ts-ignore
new AddImagePlugin(),
/*#__PURE__*/
// 对于keyCommand的一个兜底行为
// @ts-ignore
new DefaultHandleKeyCommandPlugin(),
/*#__PURE__*/
// @ts-ignore
new InlineToolbar(),
/*#__PURE__*/
// @ts-ignore
new LinkSpanDecoratorPlugin(),
/*#__PURE__*/
// @ts-ignore
new LinkDecorator(),
/*#__PURE__*/
// @ts-ignore
new SidebarPlugin(),
/*#__PURE__*/
// @ts-ignore
new StateFilterPlugin(),
/*#__PURE__*/
// @ts-ignore
new DNDPlugin(),
/*#__PURE__*/
// @ts-ignore
new FinalNewLinePlugin(),
/*#__PURE__*/
// @ts-ignore
new UpdateBlockDepthData()];
var ModernEditor = /*#__PURE__*/createEditor(defaultPlugins);

exports.default = ModernEditor;
//# sourceMappingURL=react-tapable-editor.cjs.development.js.map
